@@ -18,7 +18,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 #
-# $Id: autogen.sh,v 1.16 2005/05/03 20:10:56 mroi Exp $
+# $Id: autogen.sh,v 1.19 2007/04/15 20:06:50 dsalt Exp $
 #
 # Maintained by Stephen Torri <storri@users.sourceforge.net>
 #
@@ -27,10 +27,8 @@
 PROG=xine-ui
 
 # Minimum value required to build
-WANT_AUTOMAKE_1_6=1 export WANT_AUTOMAKE_1_6
-AUTOMAKE_MIN=1.6.0
+AUTOMAKE_MIN=1.9.0
 AUTOCONF_MIN=2.53
-LIBTOOL_MIN=1.4.0
 
 # Check how echo works in this /bin/sh
 case `echo -n` in
@@ -50,6 +48,17 @@ detect_configure_ac() {
   }
 }
 
+parse_version_no() {
+  # version no. is extended/truncated to three parts; only digits are handled
+  perl -e 'my $v = <>;
+	   chomp $v;
+	   my @v = split (" ", $v);
+	   $v = $v[$#v];
+	   $v =~ s/[^0-9.].*$//;
+	   @v = split (/\./, $v);
+	   push @v, 0 while $#v < 2;
+	   print $v[0] * 10000 + $v[1] * 100 + $v[2], "\n"'
+}
 
 #--------------------
 # AUTOCONF
@@ -60,12 +69,8 @@ detect_autoconf() {
   NUM_RESULT=$#
   RESULT_FILE=$3
   if [ $RETVAL -eq 0 -a $NUM_RESULT -eq 3 -a -f "$RESULT_FILE" ]; then
-    AC="`autoconf --version | sed -n 1p | sed -e 's/[a-zA-Z\ \.\(\)\-]//g'`"
-    AUTOCONF_MIN="`echo $AUTOCONF_MIN | sed -e 's/[a-zA-Z\ \.\(\)\-]//g'`"
-    if test $AC -lt 100 ; then
-      AC=`expr $AC \* 10`
-    fi
-    if [ `expr $AC` -ge $AUTOCONF_MIN ]; then
+    AC="`autoconf --version | parse_version_no`"
+    if [ `expr $AC` -ge "`echo $AUTOCONF_MIN | parse_version_no`" ]; then
       autoconf_ok=yes
     fi
   else
@@ -103,55 +108,6 @@ run_autoconf () {
   echo "done."
 }
 
-#--------------------
-# LIBTOOL
-#-------------------
-try_libtool_executable() {
-  libtool=$1
-  set -- `type $libtool 2>/dev/null`
-  RETVAL=$?
-  NUM_RESULT=$#
-  RESULT_FILE=$3
-  if [ $RETVAL -eq 0 -a $NUM_RESULT -eq 3 -a -f "$RESULT_FILE" ]; then
-    LT="`$libtool --version | awk '{ print $4 }' | sed -e 's/[a-zA-Z\ \.\(\)\-\;]//g'`"
-    LIBTOOL_MIN="`echo $LIBTOOL_MIN | sed -e 's/[a-zA-Z\ \.\(\)\-]//g'`"
-    if test $LT -lt 100 ; then
-      LT=`expr $LT \* 10`
-    fi
-    if [ `expr $LT` -ge $LIBTOOL_MIN ]; then
-      libtool_ok=yes
-    fi
-  fi
-}
-
-detect_libtool() {
-  # try glibtool first, then libtool
-  try_libtool_executable 'glibtool'
-  if [ "x$libtool_ok" != "xyes" ]; then
-    try_libtool_executable 'libtool'
-    if [ "x$libtool_ok" != "xyes" ]; then
-      echo
-      echo "**Error**: You must have \`libtool' >= $LIBTOOL_MIN installed to" 
-      echo "           compile $PROG. Download the appropriate package"
-      echo "           for your distribution or source from ftp.gnu.org."
-      exit 1
-    fi
-  fi
-}
-
-run_libtoolize() {
-  if test x"$libtool_ok" != x"yes"; then
-    echo
-    echo "**Warning**: Version of libtool is less than $LIBTOOL_MIN."
-    echo "             Some warning message might occur from libtool"
-    echo
-  fi
-
-  echo $_echo_n " + Running libtoolize: $_echo_c";
-    "${libtool}ize" --force --copy >/dev/null 2>&1;
-  echo "done."
-}
-
 #--------------------
 # AUTOMAKE
 #--------------------
@@ -165,12 +121,8 @@ detect_automake() {
   NUM_RESULT=$#
   RESULT_FILE=$3
   if [ $RETVAL -eq 0 -a $NUM_RESULT -eq 3 -a -f "$RESULT_FILE" ]; then
-    AM="`automake --version | sed -n 1p | sed -e 's/[a-zA-Z\ \.\(\)\-]//g'`"
-    AUTOMAKE_MIN="`echo $AUTOMAKE_MIN | sed -e 's/[a-zA-Z\ \.\(\)\-]//g'`"
-    if test $AM -lt 100 ; then
-      AM=`expr $AM \* 10`
-    fi
-    if [ `expr $AM` -ge $AUTOMAKE_MIN ]; then
+    AM="`automake --version | parse_version_no`"
+    if [ `expr $AM` -ge "`echo $AUTOMAKE_MIN | parse_version_no`" ]; then
       automake_ok=yes
     fi
   else
@@ -207,12 +159,8 @@ detect_aclocal() {
   NUM_RESULT=$#
   RESULT_FILE=$3
   if [ $RETVAL -eq 0 -a $NUM_RESULT -eq 3 -a -f "$RESULT_FILE" ]; then
-    AC="`aclocal --version | sed -n 1p | sed -e 's/[a-zA-Z\ \.\(\)\-]//g'`"
-    ACLOCAL_MIN="`echo $AUTOMAKE_MIN | sed -e 's/[a-zA-Z\ \.\(\)\-]//g'`"
-    if test $AC -lt 100 ; then
-      AC=`expr $AC \* 10`
-    fi
-    if [ `expr $AC` -ge $ACLOCAL_MIN ]; then
+    AC="`aclocal --version | parse_version_no`"
+    if [ `expr $AC` -ge "`echo $AUTOMAKE_MIN | parse_version_no`" ]; then
       aclocal_ok=yes
     fi
   else
@@ -259,7 +207,6 @@ run_configure () {
 #---------------
 detect_configure_ac
 detect_autoconf
-detect_libtool
 detect_automake
 detect_aclocal
 
@@ -281,19 +228,14 @@ case "$1" in
     run_aclocal
     run_autoconf
     ;;
-  libtoolize)
-    run_libtoolize
-    ;;
   noconfig)
     run_aclocal
-    run_libtoolize
     run_autoheader
     run_automake
     run_autoconf
     ;;
   *)
     run_aclocal
-    run_libtoolize
     run_autoheader
     run_automake
     run_autoconf
