@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: main.c,v 1.56 2004/02/28 20:06:37 valtri Exp $
+ * $Id: main.c,v 1.60 2007/01/18 19:56:56 siretart Exp $
  *
  * xine main for aalib and caca
  *
@@ -64,6 +64,7 @@
 
 #ifdef CACA
 #  include <caca.h>
+#  include <cucul.h>
 #endif
 
 /* Sound mixer capabilities */
@@ -87,6 +88,10 @@ typedef struct {
   char                *configfile;
 #ifdef AA
   aa_context          *context;
+#endif
+#ifdef CACA
+  cucul_canvas_t      *canvas;
+  caca_display_t      *display;
 #endif
   char                *mrl[1024];
   int                  num_mrls;
@@ -436,7 +441,7 @@ static int aaxine_xine_play(xine_stream_t *stream, int start_pos, int start_time
   return ret;
 }
 
-int aaxine_xine_open_and_play(char *mrl, int start_pos, int start_time) {
+static int aaxine_xine_open_and_play(char *mrl, int start_pos, int start_time) {
   
 
   if(!xine_open(aaxine.stream, (const char *)mrl)) {
@@ -569,7 +574,7 @@ static void xinit_thread(void) {
      printf("dlopen() failed: %s\n", dlerror());
 }
 
-int aaxine_get_key_event() {
+static int aaxine_get_key_event(void) {
 #ifdef AA
   int key;
 
@@ -583,14 +588,12 @@ int aaxine_get_key_event() {
 #endif
 
 #ifdef CACA
-  unsigned int key;
+  caca_event_t ev;
 
-  key = CACA_EVENT_NONE;
-  while (((key = caca_get_event(CACA_EVENT_KEY_PRESS)) == CACA_EVENT_NONE) && aaxine.running)
-    usleep(50000);
+  while ( !caca_get_event(aaxine.display, CACA_EVENT_KEY_PRESS, &ev, 50000) && aaxine.running ) ;
 
   if (!aaxine.running) return 0;
-  return key & ~CACA_EVENT_ANY;
+  return ev.data.key.ch;
 #endif
 }
 
@@ -628,6 +631,10 @@ int main(int argc, char *argv[]) {
   aaxine.post_running = 0;
   aaxine.post_plugin_name = NULL;
 
+#ifdef CACA
+  aaxine.display = NULL;
+#endif
+
 #ifdef AA
   /* 
    * AALib help and option-parsing
@@ -804,6 +811,8 @@ int main(int argc, char *argv[]) {
 					  video_driver_id,
 					  XINE_VISUAL_TYPE_CACA,
 					  NULL);
+  aaxine.canvas = cucul_create_canvas(0, 0);
+  aaxine.display = caca_create_display(aaxine.canvas);
 #endif
 
   if (!aaxine.vo_port) {
@@ -872,9 +881,9 @@ int main(int argc, char *argv[]) {
 	else {
 	  /* reduce goom default values */
 	  if(!strcmp(aaxine.post_plugin_name, "goom")) {
-	    config_update_num("post.goom_fps", 10);
-	    config_update_num("post.goom_height", 120);
-	    config_update_num("post.goom_width", 120);
+	    config_update_num("effects.goom.fps", 10);
+	    config_update_num("effects.goom.height", 120);
+	    config_update_num("effects.goom.width", 120);
 	  }
 	}
       }
@@ -1108,6 +1117,13 @@ int main(int argc, char *argv[]) {
     aa_close(aaxine.context);
   }
 #endif
+
+#ifdef CACA
+  if(aaxine.display) {
+    caca_free_display(aaxine.display); 
+    cucul_free_canvas(aaxine.canvas);
+  }
+#endif
   
   if(xlib_handle)
     dlclose(xlib_handle);
