@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2006 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,13 +17,28 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: libcommon.h,v 1.1 2003/02/02 16:47:58 f1rmb Exp $
+ * $Id: libcommon.h,v 1.5 2006/06/19 00:24:16 komadori Exp $
  *
  */
 #ifndef _LIBCOMMON__H
 #define _LIBCOMMON__H
 
 #include "utils.h"
-#include "xscreensaver-remote.h"
+
+#ifndef HAVE_GETLINE
+ssize_t getline(char *lineptr, size_t *n, FILE *f);
+#endif
+
+#ifndef HAVE_GETSUBOPT
+int getsubopt(char **optionp, char * const *keylistp, char **valuep);
+#endif
+
+#ifndef HAVE_STRNDUP
+char *strndup(const char *s, size_t n);
+#endif
+
+#ifndef HAVE_STRSEP
+char *strsep(char **stringp, const char *delim);
+#endif
 
 #endif
