@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2007 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -22,6 +22,7 @@
 #ifndef ACTIONS_H
 #define ACTIONS_H
 
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -122,6 +123,7 @@ typedef enum {
   ACTID_ADDMEDIAMARK,
   ACTID_SKINDOWNLOAD,
   ACTID_OSD_SINFOS,
+  ACTID_OSD_WTEXT,
   ACTID_FILESELECTOR,
   ACTID_SUBSELECT,
 
@@ -154,47 +156,65 @@ typedef enum {
   ACTID_EVENT_NUMBER_7      = XINE_EVENT_INPUT_NUMBER_7       | ACTID_IS_INPUT_EVENT,
   ACTID_EVENT_NUMBER_8      = XINE_EVENT_INPUT_NUMBER_8       | ACTID_IS_INPUT_EVENT,
   ACTID_EVENT_NUMBER_9      = XINE_EVENT_INPUT_NUMBER_9       | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_NUMBER_10_ADD = XINE_EVENT_INPUT_NUMBER_10_ADD  | ACTID_IS_INPUT_EVENT
+  ACTID_EVENT_NUMBER_10_ADD = XINE_EVENT_INPUT_NUMBER_10_ADD  | ACTID_IS_INPUT_EVENT,
 #ifdef ENABLE_VDR_KEYS
-,
-  ACTID_EVENT_VDR_RED           = XINE_EVENT_VDR_RED              | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_GREEN         = XINE_EVENT_VDR_GREEN            | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_YELLOW        = XINE_EVENT_VDR_YELLOW           | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_BLUE          = XINE_EVENT_VDR_BLUE             | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_PLAY          = XINE_EVENT_VDR_PLAY             | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_PAUSE         = XINE_EVENT_VDR_PAUSE            | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_STOP          = XINE_EVENT_VDR_STOP             | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_RECORD        = XINE_EVENT_VDR_RECORD           | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_FASTFWD       = XINE_EVENT_VDR_FASTFWD          | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_FASTREW       = XINE_EVENT_VDR_FASTREW          | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_POWER         = XINE_EVENT_VDR_POWER            | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_CHANNELPLUS   = XINE_EVENT_VDR_CHANNELPLUS      | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_CHANNELMINUS  = XINE_EVENT_VDR_CHANNELMINUS     | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_SCHEDULE      = XINE_EVENT_VDR_SCHEDULE         | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_CHANNELS      = XINE_EVENT_VDR_CHANNELS         | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_TIMERS        = XINE_EVENT_VDR_TIMERS           | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_RECORDINGS    = XINE_EVENT_VDR_RECORDINGS       | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_SETUP         = XINE_EVENT_VDR_SETUP            | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_COMMANDS      = XINE_EVENT_VDR_COMMANDS         | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_BACK          = XINE_EVENT_VDR_BACK             | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_USER1         = XINE_EVENT_VDR_USER1            | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_USER2         = XINE_EVENT_VDR_USER2            | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_USER3         = XINE_EVENT_VDR_USER3            | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_USER4         = XINE_EVENT_VDR_USER4            | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_USER5         = XINE_EVENT_VDR_USER5            | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_USER6         = XINE_EVENT_VDR_USER6            | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_USER7         = XINE_EVENT_VDR_USER7            | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_USER8         = XINE_EVENT_VDR_USER8            | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_USER9         = XINE_EVENT_VDR_USER9            | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_VOLPLUS       = XINE_EVENT_VDR_VOLPLUS          | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_VOLMINUS      = XINE_EVENT_VDR_VOLMINUS         | ACTID_IS_INPUT_EVENT,
-  ACTID_EVENT_VDR_MUTE          = XINE_EVENT_VDR_MUTE             | ACTID_IS_INPUT_EVENT
+  ACTID_EVENT_VDR_RED              = XINE_EVENT_VDR_RED              | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_GREEN            = XINE_EVENT_VDR_GREEN            | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_YELLOW           = XINE_EVENT_VDR_YELLOW           | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_BLUE             = XINE_EVENT_VDR_BLUE             | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_PLAY             = XINE_EVENT_VDR_PLAY             | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_PAUSE            = XINE_EVENT_VDR_PAUSE            | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_STOP             = XINE_EVENT_VDR_STOP             | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_RECORD           = XINE_EVENT_VDR_RECORD           | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_FASTFWD          = XINE_EVENT_VDR_FASTFWD          | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_FASTREW          = XINE_EVENT_VDR_FASTREW          | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_POWER            = XINE_EVENT_VDR_POWER            | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_CHANNELPLUS      = XINE_EVENT_VDR_CHANNELPLUS      | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_CHANNELMINUS     = XINE_EVENT_VDR_CHANNELMINUS     | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_SCHEDULE         = XINE_EVENT_VDR_SCHEDULE         | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_CHANNELS         = XINE_EVENT_VDR_CHANNELS         | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_TIMERS           = XINE_EVENT_VDR_TIMERS           | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_RECORDINGS       = XINE_EVENT_VDR_RECORDINGS       | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_SETUP            = XINE_EVENT_VDR_SETUP            | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_COMMANDS         = XINE_EVENT_VDR_COMMANDS         | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_BACK             = XINE_EVENT_VDR_BACK             | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_USER1            = XINE_EVENT_VDR_USER1            | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_USER2            = XINE_EVENT_VDR_USER2            | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_USER3            = XINE_EVENT_VDR_USER3            | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_USER4            = XINE_EVENT_VDR_USER4            | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_USER5            = XINE_EVENT_VDR_USER5            | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_USER6            = XINE_EVENT_VDR_USER6            | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_USER7            = XINE_EVENT_VDR_USER7            | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_USER8            = XINE_EVENT_VDR_USER8            | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_USER9            = XINE_EVENT_VDR_USER9            | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_VOLPLUS          = XINE_EVENT_VDR_VOLPLUS          | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_VOLMINUS         = XINE_EVENT_VDR_VOLMINUS         | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_MUTE             = XINE_EVENT_VDR_MUTE             | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_AUDIO            = XINE_EVENT_VDR_AUDIO            | ACTID_IS_INPUT_EVENT,
+  ACTID_EVENT_VDR_INFO             = XINE_EVENT_VDR_INFO             | ACTID_IS_INPUT_EVENT,
+#ifdef XINE_EVENT_VDR_CHANNELPREVIOUS /* #ifdef is precaution for backward compatibility at the moment */
+  ACTID_EVENT_VDR_CHANNELPREVIOUS  = XINE_EVENT_VDR_CHANNELPREVIOUS  | ACTID_IS_INPUT_EVENT,
+#endif
 #endif
 
   /*
    * End of numeric mapping.
    */
 } action_id_t;
+ 
+typedef struct {
+
+  struct {
+    int set;
+    int arg;
+  } numeric;
+
+  struct {
+    int   set;
+    char *arg;
+  } alphanum;
+
+} parameter_t;
 
 int default_command_action(char *action_str);
 void do_action(int action);
