@@ -202,6 +202,8 @@ static int init_stream(void)
 
 	fbxine.stream = xine_stream_new(fbxine.xine, fbxine.audio_port,
 					fbxine.video_port);
+	xine_set_param(fbxine.stream, XINE_PARAM_VERBOSITY, fbxine.verbosity);
+
 	fbxine.event_queue = xine_event_new_queue(fbxine.stream);  
   
 	xine_event_create_listener_thread(fbxine.event_queue, 
@@ -250,14 +252,14 @@ static int fbxine_init(int argc, char **argv)
 {
 	if(!check_version())
 		return 0;
+	if(!init_xine())
+		return 0;
 	switch(parse_options(argc, argv))
 	{
 		case 0:
 		case -1:
 			return 0;
 	}
-	if(!init_xine())
-		return 0;
 
 	if (stdctl)
 	        fbxine_init_stdctl();
@@ -303,7 +305,7 @@ static int fbxine_init(int argc, char **argv)
 
 static void install_abort(void)
 {
-	static int trapped[] = {
+	static const int trapped[] = {
 		SIGINT, SIGQUIT, SIGILL,
 		SIGFPE, SIGKILL, SIGBUS,
 		SIGSEGV, SIGSYS, SIGPIPE,
@@ -312,7 +314,7 @@ static void install_abort(void)
 		SIGSTKFLT,
 #endif
 	};
-	int i;
+	size_t i;
 	
 	for(i = 0; i < sizeof(trapped)/sizeof(int); i++)
 		signal(trapped[i], (void(*)(int))fbxine_do_abort);
