@@ -96,13 +96,7 @@ static void print_usage(void)
 		       *driver_id++);
 	
 	printf("\n"
-	       "  -a, --audio-channel <number>   Select audio channel.\n"
-	       "  -R, --recognize-by [option]    Try to recognize stream type. Option are:\n"
-	       "                                 'default': by content, then by extension,\n"
-	       "                                 'revert': by extension, then by content,\n"
-	       "                                 'content': only by content,\n"
-	       "                                 'extension': only by extension.\n"
-	       "                                 -if no option is given, 'revert' is selected\n");
+	       "  -a, --audio-channel <number>   Select audio channel.\n");
 #ifdef HAVE_LIRC
 	printf("  -L, --no-lirc                  Turn off LIRC support.\n");
 #endif
@@ -139,14 +133,20 @@ int parse_options(int argc, char **argv)
 			{ "audio-driver",  required_argument, 0, 'A' },
 			{ "video-driver",  required_argument, 0, 'V' },
 			{ "version",       no_argument,       0, 'v' },
+#ifdef HAVE_LIRC
 			{ "no-lirc",       no_argument,       0, 'L' },
+#endif
 			{ "stdctl",        optional_argument, 0, OPTION_STDCTL },
 			{ "post",          required_argument, 0, OPTION_POST },
 			{ "deinterlace",   no_argument,       0, 'D' },
 			{ "verbose",       optional_argument, 0, OPTION_VERBOSE },
 			{ 0,               no_argument,       0,  0  }
 		};
-	const char *short_options = "?hda:qA:V:R::vD";
+	const char *short_options = "?hda:qA:V:"
+#ifdef HAVE_LIRC
+		"L"
+#endif
+		"vD";
 	int c = '?', option_index = 0;
   
 	opterr = 0;
