@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: osd.c,v 1.4 2003/12/01 17:23:27 f1rmb Exp $
+ * $Id: osd.c,v 1.10 2006/07/15 20:26:20 dgp85 Exp $
  *
  */
 
@@ -34,7 +34,7 @@
 #include "main.h"
 #include "osd.h"
 #include "actions.h"
-
+#include "utils.h"
 
 #define OVL_PALETTE_SIZE 256
 
@@ -273,7 +273,10 @@ void osd_display_info(char *info, ...) {
 
     xine_osd_draw_text(fbxine.osd.info, 0, 0, buf, XINE_OSD_TEXT1);
     xine_osd_set_position(fbxine.osd.info, 20, 10 + 30);
-    xine_osd_show(fbxine.osd.info, 0);
+    if (xine_osd_get_capabilities(fbxine.osd.info) & XINE_OSD_CAP_UNSCALED)
+      xine_osd_show_unscaled(fbxine.osd.info, 0);
+    else
+      xine_osd_show(fbxine.osd.info, 0); 
     fbxine.osd.info_visible = fbxine.osd.timeout;
     SAFE_FREE(buf);
   }
@@ -305,7 +308,7 @@ void osd_update_status(void) {
     switch(status) {
     case XINE_STATUS_IDLE:
     case XINE_STATUS_STOP:
-      sprintf(buffer, "%s", (_osd_get_status_sym(status)));
+      strcpy(buffer, (_osd_get_status_sym(status)));
       break;
       
     case XINE_STATUS_PLAY:
@@ -320,7 +323,7 @@ void osd_update_status(void) {
 		  secs / (60*60), (secs / 60) % 60, secs % 60);
 	}
 	else
-	  sprintf(buffer, "%s", (_osd_get_speed_sym(speed)));
+	  strcpy(buffer, (_osd_get_speed_sym(speed)));
       }
       break;
       
@@ -331,7 +334,10 @@ void osd_update_status(void) {
     
     xine_osd_draw_text(fbxine.osd.status, 0, 0, buffer, XINE_OSD_TEXT1);
     xine_osd_set_position(fbxine.osd.status, 20, 10);
-    xine_osd_show(fbxine.osd.status, 0);
+    if (xine_osd_get_capabilities(fbxine.osd.status) & XINE_OSD_CAP_UNSCALED)
+      xine_osd_show_unscaled(fbxine.osd.status, 0);
+    else
+      xine_osd_show(fbxine.osd.status, 0);
     fbxine.osd.status_visible = fbxine.osd.timeout;
   }
 }
@@ -339,6 +345,7 @@ void osd_update_status(void) {
 void osd_stream_infos(void) {
 
   if(fbxine.osd.enabled) {
+    uint32_t    width;
     uint32_t    vwidth, vheight, asrate;
     const char *vcodec, *acodec;
     char        buffer[256], *p;
@@ -386,40 +393,38 @@ void osd_stream_infos(void) {
       y += h;
     }
     
-    memset(&buffer, 0, sizeof(buffer));
-
-    sprintf(buffer, "Audio: ");
+    strncpy(buffer, "Audio: ", sizeof(buffer)-1);
     len = strlen(buffer);
     switch(audiochannel) {
     case -2:
-      sprintf(buffer, "%soff", buffer);
+      strlcat(buffer, "off", sizeof(buffer));
       break;
     case -1:
       if(!xine_get_audio_lang (fbxine.stream, audiochannel, &buffer[len]))
-	sprintf(buffer, "%sauto", buffer);
+	strlcat(buffer, "auto", sizeof(buffer));
       break;
     default:
       if(!xine_get_audio_lang (fbxine.stream, audiochannel, &buffer[len]))
-	sprintf(buffer, "%s%3d", buffer, audiochannel);
+	snprintf(buffer+strlen(buffer), sizeof(buffer)-strlen(buffer), "%3d", audiochannel);
       break;
     }
 
-    sprintf(buffer, "%s, Spu: ", buffer);
+    strlcat(buffer, ", Spu: ", sizeof(buffer));
     len = strlen(buffer);
     switch (spuchannel) {
     case -2:
-      sprintf(buffer, "%soff", buffer);
+      strlcat(buffer, "off", sizeof(buffer));
       break;
     case -1:
       if(!xine_get_spu_lang (fbxine.stream, spuchannel, &buffer[len]))
-	sprintf(buffer, "%sauto", buffer);
+	strlcat(buffer, "auto", sizeof(buffer));
       break;
     default:
       if(!xine_get_spu_lang (fbxine.stream, spuchannel, &buffer[len]))
-        sprintf(buffer, "%s%3d", buffer, spuchannel);
+        sprintf(buffer+strlen(buffer), "%3d", spuchannel);
       break;
     }
-    sprintf(buffer, "%s.", buffer);
+    strlcat(buffer, ".", sizeof(buffer));
     xine_osd_draw_text(fbxine.osd.sinfo, x, y, buffer, XINE_OSD_TEXT1);
     xine_osd_get_text_size(fbxine.osd.sinfo, buffer, &w, &h);
     if(w > osdw)
@@ -439,11 +444,20 @@ void osd_stream_infos(void) {
 
       osd_stream_position();
     }
-    
-    x = (vwidth - osdw) - 40;
-    xine_osd_set_position(fbxine.osd.sinfo, (x >= 0) ? x : 0, 15);
 
-    xine_osd_show(fbxine.osd.sinfo, 0);
+#ifdef XINE_PARAM_VO_WINDOW_WIDTH
+    if (xine_osd_get_capabilities(fbxine.osd.sinfo) & XINE_OSD_CAP_UNSCALED)
+      width = xine_get_param(fbxine.stream, XINE_PARAM_VO_WINDOW_WIDTH);
+    else
+#endif
+      width = vwidth;
+    
+    x = (width - osdw) - 40;
+    xine_osd_set_position(fbxine.osd.sinfo, (x >= 0) ? x : 0, 15);
+    if (xine_osd_get_capabilities(fbxine.osd.sinfo) & XINE_OSD_CAP_UNSCALED)
+      xine_osd_show_unscaled(fbxine.osd.sinfo, 0);
+    else
+      xine_osd_show(fbxine.osd.sinfo, 0);
     fbxine.osd.sinfo_visible = fbxine.osd.timeout;
   }
 }
@@ -451,7 +465,7 @@ void osd_stream_infos(void) {
 void osd_draw_bar(char *title, int min, int max, int val, int type) {
 
   if(fbxine.osd.enabled) {
-    uint32_t vwidth, vheight;
+    uint32_t width, height;
     int      bar_color[40];
     int      i, x;
     float    _val = (int) val;
@@ -540,23 +554,38 @@ void osd_draw_bar(char *title, int min, int max, int val, int type) {
       xine_osd_get_text_size(fbxine.osd.bar[1], title, &tw, &th);
       xine_osd_draw_text(fbxine.osd.bar[1], (BAR_WIDTH - tw) >> 1, 0, title, XINE_OSD_TEXT1);
     }
+
+#ifdef XINE_PARAM_VO_WINDOW_WIDTH
+    if (xine_osd_get_capabilities(fbxine.osd.bar[0]) & XINE_OSD_CAP_UNSCALED) {
+      width  = xine_get_param(fbxine.stream, XINE_PARAM_VO_WINDOW_WIDTH);
+      height = xine_get_param(fbxine.stream, XINE_PARAM_VO_WINDOW_HEIGHT);
+    } else
+#endif
+    {
+      width  = xine_get_stream_info(fbxine.stream, XINE_STREAM_INFO_VIDEO_WIDTH);
+      height = xine_get_stream_info(fbxine.stream, XINE_STREAM_INFO_VIDEO_HEIGHT);
+    }
     
-    vwidth  = xine_get_stream_info(fbxine.stream, XINE_STREAM_INFO_VIDEO_WIDTH);
-    vheight = xine_get_stream_info(fbxine.stream, XINE_STREAM_INFO_VIDEO_HEIGHT);
+    x = (width - BAR_WIDTH) >> 1;
+    xine_osd_set_position(fbxine.osd.bar[0], (x >= 0) ? x : 0, (height - BAR_HEIGHT) - 40);
+    xine_osd_set_position(fbxine.osd.bar[1], (x >= 0) ? x : 0, (height - (BAR_HEIGHT * 2)) - 40);
     
-    x = (vwidth - BAR_WIDTH) >> 1;
-    xine_osd_set_position(fbxine.osd.bar[0], (x >= 0) ? x : 0, (vheight - BAR_HEIGHT) - 40);
-    xine_osd_set_position(fbxine.osd.bar[1], (x >= 0) ? x : 0, (vheight - (BAR_HEIGHT * 2)) - 40);
-    
-    xine_osd_show(fbxine.osd.bar[0], 0);
-    if(title)
-      xine_osd_show(fbxine.osd.bar[1], 0);
+    if (xine_osd_get_capabilities(fbxine.osd.bar[0]) & XINE_OSD_CAP_UNSCALED) {
+      xine_osd_show_unscaled(fbxine.osd.bar[0], 0);
+      if (title)
+        xine_osd_show_unscaled(fbxine.osd.bar[1], 0);
+    }
+    else {
+      xine_osd_show(fbxine.osd.bar[0], 0);
+      if (title)
+        xine_osd_show(fbxine.osd.bar[1], 0);
+    }
     
     fbxine.osd.bar_visible = fbxine.osd.timeout;
   }
 }
 
-void osd_stream_position() {
+void osd_stream_position(void) {
   int pos;
   if(get_pos_length(fbxine.stream, &pos, NULL, NULL)) {
       osd_draw_bar("Position in Stream", 0, 65535, pos, OSD_BAR_POS2);
@@ -589,7 +618,7 @@ void osd_display_spu_lang(void) {
   }
   
   sprintf(buffer, "Subtitles: %s", lang);
-  osd_display_info(buffer);
+  osd_display_info("%s", buffer);
 }
 
 void osd_display_audio_lang(void) {
@@ -618,5 +647,14 @@ void osd_display_audio_lang(void) {
   }
 
   sprintf(buffer, "Audio Channel: %s", lang);
-  osd_display_info(buffer);
+  osd_display_info("%s", buffer);
+}
+
+void osd_display_zoom(void) {
+
+  /* show some average of x/y zoom (in case aspect ratio was changed) */
+  osd_draw_bar("Zoom Setting", 0, 200,
+	       (xine_get_param(fbxine.stream, XINE_PARAM_VO_ZOOM_X) +
+		xine_get_param(fbxine.stream, XINE_PARAM_VO_ZOOM_Y)) / 2,
+	       OSD_BAR_POS2);
 }
