@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: osd.h,v 1.1 2003/07/28 19:58:29 f1rmb Exp $
+ * $Id: osd.h,v 1.4 2006/04/20 19:02:13 dsalt Exp $
  *
  */
 
@@ -35,9 +35,11 @@ void osd_init(void);
 void osd_deinit(void);
 void osd_display_spu_lang(void);
 void osd_display_audio_lang(void);
+void osd_display_zoom(void);
 void osd_update_status(void);
 void osd_stream_position(void);
-void osd_display_info(char *info, ...);
+void osd_display_info(char *info, ...) __attribute__ ((format (printf, 1, 2)));
   /* see OSD_BAR_* */
+void osd_stream_infos(void);
 void osd_draw_bar(char *title, int min, int max, int val, int type);
 #endif
