@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2007 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: actions.c,v 1.261 2005/07/24 02:40:36 fsck-p Exp $
+ * $Id: actions.c,v 1.274 2007/04/07 00:52:54 fsck-p Exp $
  *
  * implementation of all the various actions for the gui (play, stop, open, pause...)
  *
@@ -48,6 +48,8 @@ filebrowser_t          *load_stream = NULL, *load_sub = NULL;
 
 static pthread_mutex_t new_pos_mutex =  PTHREAD_MUTEX_INITIALIZER;
 
+static int             last_playback_speed = XINE_SPEED_NORMAL;
+
 void reparent_all_windows(void) {
   int                    i;
   typedef struct {
@@ -176,9 +178,24 @@ void try_to_set_input_focus(Window window) {
   wait_for_window_visible(gGui->display, window);
   
   if(xitk_is_window_visible(gGui->display, window)) {
-    XLockDisplay (gGui->display);
-    XSetInputFocus(gGui->display, window, RevertToParent, CurrentTime);
-    XUnlockDisplay (gGui->display);
+    int    retry = 0;
+    Window focused_win;
+
+    do {
+      int revert;
+
+      XLockDisplay (gGui->display);
+      XSetInputFocus(gGui->display, window, RevertToParent, CurrentTime);
+      XSync(gGui->display, False);
+      XUnlockDisplay (gGui->display);
+
+      /* Retry until the WM was mercyful to give us the focus (but not indefinitely) */
+      xine_usec_sleep(5000);
+      XLockDisplay (gGui->display);
+      XGetInputFocus(gGui->display, &focused_win, &revert);
+      XUnlockDisplay (gGui->display);
+
+    } while((focused_win != window) && (retry++ < 30));
   }
 }
 
@@ -363,14 +380,12 @@ int gui_xine_play(xine_stream_t *stream, int start_pos, int start_time_in_secs,
      || (a_unhandled = (has_audio && (!audio_handled)))) {
     char      buffer[4096];
     
-    memset(&buffer, 0, sizeof(buffer));
-
     if(v_unhandled && a_unhandled) {
       snprintf(buffer, sizeof(buffer), _("The stream '%s' isn't supported by xine:\n\n"),
 	       (stream == gGui->stream) ? gGui->mmk.mrl : gGui->visual_anim.mrls[gGui->visual_anim.current]);
     }
     else {
-      snprintf(buffer, sizeof(buffer), _("The stream '%s' use an unsupported codec:\n\n"),
+      snprintf(buffer, sizeof(buffer), _("The stream '%s' uses an unsupported codec:\n\n"),
 	       (stream == gGui->stream) ? gGui->mmk.mrl : gGui->visual_anim.mrls[gGui->visual_anim.current]);
     }
     
@@ -380,7 +395,7 @@ int gui_xine_play(xine_stream_t *stream, int start_pos, int start_time_in_secs,
       
       minfo = xine_get_meta_info(stream, XINE_META_INFO_VIDEOCODEC);
       vfcc = xine_get_stream_info(stream, XINE_STREAM_INFO_VIDEO_FOURCC);
-      sprintf(buffer, _("%sVideo Codec: %s (%s)\n"), buffer,
+      sprintf(buffer+strlen(buffer), _("%sVideo Codec: %s (%s)\n"), "",
 	      (minfo && strlen(minfo)) ? (char *) minfo : _("Unavailable"), 
 	      (get_fourcc_string(vfcc)));
     }
@@ -391,21 +406,21 @@ int gui_xine_play(xine_stream_t *stream, int start_pos, int start_time_in_secs,
       
       minfo = xine_get_meta_info(stream, XINE_META_INFO_AUDIOCODEC);
       afcc = xine_get_stream_info(stream, XINE_STREAM_INFO_AUDIO_FOURCC);
-      sprintf(buffer,  _("%sAudio Codec: %s (%s)\n"), buffer,
+      sprintf(buffer+strlen(buffer),  _("%sAudio Codec: %s (%s)\n"), "",
 	      (minfo && strlen(minfo)) ? (char *) minfo : _("Unavailable"), 
 	      (get_fourcc_string(afcc)));
     }
     
 
     if(v_unhandled && a_unhandled) {
-      xine_error(buffer);
+      xine_error("%s", buffer);
       return 0;
     }
 
     if(!gGui->play_anyway) {
       xitk_window_t *xw;
 
-      sprintf(buffer, _("%s\nStart playback anyway ?\n"), buffer);
+      sprintf(buffer+strlen(buffer), _("%s\nStart playback anyway ?\n"), "");
       
       play_data.stream             = stream;
       play_data.start_pos          = start_pos;
@@ -416,7 +431,7 @@ int gui_xine_play(xine_stream_t *stream, int start_pos, int start_time_in_secs,
       xw = xitk_window_dialog_yesno_with_width(gGui->imlib_data, _("Start Playback ?"), 
 					       start_anyway_yesno, start_anyway_yesno, 
 					       NULL, 400, ALIGN_CENTER,
-					       buffer);
+					       "%s", buffer);
       XLockDisplay(gGui->display);
       if(!gGui->use_root_window && gGui->video_display == gGui->display)
 	XSetTransientForHint(gGui->display, xitk_window_get_window(xw), gGui->video_window);
@@ -465,12 +480,11 @@ int gui_xine_open_and_play(char *_mrl, char *_sub, int start_pos,
       char *filename;
       
       filename = strrchr(url, '/');
-      if(strlen(filename) >= 2) {
-	char  fullfilename[XITK_PATH_MAX + XITK_NAME_MAX + 1];
+      if(filename && filename[1]) { /* we have a filename */
+	char  fullfilename[XITK_PATH_MAX + XITK_NAME_MAX + 2];
 	FILE *fd;
 	
 	filename++;
-	memset(&fullfilename, 0, sizeof(fullfilename));
 	snprintf(fullfilename, sizeof(fullfilename), "%s/%s", xine_get_homedir(), filename);
 	
 	if((fd = fopen(fullfilename, "w+b")) != NULL) {
@@ -516,7 +530,12 @@ int gui_xine_open_and_play(char *_mrl, char *_sub, int start_pos,
   }
 
   if(!xine_open(gGui->stream, (const char *) mrl)) {
-    
+
+#ifdef XINE_PARAM_GAPLESS_SWITCH
+    if( xine_check_version(1,1,1) )
+      xine_set_param(gGui->stream, XINE_PARAM_GAPLESS_SWITCH, 0);
+#endif
+
     if(!strcmp(mrl, gGui->mmk.mrl))
       gGui->playlist.mmk[gGui->playlist.cur]->played = 1;
     
@@ -725,7 +744,11 @@ void gui_play (xitk_widget_t *w, void *data) {
     }
   }
   else {
+    int oldspeed = xine_get_param(gGui->stream, XINE_PARAM_SPEED);
+
     xine_set_param(gGui->stream, XINE_PARAM_SPEED, XINE_SPEED_NORMAL);
+    if(oldspeed != XINE_SPEED_NORMAL)
+      osd_update_status();
   }
   
   panel_check_pause();
@@ -737,7 +760,7 @@ void gui_stop (xitk_widget_t *w, void *data) {
   xine_stop (gGui->stream);
   gGui->ignore_next = 0;
 
-  gGui->playlist.control &= !PLAYLIST_CONTROL_STOP;
+  gGui->playlist.control &= ~PLAYLIST_CONTROL_STOP;
 
   gGui->stream_length.pos = gGui->stream_length.time = gGui->stream_length.length = 0;
   
@@ -748,6 +771,9 @@ void gui_stop (xitk_widget_t *w, void *data) {
       gGui->visual_anim.running = 0;
   }
 
+  osd_hide_sinfo();
+  osd_hide_bar();
+  osd_hide_info();
   osd_update_status();
   panel_reset_slider ();
   panel_check_pause();
@@ -766,14 +792,21 @@ void gui_stop (xitk_widget_t *w, void *data) {
 }
 
 void gui_pause (xitk_widget_t *w, void *data, int state) {
-  if(xine_get_param(gGui->stream, XINE_PARAM_SPEED) != XINE_SPEED_PAUSE)
+  int speed = xine_get_param(gGui->stream, XINE_PARAM_SPEED);
+
+  if(speed != XINE_SPEED_PAUSE) {
+    last_playback_speed = speed;
     xine_set_param(gGui->stream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE);
+  }
   else {
-    xine_set_param(gGui->stream, XINE_PARAM_SPEED, XINE_SPEED_NORMAL);
+    xine_set_param(gGui->stream, XINE_PARAM_SPEED, last_playback_speed);
     video_window_reset_ssaver();
   }
   
   panel_check_pause();
+  /* Give xine engine some time before updating OSD, otherwise the */
+  /* time disp may be empty when switching to XINE_SPEED_PAUSE.    */
+  xine_usec_sleep(10000);
   osd_update_status();
 }
 
@@ -799,8 +832,7 @@ void gui_eject(xitk_widget_t *w, void *data) {
 	char *cur_mrl = (char *)mediamark_get_current_mrl();
 	
 	len = (mrl - cur_mrl) + 3;
-	tok = (char *) alloca(len + 1);
-	memset(tok, 0, len + 1);
+	tok = (char *) alloca(len);
   	snprintf(tok, len, "%s", cur_mrl);
       }
 
@@ -884,8 +916,11 @@ void gui_toggle_visibility(xitk_widget_t *w, void *data) {
 
     /* We need to reparent all visible windows because of redirect tweaking */
     if(!visible) { /* Show the panel in taskbar */
+      int x = 0, y = 0;
+
+      xitk_get_window_position(gGui->display, gGui->panel_window, &x, &y, NULL, NULL);
       XLockDisplay(gGui->display);
-      XReparentWindow(gGui->display, gGui->panel_window, gGui->imlib_data->x.root, 0, 0);
+      XReparentWindow(gGui->display, gGui->panel_window, gGui->imlib_data->x.root, x, y);
       XUnlockDisplay(gGui->display);
     }
     reparent_all_windows();
@@ -1088,24 +1123,32 @@ void gui_change_spu_channel(xitk_widget_t *w, void *data) {
 }
 
 void gui_change_speed_playback(xitk_widget_t *w, void *data) {
+  int speed = xine_get_param(gGui->stream, XINE_PARAM_SPEED);
 
   if(((int)data) == GUI_NEXT) {
-    if (xine_get_param(gGui->stream, XINE_PARAM_SPEED) > XINE_SPEED_PAUSE)
-      xine_set_param (gGui->stream, XINE_PARAM_SPEED, 
-		      (xine_get_param(gGui->stream, XINE_PARAM_SPEED)) / 2);
+    if(speed > XINE_SPEED_PAUSE)
+      xine_set_param(gGui->stream, XINE_PARAM_SPEED, (speed /= 2));
   }
   else if(((int)data) == GUI_PREV) {
-    if (xine_get_param (gGui->stream, XINE_PARAM_SPEED) < XINE_SPEED_FAST_4) {
-      if (xine_get_param (gGui->stream, XINE_PARAM_SPEED) > XINE_SPEED_PAUSE)
-	xine_set_param (gGui->stream, XINE_PARAM_SPEED, 
-			(xine_get_param(gGui->stream, XINE_PARAM_SPEED)) * 2);
-      else
-	xine_set_param (gGui->stream, XINE_PARAM_SPEED, XINE_SPEED_SLOW_4);
+    if(speed < XINE_SPEED_FAST_4) {
+      if(speed > XINE_SPEED_PAUSE)
+	xine_set_param(gGui->stream, XINE_PARAM_SPEED, (speed *= 2));
+      else {
+	xine_set_param(gGui->stream, XINE_PARAM_SPEED, (speed = XINE_SPEED_SLOW_4));
+	video_window_reset_ssaver();
+      }
     }
   }
   else if(((int)data) == GUI_RESET) {
-    xine_set_param (gGui->stream, XINE_PARAM_SPEED, XINE_SPEED_NORMAL);
+    xine_set_param(gGui->stream, XINE_PARAM_SPEED, (speed = XINE_SPEED_NORMAL));
   }
+  if(speed != XINE_SPEED_PAUSE)
+    last_playback_speed = speed;
+
+  panel_check_pause();
+  /* Give xine engine some time before updating OSD, otherwise the        */
+  /* time disp may be empty when switching to speeds < XINE_SPEED_NORMAL. */
+  xine_usec_sleep(10000);
   osd_update_status();
 }
 
@@ -1187,6 +1230,7 @@ static void *_gui_set_current_position(void *data) {
   } while(pos != -1);
   
   gGui->ignore_next = 0;
+  osd_hide_status();
   panel_check_pause();
 
   pthread_mutex_unlock(&gGui->xe_mutex);
@@ -1244,6 +1288,7 @@ static void *_gui_seek_relative(void *data) {
     osd_stream_position(pos);
   
   gGui->ignore_next = 0;
+  osd_hide_status();
   panel_check_pause();
 
   pthread_exit(NULL);
@@ -1356,6 +1401,13 @@ static void *_gui_dndcallback(void *data) {
       more_than_one = gGui->playlist.cur;
     }
     else {
+      char *ident;
+
+      /* If possible, use only base name as identifier to better fit into the display */
+      if((ident = strrchr(buffer, '/')) && ident[1])
+	ident++;
+      else
+	ident = buffer;
       
       if(mrl_look_like_playlist(buffer)) {
 	int cur = gGui->playlist.cur;
@@ -1364,10 +1416,10 @@ static void *_gui_dndcallback(void *data) {
 	if(mediamark_concat_mediamarks(buffer))
 	  gGui->playlist.cur = cur;
 	else
-	  mediamark_append_entry(buffer, buffer, NULL, 0, -1, 0, 0);
+	  mediamark_append_entry(buffer, ident, NULL, 0, -1, 0, 0);
       }
       else
-	mediamark_append_entry(buffer, buffer, NULL, 0, -1, 0, 0);
+	mediamark_append_entry(buffer, ident, NULL, 0, -1, 0, 0);
 
     }
     
@@ -1423,6 +1475,8 @@ void gui_direct_nextprev(xitk_widget_t *w, void *data, int value) {
 
   if(((int)data) == GUI_NEXT) {
 
+    osd_hide();
+
     if(by_chapter) {
 
       for(i = 0; i < value; i++)
@@ -1476,6 +1530,8 @@ void gui_direct_nextprev(xitk_widget_t *w, void *data, int value) {
     }
   }
   else if(((int)data) == GUI_PREV) {
+
+    osd_hide();
     
     if(by_chapter) {
 
@@ -1620,10 +1676,9 @@ void gui_set_current_mmk(mediamark_t *mmk) {
   else {
     char buffer[1024];
     
-    memset(&buffer, 0, sizeof(buffer));
     snprintf(buffer, sizeof(buffer), "xine-ui version %s", VERSION);
     
-    gGui->mmk.mrl           = strdup(_("There is no mrl."));
+    gGui->mmk.mrl           = strdup(_("There is no MRL."));
     gGui->mmk.ident         = strdup(buffer);
     gGui->mmk.sub           = NULL;
     gGui->mmk.start         = 0;
@@ -1787,13 +1842,7 @@ void gui_help_show(xitk_widget_t *w, void *data) {
  */
 int is_layer_above(void) {
   
-  if(gGui->always_layer_above || gGui->layer_above) {
-    if(!gGui->layer_above)
-      config_update_num("gui.layer_above", 1);
-    return 1;
-  }
-  
-  return 0;
+  return (gGui->always_layer_above || gGui->layer_above) ? 1 : 0;
 }
 /*
  * set window layer property to something above GNOME (and KDE?) panel
@@ -1819,40 +1868,44 @@ void layer_above_video(Window w) {
 }
 
 void change_amp_vol(int value) {
+  if(value < 0)
+    value = 0;
+  if(value > 200)
+    value = 200;
   gGui->mixer.amp_level = value;
   xine_set_param(gGui->stream, XINE_PARAM_AUDIO_AMP_LEVEL, gGui->mixer.amp_level);
-  xitk_slider_set_pos(panel->mixer.slider, gGui->mixer.amp_level);
+  if(gGui->mixer.method == SOFTWARE_MIXER)
+    xitk_slider_set_pos(panel->mixer.slider, gGui->mixer.amp_level);
   osd_draw_bar(_("Amplification Level"), 0, 200, gGui->mixer.amp_level, OSD_BAR_STEPPER);
 }
 void gui_increase_amp_level(void) {
-  if(gGui->mixer.amp_level < 200)
-    change_amp_vol((gGui->mixer.amp_level + 1));
+  change_amp_vol((gGui->mixer.amp_level + 1));
 }
 void gui_decrease_amp_level(void) {
-  if(gGui->mixer.amp_level > 0)
-    change_amp_vol((gGui->mixer.amp_level - 1));
+  change_amp_vol((gGui->mixer.amp_level - 1));
 }
 void gui_reset_amp_level(void) {
   change_amp_vol(100);
 }
 
 void change_audio_vol(int value) {
+  if(value < 0)
+    value = 0;
+  if(value > 100)
+    value = 100;
   gGui->mixer.volume_level = value;
   xine_set_param(gGui->stream, XINE_PARAM_AUDIO_VOLUME, gGui->mixer.volume_level);
   xitk_slider_set_pos(panel->mixer.slider, gGui->mixer.volume_level);
   osd_draw_bar(_("Audio Volume"), 0, 100, gGui->mixer.volume_level, OSD_BAR_STEPPER);
 }
 void gui_increase_audio_volume(void) {
-  if((gGui->mixer.method == SOUND_CARD_MIXER) && 
-     (gGui->mixer.caps & MIXER_CAP_VOL) && (gGui->mixer.volume_level < 100))
+  if((gGui->mixer.method == SOUND_CARD_MIXER) && (gGui->mixer.caps & MIXER_CAP_VOL))
     change_audio_vol((gGui->mixer.volume_level + 1));
   else if(gGui->mixer.method == SOFTWARE_MIXER)
     gui_increase_amp_level();
-  
 }
 void gui_decrease_audio_volume(void) {
-  if((gGui->mixer.method == SOUND_CARD_MIXER) &&
-     (gGui->mixer.caps & MIXER_CAP_VOL) && (gGui->mixer.volume_level > 0))
+  if((gGui->mixer.method == SOUND_CARD_MIXER) && (gGui->mixer.caps & MIXER_CAP_VOL))
     change_audio_vol((gGui->mixer.volume_level - 1));
   else if(gGui->mixer.method == SOFTWARE_MIXER)
     gui_decrease_amp_level();
@@ -1983,94 +2036,132 @@ static void fileselector_cancel_callback(filebrowser_t *fb) {
     load_sub = NULL;
 }
 
+
+/* Callback function for file select button or double-click in file browser.
+   Append selected file to the current playlist */
 static void fileselector_callback(filebrowser_t *fb) {
   char *file;
   char *cur_dir = filebrowser_get_current_dir(fb);
   
+  /* Upate configuration with the selected directory path */
   if(cur_dir && strlen(cur_dir)) {
     snprintf(gGui->curdir, sizeof(gGui->curdir), "%s", cur_dir);
     config_update_string("media.files.origin_path", gGui->curdir);
   }
   
+  /* Get the file path/name */
   if(((file = filebrowser_get_full_filename(fb)) != NULL) && strlen(file)) {
     int first  = gGui->playlist.num;
+    char *ident;
 
-    if(file)  {
-      gGui->playlist.control |= PLAYLIST_CONTROL_IGNORE;
-      gui_dndcallback(file);
-      gGui->playlist.control &= !PLAYLIST_CONTROL_IGNORE;
-      free(file);
+    /* Get the name only to use as an identifier in the playlist display */
+    ident = filebrowser_get_current_filename(fb);
+
+    /* If the file has an extension which could be a playlist, attempt to append
+       it to the current list as a list; otherwise, append it as an ordinary file. */
+    if(mrl_look_like_playlist(file)) {
+      if(!mediamark_concat_mediamarks(file))
+        mediamark_append_entry(file, ident, NULL, 0, -1, 0, 0);
     }
+    else
+      mediamark_append_entry(file, ident, NULL, 0, -1, 0, 0);
 
-    if((first != gGui->playlist.num) && gGui->smart_mode) {
-      if(xine_get_status(gGui->stream) != XINE_STATUS_STOP) {
-	gGui->ignore_next = 1;
-	xine_stop(gGui->stream);
-	gGui->ignore_next = 0;
-      }
-      
+    playlist_update_playlist();
+    free(file);
+    free(ident);
+
+    /* Enable controls on display */
+    if((!is_playback_widgets_enabled()) && gGui->playlist.num)
+      enable_playback_controls(1);
+
+    /* If an MRL is not being played, select the first file appended. If in "smart mode" start
+       playing the entry.  If a an MRL is currently being played, let it continue normally */
+    if((first != gGui->playlist.num) && (xine_get_status(gGui->stream) == XINE_STATUS_STOP)) {
       gGui->playlist.cur = first;
-      gui_set_current_mmk(mediamark_get_current_mmk());
-      gui_play(NULL, NULL);
+      if(gGui->smart_mode) {
+        gui_set_current_mmk(mediamark_get_current_mmk());
+        gui_play(NULL, NULL);
+      }
     }
-  }
+  } /* If valid file name */
 
   load_stream = NULL;
 }
 
+
+/* Callback function for "Select All" button in file browser. Append all files in the
+   currently selected directory to the current playlist. */
 static void fileselector_all_callback(filebrowser_t *fb) {
   char **files;
   char  *path = filebrowser_get_current_dir(fb);
   
+  /* Update the configuration with the current path */
   if(path && strlen(path)) {
     snprintf(gGui->curdir, sizeof(gGui->curdir), "%s", path);
     config_update_string("media.files.origin_path", gGui->curdir);
   }
   
+  /* Get all of the file names in the current directory as an array of pointers to strings */
   if((files = filebrowser_get_all_files(fb)) != NULL) {
     int i = 0;
 
     if(path && strlen(path)) {
-      char pathname[XITK_PATH_MAX + XITK_NAME_MAX + 1];
-      char fullfilename[XITK_PATH_MAX + XITK_NAME_MAX + 1];
-      int  first = gGui->playlist.num;
+      char pathname[XITK_PATH_MAX + 1 + 1]; /* +1 for trailing '/' */
+      char fullfilename[XITK_PATH_MAX + XITK_NAME_MAX + 2];
+      int  first = gGui->playlist.num; /* current count of entries in playlist */
 
+      /* If the path is anything other than "/", append a slash to it so that it can
+         be concatenated with the file name */
       if(strcasecmp(path, "/"))
-	snprintf(pathname, sizeof(pathname), "%s/", path);
+        snprintf(pathname, sizeof(pathname), "%s/", path);
       else
-	snprintf(pathname, sizeof(pathname), "%s", path);
+        snprintf(pathname, sizeof(pathname), "%s", path);
       
-      gGui->playlist.control |= PLAYLIST_CONTROL_IGNORE;
+      /* For each file, concatenate the path with the name and append it to the playlist */
       while(files[i]) {
-	snprintf(fullfilename, sizeof(fullfilename), "%s%s", pathname, files[i++]);
-	gui_dndcallback(fullfilename);
-      }
-      gGui->playlist.control &= !PLAYLIST_CONTROL_IGNORE;
+        snprintf(fullfilename, sizeof(fullfilename), "%s%s", pathname, files[i]);
+
+        /* If the file has an extension which could be a playlist, attempt to append
+           it to the current list as a list; otherwise, append it as an ordinary file. */
+        if(mrl_look_like_playlist(fullfilename)) {
+          if(!mediamark_concat_mediamarks(fullfilename))
+            mediamark_append_entry(fullfilename, files[i], NULL, 0, -1, 0, 0);
+        }
+        else
+          mediamark_append_entry(fullfilename, files[i], NULL, 0, -1, 0, 0);
+
+        i++;
+      } /* End while */
       
+      playlist_update_playlist();
       free(path);
 
-      if((first != gGui->playlist.num) && gGui->smart_mode) {
-	if(xine_get_status(gGui->stream) == XINE_STATUS_PLAY) {
-	  gGui->ignore_next = 1;
-	  xine_stop(gGui->stream);
-	  gGui->ignore_next = 0;
-	}
-	gGui->playlist.cur = first;
-	gui_set_current_mmk(mediamark_get_current_mmk());
-	gui_play(NULL, NULL);
+      /* Enable playback controls on display */
+      if((!is_playback_widgets_enabled()) && gGui->playlist.num)
+        enable_playback_controls(1);
+
+      /* If an MRL is not being played, select the first file appended. If in "smart mode" start
+         playing the entry.  If an MRL is currently being played, let it continue normally */
+      if((first != gGui->playlist.num) && (xine_get_status(gGui->stream) == XINE_STATUS_STOP)) {
+        gGui->playlist.cur = first;
+        if(gGui->smart_mode) {
+          gui_set_current_mmk(mediamark_get_current_mmk());
+          gui_play(NULL, NULL);
+        }
       }
-    }
+    } /* If valid path */
 
     i = 0;
     while(files[i])
       free(files[i++]);
     
     free(files);
-  }
+  } /* If valid file list */
 
   load_stream = NULL;
 }
 
+
 void gui_file_selector(void) {
   filebrowser_callback_button_t  cbb[3];
 
@@ -2087,7 +2178,7 @@ void gui_file_selector(void) {
     cbb[1].need_a_file = 0;
     cbb[2].callback = fileselector_cancel_callback;
     cbb[2].need_a_file = 0;
-    load_stream = create_filebrowser(_("Stream(s) loading"), gGui->curdir, hidden_file_cb, &cbb[0], &cbb[1], &cbb[2]);
+    load_stream = create_filebrowser(_("Stream(s) Loading"), gGui->curdir, hidden_file_cb, &cbb[0], &cbb[1], &cbb[2]);
   }
 }
 
@@ -2178,7 +2269,6 @@ void gui_select_sub(void) {
 void visual_anim_init(void) {
   char buffer[4096];
 
-  memset(&buffer, 0, sizeof(buffer));
   snprintf(buffer, sizeof(buffer), "%s/%s", XINE_VISDIR, "default.avi");
   
   gGui->visual_anim.mrls = (char **) xine_xmalloc(sizeof(char *) * 3);
