@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2007 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: common.h,v 1.82 2005/07/16 19:05:30 miguelfreitas Exp $
+ * $Id: common.h,v 1.84 2007/04/09 22:39:35 fsck-p Exp $
  *
  */
 
@@ -343,7 +343,12 @@ typedef struct {
     int                      set;
     int                      arg;
   } numeric;
-  
+
+  struct {
+    int                      set;
+    char                    *arg;
+  } alphanum;
+
   int                        eventer_sticky;
   int                        stream_info_auto_update;
 
@@ -358,6 +363,9 @@ typedef struct {
   FILE                      *report;
 
   int                        no_gui;
+  
+  int                        no_mouse;
+  int                        wid;
 
   void                     (*nongui_error_msg)(char *text);
 
