@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: errors.h,v 1.9 2004/05/22 23:14:31 miguelfreitas Exp $
+ * $Id: errors.h,v 1.10 2006/04/20 19:02:13 dsalt Exp $
  *
  */
 
@@ -26,11 +26,11 @@
 
 #include <stdarg.h>
 
-void xine_error(char *message, ...);
+void xine_error(char *message, ...) __attribute__ ((format (printf, 1, 2)));
 
-void xine_error_with_more(char *message, ...);
+void xine_error_with_more(char *message, ...) __attribute__ ((format (printf, 1, 2)));
 
-void xine_info(char *message, ...);
+void xine_info(char *message, ...) __attribute__ ((format (printf, 1, 2)));
 
 void gui_handle_xine_error(xine_stream_t *stream, char *mrl);
 
