@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2007 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: event.c,v 1.249 2004/04/14 22:38:29 f1rmb Exp $
+ * $Id: event.c,v 1.283 2007/04/09 22:39:35 fsck-p Exp $
  *
  * gui initialization and top-level event handling stuff
  *
@@ -43,6 +43,7 @@
 #include <stdlib.h>
 
 #include "common.h"
+#include "oxine/oxine.h"
 
 #define STEP_SIZE 256
 
@@ -55,7 +56,7 @@ extern _panel_t        *panel;
 static pid_t            xine_pid;
 
 /* Icon data */
-static unsigned char xine_bits[] = {
+static unsigned char icon_datas[] = {
    0x11, 0x00, 0x00, 0x00, 0x88, 0x11, 0x00, 0x00, 0x00, 0x88, 0xff, 0xff,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xff, 0xff, 0xff,
    0x8f, 0xf1, 0xff, 0xff, 0xff, 0x8f, 0x1f, 0x00, 0x00, 0x00, 0xf8, 0x1f,
@@ -107,6 +108,20 @@ static char *shortcut_style[] = {
   NULL
 };
 
+int hidden_file_cb(int action, int value) {
+  xine_cfg_entry_t  cfg_entry;
+  int               retval = 0;
+  
+  if(xine_config_lookup_entry(gGui->xine, "media.files.show_hidden_files", &cfg_entry)) {
+    if(action)
+      config_update_bool("media.files.show_hidden_files", value);
+    else
+      retval = cfg_entry.num_value;
+  }
+
+  return retval;
+}
+
 void dummy_config_cb(void *data, xine_cfg_entry_t *cfg) {
   /* It exist to avoid "restart" window message in setup window */
 }
@@ -126,10 +141,44 @@ static void ssaver_timeout_cb(void *data, xine_cfg_entry_t *cfg) {
 
 static void visual_anim_cb(void *data, xine_cfg_entry_t *cfg) {
   
-  if(((gGui->visual_anim.enabled == 2) && (cfg->num_value != 2)) && gGui->visual_anim.running)
-     visual_anim_stop();
+  if(gGui->visual_anim.enabled == cfg->num_value)
+    return;
+
+  if((gGui->visual_anim.enabled) && (cfg->num_value == 0) && gGui->visual_anim.running)
+    visual_anim_stop();
+  
+  if(gGui->visual_anim.enabled && gGui->visual_anim.running) {
+    if((gGui->visual_anim.enabled == 1) && (cfg->num_value != 1)) {
+      if(post_rewire_audio_port_to_stream(gGui->stream))
+	gGui->visual_anim.running = 0;
+    }
+    if((gGui->visual_anim.enabled == 2) && (cfg->num_value != 2)) {
+      visual_anim_stop();
+      gGui->visual_anim.running = 0;
+    }
+  }
   
   gGui->visual_anim.enabled = cfg->num_value;
+  
+  if(gGui->visual_anim.enabled) {
+    int  has_video = xine_get_stream_info(gGui->stream, XINE_STREAM_INFO_HAS_VIDEO);
+    
+    if(has_video)
+      has_video = !xine_get_stream_info(gGui->stream, XINE_STREAM_INFO_IGNORE_VIDEO);
+    
+    if(!has_video && !gGui->visual_anim.running) {
+      if(gGui->visual_anim.enabled == 1) {
+	if(gGui->visual_anim.post_output_element.post) {
+	  if(post_rewire_audio_post_to_stream(gGui->stream))
+	    gGui->visual_anim.running = 1;
+	}
+      }
+      else if(gGui->visual_anim.enabled == 2) {
+	visual_anim_play();
+	gGui->visual_anim.running = 1;
+      }
+    }
+  }
 }
 
 static void stream_info_auto_update_cb(void *data, xine_cfg_entry_t *cfg) {
@@ -285,17 +334,19 @@ static void gui_signal_handler (int sig, void *data) {
 /*
  * convert pts value to string
  */
-char *pts2str(int64_t pts) {
-  static char buf[40];
-  int64_t min;
-  double sec;
-  int ds;
-  int sign;
+static char *pts2str(int64_t pts) {
+  static char  buf[40];
+  int64_t      min;
+  double       sec;
+  int          ds;
+  int          sign;
+
+  if((sign = (pts < 0)) != 0)
+    pts = -pts;
 
-  if ((sign = pts < 0) != 0) pts = -pts;
   min = pts / (90000 * 60);
   sec = (double)pts / 90000 - 60 * min;
-  ds = sec / 10;
+  ds  = sec / 10;
   sec -= 10 * ds;
 
   snprintf(buf, sizeof(buf), "%s%02" PRIi64 ":%d%.2f (%" PRIi64 " pts)", sign ? "-" : "", min, ds, sec, pts);
@@ -344,6 +395,10 @@ void gui_execute_action_id(action_id_t action) {
       gGui->numeric.arg = 0;
     }
 
+    /* check if osd menu like this event */
+    if( oxine_action_event(action & ~ACTID_IS_INPUT_EVENT) )
+      return;
+    
     /* events for advanced input plugins. */
     xine_event.type        = action & ~ACTID_IS_INPUT_EVENT;
     xine_event.data_length = 0;
@@ -358,27 +413,60 @@ void gui_execute_action_id(action_id_t action) {
   switch(action) {
     
   case ACTID_WINDOWREDUCE:
-    video_window_set_mag (0.8 * video_window_get_mag());
+    {
+      int	output_width, output_height;
+
+      /* Limit size to a practical minimum.                         */
+      /* Too small window is hard to view or to hit with the mouse. */
+
+      video_window_get_output_size(&output_width, &output_height);
+      if(output_width > 50 && output_height > 50) {
+	float	xmag, ymag;
+
+	video_window_get_mag(&xmag, &ymag);
+	video_window_set_mag(xmag * (1/1.2f), ymag * (1/1.2f));
+      }
+    }
     break;
 
   case ACTID_WINDOWENLARGE:
-    video_window_set_mag (1.2 * (video_window_get_mag() + 0.002));
+    {
+      int	output_width, output_height, window_width, window_height;
+
+      /* Limit size to a practical maximum.                                        */
+      /* Too large window can grab system resources up to a virtual dead lock.     */
+      /* Compare output size to window size as a WM (like fvwm) can be configured  */
+      /* to impose limits on window size which may not be reflected back to output */
+      /* size (no further ConfigureNotify after size limit is exceeded),           */
+      /* so we'll not end up in endless magnification.                             */
+
+      video_window_get_output_size(&output_width, &output_height);
+      xitk_get_window_position(gGui->video_display, gGui->video_window,
+			       NULL, NULL, &window_width, &window_height);
+      if(output_width < 5000 && output_height < 5000 &&
+	 output_width <= window_width && output_height <= window_height) {
+	float	xmag, ymag;
+
+	video_window_get_mag(&xmag, &ymag);
+	video_window_set_mag(xmag * 1.2f, ymag * 1.2f);
+      }
+    }
     break;
 
   case ACTID_ZOOM_1_1:
   case ACTID_WINDOW100:
-    video_window_set_mag (1.0);
-    osd_display_info(_("Zoom: 1:1"));
+    if(video_window_set_mag (1.0f, 1.0f))
+      osd_display_info(_("Zoom: 1:1"));
     break;
 
   case ACTID_WINDOW200:
-    video_window_set_mag (2.0);
-    osd_display_info(_("Zoom: 200%%"));
+    if(video_window_set_mag (2.0f, 2.0f))
+      osd_display_info(_("Zoom: 200%%"));
     break;
 
   case ACTID_WINDOW50:
-    video_window_set_mag (0.5);
-    osd_display_info(_("Zoom: 50%%"));
+    if(video_window_set_mag (0.5f, 0.5f))
+      osd_display_info(_("Zoom: 50%%"));
     break;
 
   case ACTID_SPU_NEXT:
@@ -400,9 +488,13 @@ void gui_execute_action_id(action_id_t action) {
     break;
     
   case ACTID_TOGGLE_WINOUT_VISIBLITY:
-    gui_toggle_visibility (NULL, NULL);
+    gui_toggle_visibility(NULL, NULL);
     break;
     
+  case ACTID_TOGGLE_WINOUT_BORDER:
+    gui_toggle_border(NULL, NULL);
+    break;
+
   case ACTID_AUDIOCHAN_NEXT:
     if(!gGui->numeric.set)
       gui_change_audio_channel(NULL, (void*)GUI_NEXT);
@@ -592,8 +684,10 @@ void gui_execute_action_id(action_id_t action) {
       int av_offset = (xine_get_param(gGui->stream, XINE_PARAM_AV_OFFSET) - 3600);
 
       gGui->mmk.av_offset = av_offset;
-      if (gGui->playlist.cur < gGui->playlist.num)
-        gGui->playlist.mmk[gGui->playlist.cur]->av_offset = av_offset;
+      if (gGui->playlist.num && gGui->playlist.cur >= 0 && gGui->playlist.mmk &&
+	  gGui->playlist.mmk[gGui->playlist.cur])
+	if (gGui->playlist.cur < gGui->playlist.num)
+	  gGui->playlist.mmk[gGui->playlist.cur]->av_offset = av_offset;
       xine_set_param(gGui->stream, XINE_PARAM_AV_OFFSET, av_offset);
       osd_display_info(_("A/V offset: %s"), pts2str(av_offset));
     }
@@ -604,14 +698,21 @@ void gui_execute_action_id(action_id_t action) {
       int av_offset = (xine_get_param(gGui->stream, XINE_PARAM_AV_OFFSET) + 3600);
       
       gGui->mmk.av_offset = av_offset;
-      if (gGui->playlist.cur < gGui->playlist.num)
-        gGui->playlist.mmk[gGui->playlist.cur]->av_offset = av_offset;
+      if (gGui->playlist.num && gGui->playlist.cur >= 0 && gGui->playlist.mmk &&
+	  gGui->playlist.mmk[gGui->playlist.cur])
+	if (gGui->playlist.cur < gGui->playlist.num)
+	  gGui->playlist.mmk[gGui->playlist.cur]->av_offset = av_offset;
       xine_set_param(gGui->stream, XINE_PARAM_AV_OFFSET, av_offset);
       osd_display_info(_("A/V offset: %s"), pts2str(av_offset));
     }
     break;
 
   case ACTID_AV_SYNC_RESET:
+    gGui->mmk.av_offset = 0;
+    if (gGui->playlist.num && gGui->playlist.cur >= 0 && gGui->playlist.mmk &&
+	gGui->playlist.mmk[gGui->playlist.cur])
+      if (gGui->playlist.cur < gGui->playlist.num)
+	gGui->playlist.mmk[gGui->playlist.cur]->av_offset = 0;
     xine_set_param(gGui->stream, XINE_PARAM_AV_OFFSET, 0);
     osd_display_info(_("A/V Offset: reset."));
     break;
@@ -661,6 +762,14 @@ void gui_execute_action_id(action_id_t action) {
     gui_decrease_audio_volume();
     break;
 
+  case ACTID_APP:
+    gui_app_show(NULL, NULL);
+    break;
+
+  case ACTID_APP_ENABLE:
+    gui_app_enable();
+    break;
+
   case ACTID_pAMP:
     gui_increase_amp_level();
     break;
@@ -708,10 +817,10 @@ void gui_execute_action_id(action_id_t action) {
   case ACTID_GRAB_POINTER:
     if(!gGui->cursor_grabbed) {
       if(!panel_is_visible()) {
-	XLockDisplay(gGui->display);
-	XGrabPointer(gGui->display, gGui->video_window, 1, None, 
+	XLockDisplay(gGui->video_display);
+	XGrabPointer(gGui->video_display, gGui->video_window, 1, None, 
 		     GrabModeAsync, GrabModeAsync, gGui->video_window, None, CurrentTime);
-	XUnlockDisplay(gGui->display);
+	XUnlockDisplay(gGui->video_display);
       }
       
       gGui->cursor_grabbed = 1;
@@ -798,6 +907,18 @@ void gui_execute_action_id(action_id_t action) {
     osd_stream_infos();
     break;
 
+  case ACTID_OSD_WTEXT:
+    if(gGui->alphanum.set) {
+      osd_display_info("%s", gGui->alphanum.arg);
+    } else {
+      osd_display_info(_("No text to display!"));
+    }
+    break;
+
+  case ACTID_OSD_MENU:
+    oxine_menu();
+    break;
+
   case ACTID_FILESELECTOR:
     gui_file_selector();
     break;
@@ -877,7 +998,7 @@ void gui_execute_action_id(action_id_t action) {
   case ACTID_PLAYLIST_STOP:
     if(xine_get_status(gGui->stream) != XINE_STATUS_STOP) {
       if(gGui->playlist.control & PLAYLIST_CONTROL_STOP)
-	gGui->playlist.control &= !PLAYLIST_CONTROL_STOP;
+	gGui->playlist.control &= ~PLAYLIST_CONTROL_STOP;
       else
 	gGui->playlist.control |= PLAYLIST_CONTROL_STOP;
       osd_display_info(_("Playlist: %s"), 
@@ -952,30 +1073,37 @@ void gui_playlist_start_next(void) {
   if (gGui->ignore_next)
     return;
 
+  osd_hide();
   panel_reset_slider ();
   
   if(gGui->playlist.control & PLAYLIST_CONTROL_STOP) {
-    gGui->playlist.control &= !PLAYLIST_CONTROL_STOP;
+    gGui->playlist.control &= ~PLAYLIST_CONTROL_STOP;
     gui_display_logo();
     return;
   }
   
   switch(gGui->playlist.loop) {
-    
+
   case PLAYLIST_LOOP_NO_LOOP:
   case PLAYLIST_LOOP_LOOP:
     gGui->playlist.cur++;
     
     if(gGui->playlist.cur < gGui->playlist.num) {
-      gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+      gui_set_current_mmk(mediamark_get_current_mmk());
+      
       if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
-				 gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset)) {
-
-	if((gGui->playlist.loop == PLAYLIST_LOOP_NO_LOOP) && 
-	   mediamark_all_played() && (gGui->actions_on_start[0] == ACTID_QUIT))
-	  gui_exit(NULL, NULL);
-
-	gui_display_logo();
+                                 gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset, 
+                                 !mediamark_have_alternates(&(gGui->mmk)))) {
+      
+        if(!mediamark_have_alternates(&(gGui->mmk)) || 
+	   !gui_open_and_play_alternates(&(gGui->mmk), gGui->mmk.sub)) {
+        
+          if((gGui->playlist.loop == PLAYLIST_LOOP_NO_LOOP) && 
+              mediamark_all_played() && (gGui->actions_on_start[0] == ACTID_QUIT))
+            gui_exit(NULL, NULL);
+          
+          gui_display_logo();
+        }
       }
     }
     else {
@@ -990,23 +1118,35 @@ void gui_playlist_start_next(void) {
       }
       else if(gGui->playlist.loop == PLAYLIST_LOOP_LOOP) {
 	gGui->playlist.cur = 0;
-	gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
-	if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0,
-				   gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset)) {
-	  if(mediamark_all_played() && (gGui->actions_on_start[0] == ACTID_QUIT))
-	    gui_exit(NULL, NULL);
-	  gui_display_logo();
-	}
-      }
+	gui_set_current_mmk(mediamark_get_current_mmk());
 
+        if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0,
+                                   gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset, 
+                                   !mediamark_have_alternates(&(gGui->mmk)))) {
+          
+          if(!mediamark_have_alternates(&(gGui->mmk)) || !gui_open_and_play_alternates(&(gGui->mmk), gGui->mmk.sub)) {
+            if(mediamark_all_played() && (gGui->actions_on_start[0] == ACTID_QUIT))
+              gui_exit(NULL, NULL);
+	    
+	    gui_display_logo();
+          }
+        }
+      }
     }
     break;
     
   case PLAYLIST_LOOP_REPEAT:
-    gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+    gui_set_current_mmk(mediamark_get_current_mmk());
     if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
-			       gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset))
-      gui_display_logo();
+                               gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset,
+                               !mediamark_have_alternates(&(gGui->mmk)))) {
+      
+      if(mediamark_have_alternates(&(gGui->mmk))) {
+        if(!gui_open_and_play_alternates(&(gGui->mmk), gGui->mmk.sub))
+          gui_display_logo();
+
+      }
+    }
     break;
 
   case PLAYLIST_LOOP_SHUFFLE:
@@ -1016,13 +1156,19 @@ void gui_playlist_start_next(void) {
     __shuffle_restart:
       gGui->playlist.cur = mediamark_get_shuffle_next();
       
-      gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+      gui_set_current_mmk(mediamark_get_current_mmk());
       if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
-				 gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset)) {
-	if(!mediamark_all_played())
-	  goto __shuffle_restart;
-	else
-	  gui_display_logo();
+                                 gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset,
+                                 !mediamark_have_alternates(&(gGui->mmk)))) {
+        
+        if(!mediamark_have_alternates(&(gGui->mmk)) ||
+           !gui_open_and_play_alternates(&(gGui->mmk), gGui->mmk.sub)) {
+        
+          if(!mediamark_all_played())
+            goto __shuffle_restart;
+          else
+            gui_display_logo();
+        }
       }
     }
     else {
@@ -1040,7 +1186,7 @@ void gui_playlist_start_next(void) {
   }
   
   if(is_playback_widgets_enabled() && (!gGui->playlist.num)) {
-    gui_set_current_mrl(NULL);
+    gui_set_current_mmk(NULL);
     enable_playback_controls(0);
   }
 
@@ -1143,12 +1289,12 @@ void gui_deinit(void) {
 void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attribute) {
   int    i;
   char  *server;
+  char  *video_display_name;
 
   /*
    * init playlist
    */
   for (i = 0; i < nfiles; i++) {
-    char *sub = NULL;
     char *file = atoa(filenames[i]);
 
     /* grab recursively all files from dir */
@@ -1156,22 +1302,31 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
       
       if(file[strlen(file) - 1] == '/')
 	file[strlen(file) - 1] = '\0'; 
-      
+
       mediamark_collect_from_directory(file);
     }
     else {
-      if((sub = (char *)get_last_double_semicolon(file)) != NULL) {
-	if(is_ipv6_last_double_semicolon(file))
-	  sub = NULL;
-	else {
-	  *sub = 0;
-	  sub += 2;
+
+      if(mrl_look_like_playlist(file))
+	(void) mediamark_concat_mediamarks(file);
+      else {
+	char *sub = NULL;
+
+	if((sub = (char *)get_last_double_semicolon(file)) != NULL) {
+	  if(is_ipv6_last_double_semicolon(file))
+	    sub = NULL;
+	  else {
+	    *sub = 0;
+	    sub += 2;
+	  }
 	}
+	
+	mediamark_append_entry((const char *)file, (const char *)file, sub, 0, -1, 0, 0);
       }
-      
-      mediamark_add_entry((const char *)file, (const char *)file, sub, 0, -1, 0, 0);
     }
   }
+
+  gGui->playlist.cur = gGui->playlist.num ? 0 : -1;
   
   if((gGui->playlist.loop == PLAYLIST_LOOP_SHUFFLE) || 
      (gGui->playlist.loop == PLAYLIST_LOOP_SHUF_PLUS))
@@ -1185,10 +1340,10 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
    * X / imlib stuff
    */
 
-  if (!XInitThreads ()) {
+  if(!XInitThreads()) {
     printf (_("\nXInitThreads failed - looks like you don't have a "
 	    "thread-safe xlib.\n"));
-    exit (1);
+    exit(1);
   } 
   
   if((gGui->display = XOpenDisplay((getenv("DISPLAY")))) == NULL) {
@@ -1196,10 +1351,36 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
     exit(1);
   }
 
+  video_display_name = 
+    (char *)xine_config_register_string (gGui->xine, "gui.video_display", 
+					 "",
+					 _("Name of video display"),
+					 _("Use this setting to configure to which "
+                                           "display xine will show videos. When left blank, "
+                                           "the main display will be used. Setting this "                  "option to something like ':0.1' or ':1' makes "
+                                           "possible to display video and control on different "
+                                           "screens (very useful for TV presentations)."),
+					 CONFIG_LEVEL_ADV,
+					 NULL,
+					 CONFIG_NO_DATA);
+  
+  gGui->video_display = NULL;
+  if ( strlen(video_display_name) ) {
+  
+    if ((gGui->video_display = XOpenDisplay(video_display_name)) == NULL )
+      fprintf(stderr, _("Cannot open display '%s' for video. Falling back to primary display.\n"),
+              video_display_name);
+  }
+  
+  if ( gGui->video_display == NULL ) {
+    gGui->video_display = gGui->display;
+  }
+
   if (xine_config_register_bool (gGui->xine, "gui.xsynchronize", 
 				 0,
 				 _("Synchronized X protocol (debug)"), 
-				 CONFIG_NO_HELP,
+				 _("Do X transactions in synchronous mode. "
+				   "Very slow, use only for debugging!"), 
 				 CONFIG_LEVEL_ADV,
 				 CONFIG_NO_CB,
 				 CONFIG_NO_DATA)) {
@@ -1207,13 +1388,14 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
     XLockDisplay(gGui->display);
     XSynchronize (gGui->display, True);
     XUnlockDisplay(gGui->display);
-    fprintf (stderr, _("Warning! Synchronized X activated - this is way slow...\n"));
+    fprintf (stderr, _("Warning! Synchronized X activated - this is very slow...\n"));
   }
 
   gGui->layer_above = 
     xine_config_register_bool (gGui->xine, "gui.layer_above", 0,
 			       _("Windows stacking"),
-			       _("Use wm layer property to place window on top."), 
+			       _("Use WM layer property to place windows on top\n"
+				 "except for the video window in non-fullscreen mode."), 
 			       CONFIG_LEVEL_ADV,
 			       layer_above_cb,
 			       CONFIG_NO_DATA);
@@ -1221,14 +1403,12 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
   gGui->always_layer_above = 
     xine_config_register_bool (gGui->xine, "gui.always_layer_above", 0,
 			       _("Windows stacking (more)"),
-			       _("Use WM layer property to place windows on top."), 
+			       _("Use WM layer property to place windows on top\n"
+				 "for all windows (surpasses the 'layer_above' setting)."), 
 			       CONFIG_LEVEL_ADV,
 			       always_layer_above_cb,
 			       CONFIG_NO_DATA);
 
-  if(gGui->always_layer_above && (!gGui->layer_above))
-    config_update_num("gui.layer_above", 1);
-
   gGui->snapshot_location = 
     (char *)xine_config_register_string (gGui->xine, "gui.snapshotdir", 
 					 (char *) (xine_get_homedir()),
@@ -1240,8 +1420,8 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
   
   gGui->ssaver_timeout =
     xine_config_register_num (gGui->xine, "gui.screensaver_timeout", 10,
-			      _("Screensaver wakeup"),
-			      _("Time between two screensaver fake events, 0 to disable."),
+			      _("Screensaver reset interval (s)"),
+			      _("Time, in seconds, between two faked events to keep a screensaver quiet, 0 to disable."),
 			      CONFIG_LEVEL_ADV,
 			      ssaver_timeout_cb,
 			      CONFIG_NO_DATA);
@@ -1411,6 +1591,12 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
   XLockDisplay (gGui->display);
 
   gGui->screen = DefaultScreen(gGui->display);
+  
+  XLockDisplay (gGui->video_display);
+  gGui->video_screen = DefaultScreen(gGui->video_display);
+  XUnlockDisplay (gGui->video_display);
+
+  
 
   /* Some infos */
   if(gGui->verbosity) {
@@ -1430,7 +1616,7 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
    */
   xine_pid = getppid();
   
-  xitk_init(gGui->display, (gGui->verbosity) ? 1 : 0);
+  xitk_init(gGui->display, gGui->black, (gGui->verbosity) ? 1 : 0);
   
   preinit_skins_support();
   
@@ -1450,9 +1636,9 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
    * when done before the video_window_init(). */
   gGui->kbindings = kbindings_init_kbinding();
 
-  gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+  gui_set_current_mmk(mediamark_get_current_mmk());
 
-  panel_init ();
+  panel_init();
 }
 
 void gui_init_imlib (Visual *vis) {
@@ -1499,7 +1685,7 @@ void gui_init_imlib (Visual *vis) {
   
   gGui->icon = XCreateBitmapFromData (gGui->display, 
 				      gGui->imlib_data->x.root,
-				      xine_bits, 40, 40);
+				      icon_datas, 40, 40);
 }
 
 /*
@@ -1515,8 +1701,8 @@ static void on_start(void *data) {
 
   splash_destroy();
 
-  if((!startup->start && !gGui->playlist.num) || (!startup->start && gGui->playlist.num)) {
-
+  if(!startup->start) {
+    
     gui_display_logo();
     
     do {
@@ -1525,27 +1711,12 @@ static void on_start(void *data) {
 
   }
 
-  /* Crapy Workaround:
-   * The lib seems unable to display OSD until the stream got 
-   * any video data.
-   */
-  if(startup->start || (!startup->start && !gGui->display_logo)) {
-    (void) xine_open(gGui->stream, (const char *) XINE_BLACK_STREAM);
-    
-    if(!startup->start && !gGui->display_logo)
-      xine_play(gGui->stream, 0, 0);
-    
-    gGui->ignore_next = 1;
-    xine_stop(gGui->stream);
-    gGui->ignore_next = 0;
-  }
-  
   if(startup->session_opts) {
     int i = 0;
     int dummy_session;
     
     while(startup->session_opts[i])
-      session_handle_subopt(startup->session_opts[i++], &dummy_session);
+      (void) session_handle_subopt(startup->session_opts[i++], &dummy_session);
     
   }
   
@@ -1558,8 +1729,9 @@ void gui_run(char **session_opts) {
   _startup_t  startup;
   int         i, auto_start = 0;
   
-  video_window_change_skins();
+  video_window_change_skins(0);
   panel_add_autoplay_buttons();
+  panel_show_tips();
   panel_add_mixer_control();
   panel_update_channel_display () ;
   panel_update_mrl_display ();
@@ -1585,11 +1757,11 @@ void gui_run(char **session_opts) {
 	  
 	  if(autoplay_mrls) {
 	    for (j = 0; j < num_mrls; j++)
-	      mediamark_add_entry((const char *)autoplay_mrls[j],
-				  (const char *)autoplay_mrls[j], NULL, 0, -1, 0, 0);
+	      mediamark_append_entry((const char *)autoplay_mrls[j],
+				     (const char *)autoplay_mrls[j], NULL, 0, -1, 0, 0);
 	   
 	    gGui->playlist.cur = 0;
-	    gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	    gui_set_current_mmk(mediamark_get_current_mmk());
 	  }
 	}    
       }
@@ -1652,7 +1824,7 @@ void gui_run(char **session_opts) {
       if(video_window_is_visible())
 	video_window_set_visibility(!(video_window_is_visible()));
       else
-	xine_gui_send_vo_data(gGui->stream,
+	xine_port_send_gui_data(gGui->vo_port,
 			      XINE_GUI_SEND_VIDEOWIN_VISIBLE,
 			      (int *)0);
       
@@ -1670,10 +1842,13 @@ void gui_run(char **session_opts) {
 
     /* User load a playlist on startup */
     if(actions_on_start(gGui->actions_on_start, ACTID_PLAYLIST)) {
-      mediamark_load_mediamarks(gGui->playlist.on_start);
-      gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
-      if((!is_playback_widgets_enabled()) && gGui->playlist.num)
-	enable_playback_controls(1);
+      gui_set_current_mmk(mediamark_get_current_mmk());
+
+      if(gGui->playlist.num) {
+	gGui->playlist.cur = 0;
+	if(!is_playback_widgets_enabled())
+	  enable_playback_controls(1);
+      }
     }
 
     if(actions_on_start(gGui->actions_on_start, ACTID_TOGGLE_INTERLEAVE))
@@ -1696,13 +1871,15 @@ void gui_run(char **session_opts) {
   startup.start        = auto_start;
   startup.session_opts = session_opts;
 
+  oxine_init();
+  
   xitk_run(on_start, (void *)&startup);
 
   /* save playlist */
   if(gGui->playlist.mmk && gGui->playlist.num) {
     char buffer[XITK_PATH_MAX + XITK_NAME_MAX + 1];
     
-    sprintf(buffer, "%s/.xine/xine-ui_old_playlist.tox", xine_get_homedir());
+    snprintf(buffer, sizeof(buffer), "%s/.xine/xine-ui_old_playlist.tox", xine_get_homedir());
     mediamark_save_mediamarks(buffer);
   }
 
@@ -1713,4 +1890,6 @@ void gui_run(char **session_opts) {
   kbindings_free_kbinding(&gGui->kbindings);
 
   XCloseDisplay(gGui->display);
+  if( gGui->video_display != gGui->display )
+    XCloseDisplay(gGui->video_display);
 }
