@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2007 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: kbindings.c,v 1.126 2005/07/16 19:05:31 miguelfreitas Exp $
+ * $Id: kbindings.c,v 1.135 2007/04/09 22:39:35 fsck-p Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -83,10 +83,9 @@ typedef struct {
 static _kbedit_t    *kbedit = NULL;
 static char         *fontname = "-*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*";
 static char         *br_fontname = "-misc-fixed-medium-r-normal-*-10-*-*-*-*-*-*-*";
-#define FONT_HEIGHT_MODEL "azertyuiopqsdfghjklmwxcvbnAZERTYUIOPQSDFGHJKLMWXCVBN&(-_)=~#{[|`\\^@]}%"
 
-#define WINDOW_WIDTH        520
-#define WINDOW_HEIGHT       440
+#define WINDOW_WIDTH        530
+#define WINDOW_HEIGHT       456
 #define MAX_DISP_ENTRIES    11
 
 /*
@@ -368,6 +367,8 @@ static kbinding_entry_t default_binding_table[] = {
     "StreamInfosShow",        ACTID_STREAM_INFOS            , "i",        KEYMOD_META    , 0 , 1},
   { "display stream information using OSD",
     "OSDStreamInfos",         ACTID_OSD_SINFOS              , "i",        KEYMOD_CONTROL , 0 , 0},
+  { "display information using OSD",
+    "OSDWriteText",           ACTID_OSD_WTEXT               , "VOID",     KEYMOD_CONTROL , 0 , 0},
   { "show OSD menu",
     "OSDMenu",                ACTID_OSD_MENU                , "O",        KEYMOD_NOMOD   , 0 , 0},
   { "enter key binding editor",
@@ -448,72 +449,80 @@ static kbinding_entry_t default_binding_table[] = {
     "Quit",                   ACTID_QUIT                    , "q",        KEYMOD_NOMOD   , 0 , 0},
 #ifdef ENABLE_VDR_KEYS
   { "VDR Red button",
-    "VDRButtonRed",           ACTID_EVENT_VDR_RED           , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+    "VDRButtonRed",           ACTID_EVENT_VDR_RED            , "VOID",     KEYMOD_NOMOD   , 0 , 0},
   { "VDR Green button",
-    "VDRButtonGreen",         ACTID_EVENT_VDR_GREEN         , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+    "VDRButtonGreen",         ACTID_EVENT_VDR_GREEN          , "VOID",     KEYMOD_NOMOD   , 0 , 0},
   { "VDR Yellow button",
-    "VDRButtonYellow",        ACTID_EVENT_VDR_YELLOW        , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+    "VDRButtonYellow",        ACTID_EVENT_VDR_YELLOW         , "VOID",     KEYMOD_NOMOD   , 0 , 0},
   { "VDR Blue button",
-    "VDRButtonBlue",          ACTID_EVENT_VDR_BLUE          , "VOID",     KEYMOD_NOMOD   , 0 , 0},
-  { "VDR play",
-    "VDRPlay",                ACTID_EVENT_VDR_PLAY          , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+    "VDRButtonBlue",          ACTID_EVENT_VDR_BLUE           , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+  { "VDR Play",
+    "VDRPlay",                ACTID_EVENT_VDR_PLAY           , "VOID",     KEYMOD_NOMOD   , 0 , 0},
   { "VDR Pause",
-    "VDRPause",               ACTID_EVENT_VDR_PAUSE         , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+    "VDRPause",               ACTID_EVENT_VDR_PAUSE          , "VOID",     KEYMOD_NOMOD   , 0 , 0},
   { "VDR Stop",
-    "VDRStop",                ACTID_EVENT_VDR_STOP          , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+    "VDRStop",                ACTID_EVENT_VDR_STOP           , "VOID",     KEYMOD_NOMOD   , 0 , 0},
   { "VDR Record",
-    "VDRRecord",              ACTID_EVENT_VDR_RECORD        , "VOID",     KEYMOD_NOMOD   , 0 , 0},
-  { "VDR Fast Forward",
-    "VDRFastFwd",             ACTID_EVENT_VDR_FASTFWD       , "VOID",     KEYMOD_NOMOD   , 0 , 0},
-  { "VDR Fast Rewind",
-    "VDRFastRew",             ACTID_EVENT_VDR_FASTREW       , "VOID",     KEYMOD_NOMOD   , 0 , 0},
-  { "VDR power",
-    "VDRPower",               ACTID_EVENT_VDR_POWER         , "VOID",     KEYMOD_NOMOD   , 0 , 0},
-  { "VDR next channel",
-    "VDRChannelPlus",         ACTID_EVENT_VDR_CHANNELPLUS   , "VOID",     KEYMOD_NOMOD   , 0 , 0},
-  { "VDR previous channel",
-    "VDRChannelMinus",        ACTID_EVENT_VDR_CHANNELMINUS  , "VOID",     KEYMOD_NOMOD   , 0 , 0},
-  { "VDR schedule menu",
-    "VDRSchedule",            ACTID_EVENT_VDR_SCHEDULE      , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+    "VDRRecord",              ACTID_EVENT_VDR_RECORD         , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+  { "VDR Fast forward",
+    "VDRFastFwd",             ACTID_EVENT_VDR_FASTFWD        , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+  { "VDR Fast rewind",
+    "VDRFastRew",             ACTID_EVENT_VDR_FASTREW        , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+  { "VDR Power",
+    "VDRPower",               ACTID_EVENT_VDR_POWER          , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+  { "VDR Channel +",
+    "VDRChannelPlus",         ACTID_EVENT_VDR_CHANNELPLUS    , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+  { "VDR Channel -",
+    "VDRChannelMinus",        ACTID_EVENT_VDR_CHANNELMINUS   , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+  { "VDR Schedule menu",
+    "VDRSchedule",            ACTID_EVENT_VDR_SCHEDULE       , "VOID",     KEYMOD_NOMOD   , 0 , 0},
   { "VDR Channel menu",
-    "VDRChannels",            ACTID_EVENT_VDR_CHANNELS      , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+    "VDRChannels",            ACTID_EVENT_VDR_CHANNELS       , "VOID",     KEYMOD_NOMOD   , 0 , 0},
   { "VDR Timers menu",
-    "VDRTimers",              ACTID_EVENT_VDR_TIMERS        , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+    "VDRTimers",              ACTID_EVENT_VDR_TIMERS         , "VOID",     KEYMOD_NOMOD   , 0 , 0},
   { "VDR Recordings menu",
-    "VDRRecordings",          ACTID_EVENT_VDR_RECORDINGS    , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+    "VDRRecordings",          ACTID_EVENT_VDR_RECORDINGS     , "VOID",     KEYMOD_NOMOD   , 0 , 0},
   { "VDR Setup menu",
-    "VDRSetup",               ACTID_EVENT_VDR_SETUP         , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+    "VDRSetup",               ACTID_EVENT_VDR_SETUP          , "VOID",     KEYMOD_NOMOD   , 0 , 0},
   { "VDR Command menu",
-    "VDRCommands",            ACTID_EVENT_VDR_COMMANDS      , "VOID",     KEYMOD_NOMOD   , 0 , 0},
-  { "VDR Command Back",
-    "VDRBack",                ACTID_EVENT_VDR_BACK          , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+    "VDRCommands",            ACTID_EVENT_VDR_COMMANDS       , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+  { "VDR Command back",
+    "VDRBack",                ACTID_EVENT_VDR_BACK           , "VOID",     KEYMOD_NOMOD   , 0 , 0},
   { "VDR User command 1",
-    "VDRUser1",               ACTID_EVENT_VDR_USER1         , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+    "VDRUser1",               ACTID_EVENT_VDR_USER1          , "VOID",     KEYMOD_NOMOD   , 0 , 0},
   { "VDR User command 2",
-    "VDRUser2",               ACTID_EVENT_VDR_USER2         , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+    "VDRUser2",               ACTID_EVENT_VDR_USER2          , "VOID",     KEYMOD_NOMOD   , 0 , 0},
   { "VDR User command 3",
-    "VDRUser3",               ACTID_EVENT_VDR_USER3         , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+    "VDRUser3",               ACTID_EVENT_VDR_USER3          , "VOID",     KEYMOD_NOMOD   , 0 , 0},
   { "VDR User command 4",
-    "VDRUser4",               ACTID_EVENT_VDR_USER4         , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+    "VDRUser4",               ACTID_EVENT_VDR_USER4          , "VOID",     KEYMOD_NOMOD   , 0 , 0},
   { "VDR User command 5",
-    "VDRUser5",               ACTID_EVENT_VDR_USER5         , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+    "VDRUser5",               ACTID_EVENT_VDR_USER5          , "VOID",     KEYMOD_NOMOD   , 0 , 0},
   { "VDR User command 6",
-    "VDRUser6",               ACTID_EVENT_VDR_USER6         , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+    "VDRUser6",               ACTID_EVENT_VDR_USER6          , "VOID",     KEYMOD_NOMOD   , 0 , 0},
   { "VDR User command 7",
-    "VDRUser7",               ACTID_EVENT_VDR_USER7         , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+    "VDRUser7",               ACTID_EVENT_VDR_USER7          , "VOID",     KEYMOD_NOMOD   , 0 , 0},
   { "VDR User command 8",
-    "VDRUser8",               ACTID_EVENT_VDR_USER8         , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+    "VDRUser8",               ACTID_EVENT_VDR_USER8          , "VOID",     KEYMOD_NOMOD   , 0 , 0},
   { "VDR User command 9",
-    "VDRUser9",               ACTID_EVENT_VDR_USER9         , "VOID",     KEYMOD_NOMOD   , 0 , 0},
-  { "VDR Volume plus",
-    "VDRVolumePlus",          ACTID_EVENT_VDR_VOLPLUS       , "VOID",     KEYMOD_NOMOD   , 0 , 0},
-  { "VDR Volume minus",
-    "VDRVolumeMinus",         ACTID_EVENT_VDR_VOLMINUS      , "VOID",     KEYMOD_NOMOD   , 0 , 0},
-  { "VDR Audio Mute",
-    "VDRMute",                ACTID_EVENT_VDR_MUTE          , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+    "VDRUser9",               ACTID_EVENT_VDR_USER9          , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+  { "VDR Volume +",
+    "VDRVolumePlus",          ACTID_EVENT_VDR_VOLPLUS        , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+  { "VDR Volume -",
+    "VDRVolumeMinus",         ACTID_EVENT_VDR_VOLMINUS       , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+  { "VDR Mute audio",
+    "VDRMute",                ACTID_EVENT_VDR_MUTE           , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+  { "VDR Audio menu",
+    "VDRAudio",               ACTID_EVENT_VDR_AUDIO          , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+  { "VDR Command info",
+    "VDRInfo",                ACTID_EVENT_VDR_INFO           , "VOID",     KEYMOD_NOMOD   , 0 , 0},
+#ifdef XINE_EVENT_VDR_CHANNELPREVIOUS /* #ifdef is precaution for backward compatibility at the moment */
+  { "VDR Previous channel",
+    "VDRChannelPrevious",     ACTID_EVENT_VDR_CHANNELPREVIOUS, "VOID",     KEYMOD_NOMOD   , 0 , 0},
+#endif
 #endif
   { 0,
-    0,                        0,                              0,          0              , 0 , 0}
+    0,                        0                              , 0,          0              , 0 , 0}
 };
 
 static int _kbinding_get_is_gui_from_default(char *action) {
@@ -601,7 +610,7 @@ static void _kbindings_check_redundancy(kbinding_t *kbt) {
 	  else {
 	    len += 2;
 	    kmsg = (char *) realloc(kmsg, strlen(kmsg) + len + 1);
-	    sprintf(kmsg, "%s%s%s%c%s%c%s", kmsg, ", ", action1, ' ', dna, ' ', action2);
+	    sprintf(kmsg+strlen(kmsg), "%s%s%c%s%c%s", ", ", action1, ' ', dna, ' ', action2);
 	  }
 	}
       }
@@ -613,7 +622,7 @@ static void _kbindings_check_redundancy(kbinding_t *kbt) {
     char          *footer = _(".\n\nWhat do you want to do ?\n");
     
     kmsg = (char *) realloc(kmsg, strlen(kmsg) + strlen(footer) + 1);
-    sprintf(kmsg, "%s%s", kmsg, footer);
+    strlcat(kmsg, footer, strlen(kmsg) + strlen(footer) + 1);
 
     dump_error(gGui->verbosity, kmsg);
 
@@ -622,7 +631,7 @@ static void _kbindings_check_redundancy(kbinding_t *kbt) {
 						     _("Reset"), _("Editor"), _("Cancel"),
 						     _kbinding_reset_cb, _kbinding_editor_cb, NULL,
 						     (void *) kbt, 450, ALIGN_CENTER,
-						     kmsg);
+						     "%s", kmsg);
     free(kmsg);
     XLockDisplay(gGui->display);
     if(!gGui->use_root_window && gGui->video_display == gGui->display)
@@ -1008,21 +1017,20 @@ static void _kbindings_display_kbindings_to_stream(kbinding_t *kbt, int mode, FI
 
   switch(mode) {
   case LIRC_DISPLAY_MODE:
-    fprintf(stream, "##\n# xine key bindings.\n");
-    fprintf(stream, "# Automatically generated by %s version %s.\n##\n\n", PACKAGE, VERSION);
+    fprintf(stream, "##\n# xine key bindings.\n"
+		    "# Automatically generated by %s version %s.\n##\n\n", PACKAGE, VERSION);
     
     for(k = kbt->entry, i = 0; k[i]->action != NULL; i++) {
       if(!k[i]->is_alias) {
-	sprintf(buf, "# %s\n", k[i]->comment);
-	sprintf(buf, "%sbegin\n", buf);
-	sprintf(buf, "%s\tremote = xxxxx\n", buf);
-	sprintf(buf, "%s\tbutton = xxxxx\n", buf);
-	sprintf(buf, "%s\tprog   = xine\n", buf);
-	sprintf(buf, "%s\trepeat = 0\n", buf);
-	sprintf(buf, "%s\tconfig = %s\n", buf, k[i]->action);
-	sprintf(buf, "%send\n\n", buf);
+	snprintf(buf, sizeof(buf), "# %s\n", k[i]->comment);
+	strlcat(buf, "begin\n"
+		     "\tremote = xxxxx\n"
+		     "\tbutton = xxxxx\n"
+		     "\tprog   = xine\n"
+		     "\trepeat = 0\n", sizeof(buf));
+	snprintf(buf+strlen(buf), sizeof(buf)-strlen(buf), "\tconfig = %s\n", k[i]->action);
+	strlcat(buf, "end\n\n", sizeof(buf));
 	fputs(buf, stream);
-	memset(&buf, 0, sizeof(buf));
       }
     }
     fprintf(stream, "##\n# End of xine key bindings.\n##\n");
@@ -1030,38 +1038,35 @@ static void _kbindings_display_kbindings_to_stream(kbinding_t *kbt, int mode, FI
     
   case DEFAULT_DISPLAY_MODE:
   default:
-    fprintf(stream, "##\n# xine key bindings.\n");
-    fprintf(stream, "# Automatically generated by %s version %s.\n##\n\n", PACKAGE, VERSION);
+    fprintf(stream, "##\n# xine key bindings.\n"
+		    "# Automatically generated by %s version %s.\n##\n\n", PACKAGE, VERSION);
 
     for(k = kbt->entry, i = 0; k[i]->action != NULL; i++) {
       snprintf(buf, sizeof(buf), "# %s\n", k[i]->comment);
 
       if(k[i]->is_alias) {
-	sprintf(buf, "%sAlias {\n", buf);
-	sprintf(buf, "%s\tentry = %s\n", buf, k[i]->action);
+	sprintf(buf+strlen(buf), "Alias {\n\tentry = %s\n", k[i]->action);
       }
       else
-	sprintf(buf, "%s%s {\n", buf, k[i]->action);
+	sprintf(buf+strlen(buf), "%s {\n", k[i]->action);
 
-      sprintf(buf, "%s\tkey = %s\n", buf, k[i]->key);
-      sprintf(buf, "%s%s", buf, "\tmodifier = ");
+      sprintf(buf+strlen(buf), "\tkey = %s\n\tmodifier = ", k[i]->key);
       if(k[i]->modifier == KEYMOD_NOMOD)
-	sprintf(buf, "%s%s, ", buf, "none");
+	strlcat(buf, "none, ", sizeof(buf));
       if(k[i]->modifier & KEYMOD_CONTROL)
-	sprintf(buf, "%s%s, ", buf, "control");
+	strlcat(buf, "control, ", sizeof(buf));
       if(k[i]->modifier & KEYMOD_META)
-	sprintf(buf, "%s%s, ", buf, "meta");
+	strlcat(buf, "meta, ", sizeof(buf));
       if(k[i]->modifier & KEYMOD_MOD3)
-	sprintf(buf, "%s%s, ", buf, "mod3");
+	strlcat(buf, "mod3, ", sizeof(buf));
       if(k[i]->modifier & KEYMOD_MOD4)
-	sprintf(buf, "%s%s, ", buf, "mod4");
+	strlcat(buf, "mod4, ", sizeof(buf));
       if(k[i]->modifier & KEYMOD_MOD5)
-	sprintf(buf, "%s%s, ", buf, "mod5");
+	strlcat(buf, "mod5, ", sizeof(buf));
       buf[strlen(buf) - 2] = '\n';
       buf[strlen(buf) - 1] = '\0';
-      sprintf(buf, "%s%s", buf, "}\n\n");
-      fprintf(stream, "%s", buf);
-      memset(&buf, 0, sizeof(buf));
+      strlcat(buf, "}\n\n", sizeof(buf));
+      fputs(buf, stream);
     }
     fprintf(stream, "##\n# End of xine key bindings.\n##\n");
     break;
@@ -1270,34 +1275,34 @@ static char *_kbindings_get_shortcut_from_kbe(kbinding_entry_t *kbe) {
   static char shortcut[32];
 
   if(kbe) {
-    memset(&shortcut, 0, sizeof(shortcut));
+    shortcut[0] = 0;
     
     if(gGui->shortcut_style == 0) {
       if(kbe->modifier & KEYMOD_CONTROL)
-	sprintf(shortcut, "%s%s+", shortcut, _("Ctrl"));
+	snprintf(shortcut+strlen(shortcut), sizeof(shortcut)-strlen(shortcut), "%s+", _("Ctrl"));
       if(kbe->modifier & KEYMOD_META)
-	sprintf(shortcut, "%s%s+", shortcut, _("Alt"));
+	snprintf(shortcut+strlen(shortcut), sizeof(shortcut)-strlen(shortcut), "%s+", _("Alt"));
       if(kbe->modifier & KEYMOD_MOD3)
-	sprintf(shortcut, "%s%s+", shortcut, "M3");
+	strlcat(shortcut, "M3+", sizeof(shortcut));
       if(kbe->modifier & KEYMOD_MOD4)
-	sprintf(shortcut, "%s%s+", shortcut, "M4");
+	strlcat(shortcut, "M4+", sizeof(shortcut));
       if(kbe->modifier & KEYMOD_MOD5)
-	sprintf(shortcut, "%s%s+", shortcut, "M5");
+	strlcat(shortcut, "M5+", sizeof(shortcut));
     }
     else {
       if(kbe->modifier & KEYMOD_CONTROL)
-	sprintf(shortcut, "%s%s-", shortcut, "C");
+	strlcat(shortcut, "C-", sizeof(shortcut));
       if(kbe->modifier & KEYMOD_META)
-	sprintf(shortcut, "%s%s-", shortcut, "M");
+	strlcat(shortcut, "M-", sizeof(shortcut));
       if(kbe->modifier & KEYMOD_MOD3)
-	sprintf(shortcut, "%s%s-", shortcut, "M3");
+	strlcat(shortcut, "M3-", sizeof(shortcut));
       if(kbe->modifier & KEYMOD_MOD4)
-	sprintf(shortcut, "%s%s-", shortcut, "M4");
+	strlcat(shortcut, "M4-", sizeof(shortcut));
       if(kbe->modifier & KEYMOD_MOD5)
-	sprintf(shortcut, "%s%s-", shortcut, "M5");
+	strlcat(shortcut, "M5-", sizeof(shortcut));
     }
     
-    sprintf(shortcut, "%s%s", shortcut, kbe->key);
+    strlcat(shortcut, kbe->key, sizeof(shortcut));
     
     return shortcut;
   }
@@ -1536,8 +1541,6 @@ static void kbedit_create_browser_entries(void) {
     char *sc = _kbindings_get_shortcut_from_kbe(kbedit->kbt->entry[i]);
     char  shortcut[32];
         
-    memset(&buf, 0, sizeof(buf));
-    memset(&shortcut, 0, sizeof(shortcut));
     snprintf(shortcut, sizeof(shortcut), "%c%s%c", '[', (sc ? sc : "VOID"), ']');
     
     if(kbedit->kbt->entry[i]->is_alias)
@@ -1848,7 +1851,7 @@ static void kbedit_accept_yes(xitk_widget_t *w, void *data, int state) {
     
   case KBEDIT_EDITING:
     kbedit->ksel->key = (char *) realloc(kbedit->ksel->key, sizeof(char *) * (strlen(kbe->key) + 1));
-    sprintf(kbedit->ksel->key, "%s", kbe->key);
+    strcpy(kbedit->ksel->key, kbe->key);
     kbedit->ksel->modifier = kbe->modifier;
     
     kbedit_create_browser_entries();
@@ -1918,12 +1921,12 @@ static void kbedit_grab(xitk_widget_t *w, void *data) {
   {
     int x, y, w, h;
 
-    
     xitk_get_window_position(gGui->display, 
 			     (xitk_window_get_window(kbedit->xwin)), &x, &y, &w, &h);
         
     xwin = xitk_window_create_dialog_window(gGui->imlib_data, 
-					    _("Event receiver window"), x, y, w, h);
+					    _("Event Receiver Window:  Press keyboard keys to bind..."),
+					    x, y, w, h);
 
     set_window_states_start((xitk_window_get_window(xwin)));
   }
@@ -1936,10 +1939,11 @@ static void kbedit_grab(xitk_widget_t *w, void *data) {
   try_to_set_input_focus(xitk_window_get_window(xwin));
 
   do {
-    XMaskEvent(gGui->display, ButtonReleaseMask | KeyReleaseMask, &xev);
-    XMapRaised(gGui->display, (xitk_window_get_window(xwin)));
+    /* Although only release events are evaluated, we must also grab the corresponding press */
+    /* events to hide them from the other GUI windows and prevent unexpected side effects.   */
+    XMaskEvent(gGui->display, ButtonPressMask | ButtonReleaseMask | KeyPressMask | KeyReleaseMask, &xev);
   } while ((xev.type != KeyRelease && xev.type != ButtonRelease) ||
-	   xev.xmap.event != xitk_window_get_window(xwin));
+	   xev.xany.window != xitk_window_get_window(xwin));
   
   (void) xitk_get_key_modifier(&xev, &mod);
   kbindings_convert_modifier(mod, &modifier);
@@ -1970,21 +1974,20 @@ static void kbedit_grab(xitk_widget_t *w, void *data) {
     xine_strdupa(action, (xitk_label_get_label(kbedit->comment)));
     kbedit_display_kbinding(action, kbe);
     
-    memset(&shortcut, 0, sizeof(shortcut));
     snprintf(shortcut, sizeof(shortcut), "%c%s%c", '[', _kbindings_get_shortcut_from_kbe(kbe), ']');
     
-    /* Ask if uses want/wont store new shorcut */
+    /* Ask if user wants to store new shortcut */
     xitk_window_dialog_yesno_with_width(gGui->imlib_data, _("Accept?"),
 					kbedit_accept_yes, 
 					kbedit_accept_no, 
 					(void *) kbe, 400, ALIGN_CENTER,
-					_("Store %s as\n'%s' key binding ?."),
+					_("Store %s as\n'%s' key binding ?"),
 					shortcut, kbedit->ksel->comment);
   }
   else {
     /* error, redundant */
     if(redundant >= 0) {
-      xine_error(_("This key bindings is redundant with action:\n\"%s\".\n"),
+      xine_error(_("This key binding is redundant with action:\n\"%s\".\n"),
 		 kbedit->kbt->entry[redundant]->comment);
     }
   }
@@ -2004,6 +2007,8 @@ static void kbedit_handle_event(XEvent *event, void *data) {
   case KeyPress:
     if(xitk_get_key_pressed(event) == XK_Escape)
       kbedit_exit(NULL, NULL);
+    else
+      gui_handle_event(event, data);
     break;
 
   case KeyRelease: {
@@ -2049,7 +2054,7 @@ void kbedit_reparent(void) {
  *
  */
 void kbedit_window(void) {
-  int                        x, y;
+  int                        x, y, y1;
   GC                         gc;
   xitk_pixmap_t             *bg;
   xitk_labelbutton_widget_t  lb;
@@ -2057,19 +2062,19 @@ void kbedit_window(void) {
   xitk_browser_widget_t      br;
   xitk_checkbox_widget_t     cb;
   int                        btnw = 80;
-  int                        lbear, rbear, wid, asc, des;
+  int                        fontheight;
   xitk_font_t               *fs;
   xitk_widget_t             *w;
   
   x = xine_config_register_num(gGui->xine, "gui.kbedit_x", 
-			       100, 
+			       80,
 			       CONFIG_NO_DESC,
 			       CONFIG_NO_HELP,
 			       CONFIG_LEVEL_DEB,
 			       CONFIG_NO_CB,
 			       CONFIG_NO_DATA);
   y = xine_config_register_num(gGui->xine, "gui.kbedit_y",
-			       100,
+			       80,
 			       CONFIG_NO_DESC,
 			       CONFIG_NO_HELP,
 			       CONFIG_LEVEL_DEB,
@@ -2081,7 +2086,7 @@ void kbedit_window(void) {
   kbedit->kbt           = _kbindings_duplicate_kbindings(gGui->kbindings);
   kbedit->action_wanted = KBEDIT_NOOP;
   kbedit->xwin          = xitk_window_create_dialog_window(gGui->imlib_data,
-							   _("key binding editor"), 
+							   _("Key Binding Editor"),
 							   x, y, WINDOW_WIDTH, WINDOW_HEIGHT);
   set_window_states_start((xitk_window_get_window(kbedit->xwin)));
 
@@ -2105,18 +2110,19 @@ void kbedit_window(void) {
   XUnlockDisplay (gGui->display);
   
   x = 15;
-  y = 35;
+  y = 34;
   
   draw_rectangular_inner_box(gGui->imlib_data, bg, x, y,
-			     (WINDOW_WIDTH - 30), (20 * (MAX_DISP_ENTRIES + 1)) - 2);
+			     (WINDOW_WIDTH - 30 - 1), (MAX_DISP_ENTRIES * 20 + 16 + 10 - 1));
 
-  y = y + (20 * (MAX_DISP_ENTRIES + 1)) + 45;
+  y += MAX_DISP_ENTRIES * 20 + 16 + 10 + 30;
+  y1 = y; /* remember for later */
   draw_outter_frame(gGui->imlib_data, bg, 
-		    _("Binding action"), fontname, 
+		    _("Binding Action"), fontname,
 		    x, y, 
 		    (WINDOW_WIDTH - 30), 45);
 
-  y += 50;
+  y += 45 + 3;
   draw_outter_frame(gGui->imlib_data, bg, 
 		    _("Key"), fontname, 
 		    x, y, 
@@ -2135,7 +2141,7 @@ void kbedit_window(void) {
 
   kbedit_create_browser_entries();
 
-  y = 35;
+  y = 34;
 
   XITK_WIDGET_INIT(&br, gGui->imlib_data);
   
@@ -2154,8 +2160,8 @@ void kbedit_window(void) {
 			   (kbedit->browser = 
 			    xitk_noskin_browser_create(kbedit->widget_list, &br,
 						       (XITK_WIDGET_LIST_GC(kbedit->widget_list)),
-						       x + 1, y + 1, 
-						       WINDOW_WIDTH - 30 - 2 - 16, 20,
+						       x + 5, y + 5,
+						       WINDOW_WIDTH - (30 + 10 + 16), 20,
 						       16, br_fontname)));
   xitk_enable_and_show_widget(kbedit->browser);
 
@@ -2164,7 +2170,7 @@ void kbedit_window(void) {
 			   (const char* const*) kbedit->entries, 
 			   (const char* const*) kbedit->shortcuts, kbedit->num_entries, xitk_browser_get_current_start(kbedit->browser));
 
-  y = (WINDOW_HEIGHT - 160);
+  y = y1 - 30 + 4;
 
   XITK_WIDGET_INIT(&lb, gGui->imlib_data);
   
@@ -2181,7 +2187,7 @@ void kbedit_window(void) {
 						   "Black", "Black", "White", fontname)));
   xitk_enable_and_show_widget(kbedit->alias);
   
-  x += btnw + 2;
+  x += btnw + 4;
 
   lb.button_type       = RADIO_BUTTON;
   lb.label             = _("Edit");
@@ -2196,7 +2202,7 @@ void kbedit_window(void) {
 						   "Black", "Black", "White", fontname)));
   xitk_enable_and_show_widget(kbedit->edit);
   
-  x += btnw + 2;
+  x += btnw + 4;
 
   lb.button_type       = CLICK_BUTTON;
   lb.label             = _("Delete");
@@ -2211,7 +2217,7 @@ void kbedit_window(void) {
 						   "Black", "Black", "White", fontname)));
   xitk_enable_and_show_widget(kbedit->delete);
 
-  x += btnw + 2;
+  x += btnw + 4;
 
   lb.button_type       = CLICK_BUTTON;
   lb.label             = _("Save");
@@ -2226,7 +2232,7 @@ void kbedit_window(void) {
 						   "Black", "Black", "White", fontname)));
   xitk_enable_and_show_widget(kbedit->save);
 
-  x += btnw + 2;
+  x += btnw + 4;
 
   lb.button_type       = CLICK_BUTTON;
   lb.label             = _("Reset");
@@ -2240,7 +2246,7 @@ void kbedit_window(void) {
 				       "Black", "Black", "White", fontname)));
   xitk_enable_and_show_widget(w);
 
-  x += btnw + 2;
+  x += btnw + 4;
   
   lb.button_type       = CLICK_BUTTON;
   lb.label             = _("Done");
@@ -2256,29 +2262,30 @@ void kbedit_window(void) {
   xitk_enable_and_show_widget(kbedit->done);
 
   x = 15;
-  y += 30;
   
   XITK_WIDGET_INIT(&l, gGui->imlib_data);
 
   fs = xitk_font_load_font(gGui->display, fontname);
   xitk_font_set_font(fs, (XITK_WIDGET_LIST_GC(kbedit->widget_list)));
-  xitk_font_string_extent(fs, FONT_HEIGHT_MODEL, &lbear, &rbear, &wid, &asc, &des);
+  fontheight = xitk_font_get_string_height(fs, " ");
   xitk_font_unload_font(fs);
   
-  y += (asc+des) + 2;
+  y = y1 + (45 / 2);                /* Checkbox                     */
+  y1 = y - ((fontheight - 10) / 2); /* Text, v-centered to ckeckbox */
 
   l.window            = (XITK_WIDGET_LIST_WINDOW(kbedit->widget_list));
   l.gc                = (XITK_WIDGET_LIST_GC(kbedit->widget_list));
   l.skin_element_name = NULL;
-  l.label             = FONT_HEIGHT_MODEL;
+  l.label             = "Binding Action";
   l.callback          = NULL;
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(kbedit->widget_list)), 
-		   (kbedit->comment = 
-		    xitk_noskin_label_create(kbedit->widget_list, &l, x + 1 + 10, y, 
-					     WINDOW_WIDTH - 50 - 2, (asc+des), fontname)));
+			   (kbedit->comment =
+			    xitk_noskin_label_create(kbedit->widget_list, &l, x + 10, y1,
+						     WINDOW_WIDTH - 50 - 2, fontheight, fontname)));
   xitk_enable_and_show_widget(kbedit->comment);
 
-  y += 50;
+  y += 45 + 3;
+  y1 += 45 + 3;
 
   l.window            = (XITK_WIDGET_LIST_WINDOW(kbedit->widget_list));
   l.gc                = (XITK_WIDGET_LIST_GC(kbedit->widget_list));
@@ -2286,8 +2293,9 @@ void kbedit_window(void) {
   l.label             = "THE Key";
   l.callback          = NULL;
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(kbedit->widget_list)), 
-		   (kbedit->key = xitk_noskin_label_create(kbedit->widget_list, &l, x + 10, y, 
-							   100, (asc+des), fontname)));
+			   (kbedit->key =
+			    xitk_noskin_label_create(kbedit->widget_list, &l, x + 10, y1,
+						     100, fontheight, fontname)));
   xitk_enable_and_show_widget(kbedit->key);
   
   XITK_WIDGET_INIT(&cb, gGui->imlib_data);
@@ -2312,8 +2320,8 @@ void kbedit_window(void) {
   l.label             = _("ctrl");
   l.callback          = NULL;
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(kbedit->widget_list)), 
-   (w = xitk_noskin_label_create(kbedit->widget_list, &l, x, y - (((asc+des) - 10)>>1), 
-				 40, (asc+des), fontname)));
+   (w = xitk_noskin_label_create(kbedit->widget_list, &l, x, y1,
+				 40, fontheight, fontname)));
   xitk_enable_and_show_widget(w);
 
   x += 55;
@@ -2336,8 +2344,8 @@ void kbedit_window(void) {
   l.label             = _("meta");
   l.callback          = NULL;
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(kbedit->widget_list)), 
-   (w = xitk_noskin_label_create(kbedit->widget_list, &l, x, y - (((asc+des) - 10)>>1), 
-				 40, (asc+des), fontname)));
+   (w = xitk_noskin_label_create(kbedit->widget_list, &l, x, y1,
+				 40, fontheight, fontname)));
   xitk_enable_and_show_widget(w);
 
   x += 55;
@@ -2360,8 +2368,8 @@ void kbedit_window(void) {
   l.label             = _("mod3");
   l.callback          = NULL;
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(kbedit->widget_list)), 
-   (w = xitk_noskin_label_create(kbedit->widget_list, &l, x, y - (((asc+des) - 10)>>1), 
-				 40, (asc+des), fontname)));
+   (w = xitk_noskin_label_create(kbedit->widget_list, &l, x, y1,
+				 40, fontheight, fontname)));
   xitk_enable_and_show_widget(w);
 
   x += 55;
@@ -2384,8 +2392,8 @@ void kbedit_window(void) {
   l.label             = _("mod4");
   l.callback          = NULL;
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(kbedit->widget_list)), 
-   (w = xitk_noskin_label_create(kbedit->widget_list, &l, x, y - (((asc+des) - 10)>>1), 
-				 40, (asc+des), fontname)));
+   (w = xitk_noskin_label_create(kbedit->widget_list, &l, x, y1,
+				 40, fontheight, fontname)));
   xitk_enable_and_show_widget(w);
 
   x += 55;
@@ -2408,12 +2416,12 @@ void kbedit_window(void) {
   l.label             = _("mod5");
   l.callback          = NULL;
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(kbedit->widget_list)), 
-   (w = xitk_noskin_label_create(kbedit->widget_list, &l, x, y - (((asc+des) - 10)>>1), 
-				 40, (asc+des), fontname)));
+   (w = xitk_noskin_label_create(kbedit->widget_list, &l, x, y1,
+				 40, fontheight, fontname)));
   xitk_enable_and_show_widget(w);
 
   x = 15;
-  y += 30;
+  y = WINDOW_HEIGHT - (23 + 15);
 
   lb.button_type       = CLICK_BUTTON;
   lb.label             = _("Grab");
@@ -2455,7 +2463,7 @@ static void layer_above_video(Window w) {
 }
 static void raise_window(Window window, int visible, int running) {
 }
-static void xine_error(char *message, ...) {
+static void __attribute__ ((format (printf, 1, 2))) xine_error(char *message, ...) {
 }
 static void gui_execute_action_id(action_id_t actid) {
 }
@@ -2470,7 +2478,7 @@ int main(int argc, char **argv) {
   int   i;
 
   gGui              = (gGui_t *) xine_xmalloc(sizeof(gGui_t));
-  gGui->keymap_file = (char *) xine_xmalloc(XITK_PATH_MAX + XITK_NAME_MAX);
+  gGui->keymap_file = (char *) xine_xmalloc(XITK_PATH_MAX + XITK_NAME_MAX + 2);
   sprintf(gGui->keymap_file, "%s/%s/%s", xine_get_homedir(), ".xine", "keymap");
   
   if((gGui->kbindings = kbindings_init_kbinding())) {
@@ -2489,44 +2497,44 @@ int main(int argc, char **argv) {
 	memset(&buf, 0, sizeof(buf));
 	
 	if(k[i]->modifier & KEYMOD_CONTROL)
-	  sprintf(buf, "%s", "\\fBC\\fP");
+	  strcpy(buf, "\\fBC\\fP");
 	if(k[i]->modifier & KEYMOD_META) {
-	  if(strlen(buf))
-	    sprintf(buf, "%s%s", buf, "\\-");
-	  sprintf(buf, "%s%s", buf, "\\fBM\\fP");
+	  if(buf[0])
+	    strlcat(buf, "\\-", sizeof(buf));
+	  strlcat(buf, "\\fBM\\fP", sizeof(buf));
 	}
 	if(k[i]->modifier & KEYMOD_MOD3) {
-	  if(strlen(buf))
-	    sprintf(buf, "%s%s", buf, "\\-");
-	  sprintf(buf, "%s%s", buf, "\\fBM3\\fP");
+	  if(buf[0])
+	    strlcat(buf, "\\-", sizeof(buf));
+	  strlcat(buf, "\\fBM3\\fP", sizeof(buf));
 	}
 	if(k[i]->modifier & KEYMOD_MOD4) {
-	  if(strlen(buf))
-	    sprintf(buf, "%s%s", buf, "\\-");
-	  sprintf(buf, "%s%s", buf, "\\fBM4\\fP");
+	  if(buf[0])
+	    strlcat(buf, "\\-", sizeof(buf));
+	  strlcat(buf, "\\fBM4\\fP", sizeof(buf));
 	}
 	if(k[i]->modifier & KEYMOD_MOD5) {
-	  if(strlen(buf))
-	    sprintf(buf, "%s%s", buf, "\\-");
-	  sprintf(buf, "%s%s", buf, "\\fBM5\\fP");
+	  if(buf[0])
+	    strlcat(buf, "\\-", sizeof(buf));
+	  strlcat(buf, "\\fBM5\\fP", sizeof(buf));
 	}
 	printf("%s\\-", buf);
       }
       
-      printf("\\fB");
+      puts("\\fB");
       if(strlen(k[i]->key) > 1)
-	printf("\\<");
+	puts("\\<");
       
       if(!strncmp(k[i]->key, "KP_", 3))
 	printf("Keypad %s", (k[i]->key + 3));
       else
-	printf("%s", k[i]->key);
+	puts(k[i]->key);
       
       if(strlen(k[i]->key) > 1)
-	printf("\\>");
-      printf("\\fP");
+	puts("\\>");
+      puts("\\fP");
       
-      printf("\"\n");
+      puts("\"\n");
       
       printf("%c%s\n", (toupper(k[i]->comment[0])), (k[i]->comment + 1));
     }
