@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2007 the xine project
  *
  * This file is part of xine, a unix video player.
  * 
@@ -17,13 +17,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: main.c,v 1.308 2005/07/24 02:40:37 fsck-p Exp $
+ * $Id: main.c,v 1.332 2007/04/23 22:48:20 fsck-p Exp $
  *
  * xine main for X11
  *
  */
 /* required for getsubopt(); the __sun test gives us strncasecmp() on solaris */
-#ifndef __sun
+#if !defined(__sun) && ! defined(__FreeBSD__)
 #define _XOPEN_SOURCE 500
 #endif
 /* required for strncasecmp() */
@@ -38,6 +38,7 @@
 #include <stdarg.h>
 #include <unistd.h>
 #include <string.h>
+#include <strings.h>
 #include <math.h>
 #include <fcntl.h>
 #include <errno.h>
@@ -103,6 +104,7 @@ typedef struct {
 #define OPTION_LIST_PLUGINS     1012
 #define OPTION_BUG_REPORT       1013
 #define OPTION_NETWORK_PORT     1014
+#define OPTION_NO_MOUSE         1015
 
 
 /* options args */
@@ -116,7 +118,7 @@ static const char *short_options = "?hHgfvn"
 #ifdef HAVE_XF86VIDMODE
  "F"
 #endif
- "u:a:V:A:p::s:RG:BN:P:l::S:ZD::r:c:ET:I";
+ "u:a:V:A:p::s:RW:G:BN:P:l::S:ZD::r:c:ET:I";
 
 static struct option long_options[] = {
   {"help"           , no_argument      , 0, 'h'                      },
@@ -141,6 +143,7 @@ static struct option long_options[] = {
   {"network"        , no_argument      , 0, 'n'                      },
   {"network-port"   , required_argument, 0,  OPTION_NETWORK_PORT     },
   {"root"           , no_argument      , 0, 'R'                      },
+  {"wid"            , required_argument, 0, 'W'                      },
   {"geometry"       , required_argument, 0, 'G'                      },
   {"borderless"     , no_argument      , 0, 'B'                      },
   {"animation"      , required_argument, 0, 'N'                      },
@@ -159,6 +162,7 @@ static struct option long_options[] = {
   {"broadcast-port" , required_argument, 0, OPTION_BROADCAST_PORT    },
 #endif
   {"no-logo"        , no_argument      , 0, OPTION_NO_LOGO           },
+  {"no-mouse"       , no_argument      , 0, OPTION_NO_MOUSE          },
   {"no-reload"      , no_argument      , 0, 'E'                      },
   {"post"           , required_argument, 0, OPTION_POST              },
   {"disable-post"   , no_argument      , 0, OPTION_DISABLE_POST      },
@@ -342,22 +346,18 @@ static int parse_visual(VisualID *vid, int *vclass, char *visual_str) {
 
 static void xrm_parse(void) {
   Display      *display;
-  char          user_dbname[XITK_PATH_MAX + XITK_NAME_MAX + 1];
-  char          environement_buf[XITK_PATH_MAX + XITK_NAME_MAX + 1];
-  char          wide_dbname[XITK_PATH_MAX + XITK_NAME_MAX + 1];
+  char          user_dbname[XITK_PATH_MAX + XITK_NAME_MAX + 2];
+  char          environement_buf[XITK_PATH_MAX + XITK_NAME_MAX + 2];
+  char          wide_dbname[XITK_PATH_MAX + XITK_NAME_MAX + 2];
   char         *environment;
   char         *classname = "xine";
-  char         *str_type[20];
+  char         *str_type;
   XrmDatabase   rmdb, home_rmdb, server_rmdb, application_rmdb;
   XrmValue      value;
   
   XrmInitialize();
 
   rmdb = home_rmdb = server_rmdb = application_rmdb = NULL;
-  memset(&user_dbname, 0, sizeof(user_dbname));
-  memset(&environement_buf, 0, sizeof(environement_buf));
-  memset(&wide_dbname, 0, sizeof(wide_dbname));
-  memset(str_type, 0, sizeof(str_type));
 
   snprintf(wide_dbname, sizeof(wide_dbname), "%s%s", "/usr/lib/X11/app-defaults/", classname);
   
@@ -381,27 +381,27 @@ static void xrm_parse(void) {
     
     environment = environement_buf;
     snprintf(environement_buf, sizeof(environement_buf), "%s%s", (xine_get_homedir()), "/.Xdefaults-");
-    len = strlen(environment);
-    (void) gethostname(environment + len, (XITK_PATH_MAX + XITK_NAME_MAX) - len);
+    len = strlen(environement_buf);
+    (void) gethostname(environement_buf + len, sizeof(environement_buf) - len);
   }
   
   home_rmdb = XrmGetFileDatabase(environment);
   (void) XrmMergeDatabases(home_rmdb, &rmdb);
 
-  if(XrmGetResource(rmdb, "xine.geometry", "xine.Geometry", str_type, &value) == True) {
+  if(XrmGetResource(rmdb, "xine.geometry", "xine.Geometry", &str_type, &value) == True) {
     if(!parse_geometry(&window_attribute, (char *)value.addr))
       printf(_("Bad geometry '%s'\n"), (char *)value.addr);
   } 
-  if(XrmGetResource(rmdb, "xine.border", "xine.Border", str_type, &value) == True) {
+  if(XrmGetResource(rmdb, "xine.border", "xine.Border", &str_type, &value) == True) {
     window_attribute.borderless = !get_bool_value((char *) value.addr);
   }
-  if(XrmGetResource(rmdb, "xine.visual", "xine.Visual", str_type, &value) == True) {
+  if(XrmGetResource(rmdb, "xine.visual", "xine.Visual", &str_type, &value) == True) {
     if(!parse_visual(&gGui->prefered_visual_id, 
 		     &gGui->prefered_visual_class, (char *)value.addr)) {
       printf(_("Bad visual '%s'\n"), (char *)value.addr);
     }
   }
-  if(XrmGetResource(rmdb, "xine.colormap", "xine.Colormap", str_type, &value) == True) {
+  if(XrmGetResource(rmdb, "xine.colormap", "xine.Colormap", &str_type, &value) == True) {
     gGui->install_colormap = !get_bool_value((char *) value.addr);
   }
   
@@ -428,7 +428,7 @@ static void show_version(void) {
 #ifdef DEBUG
   printf("-[DEBUG]");
 #endif
-  printf(_(".\n(c) 2000-2004 The xine Team.\n"));
+  printf(_(".\n(c) 2000-2007 The xine Team.\n"));
 }
 
 /*
@@ -454,11 +454,11 @@ static void print_formatted(char *title, const char *const *plugins) {
   const char  *plugin;
   char         buffer[81];
   int          len;
-  char        *blanks = "     ";
+  static const char blanks[] = "     ";
 
-  printf(title);
+  printf("%s", title);
   
-  sprintf(buffer, "%s", blanks);
+  strcpy(buffer, blanks);
   plugin = *plugins++;
   
   while(plugin) {
@@ -466,21 +466,17 @@ static void print_formatted(char *title, const char *const *plugins) {
     len = strlen(buffer);
     
     if((len + (strlen(plugin) + 3)) < 80) {
-      sprintf(buffer, "%s%s%s", buffer, (strlen(buffer) == strlen(blanks)) ? "" : ", ", plugin);
+      snprintf(buffer+len, sizeof(buffer)-len, "%s%s", (len == sizeof(blanks)-1) ? "" : ", ", plugin);
     }
     else {
-      printf(buffer);
-      printf(",\n");
+      printf("%s,\n", buffer);
       snprintf(buffer, sizeof(buffer), "%s%s", blanks, plugin);
     }
     
     plugin = *plugins++;
   }
   
-  if(strlen(buffer))
-    printf(buffer);
-  
-  printf(".\n\n");
+  printf("%s.\n\n", buffer);
 }
 static void list_plugins(char *type) {
   typedef struct {
@@ -536,7 +532,7 @@ static void list_plugins(char *type) {
       i++;
     }
 
-    printf(_("No available plugins found of %s type!.\n"), type);
+    printf(_("No available plugins found of %s type!\n"), type);
   __found: ;
   }
   else {
@@ -583,7 +579,7 @@ static void show_usage (void) {
   printf("\n");
   printf(_("Usage: xine [OPTIONS]... [MRL]\n"));
   printf("\n");
-  printf("OPTIONS are:\n");
+  printf(_("OPTIONS are:\n"));
   printf(_("  -v, --version                Display version.\n"));
   printf(_("      --verbose [=level]       Set verbosity level. Default is 1.\n"));
   printf(_("  -c, --config <file>          Use config file instead of default one.\n"));
@@ -627,6 +623,7 @@ static void show_usage (void) {
 #endif
   printf(_("  -g, --hide-gui               hide GUI (panel, etc.)\n"));
   printf(_("  -I, --no-gui                 disable GUI\n"));
+  printf(_("      --no-mouse               disable mouse event\n"));
   printf(_("  -H, --hide-video             hide video window\n"));
 #ifdef HAVE_LIRC
   printf(_("  -L, --no-lirc                Turn off LIRC support.\n"));
@@ -643,6 +640,7 @@ static void show_usage (void) {
   printf(_("  -n, --network                Enable network remote control server.\n"));
   printf(_("      --network-port           Specify network server port number.\n"));
   printf(_("  -R, --root                   Use root window as video window.\n"));
+  printf(_("  -W, --wid                    Use a window id.\n"));
   printf(_("  -G, --geometry <WxH[+X+Y]>   Set output window geometry (X style).\n"));
   printf(_("  -B, --borderless             Borderless video output window.\n"));
   printf(_("  -N, --animation <mrl>        Specify mrl to play when video output isn't used.\n"
@@ -1018,7 +1016,10 @@ static void event_listener(void *user_data, const xine_event_t *event) {
   /* frontend can e.g. move on to next playlist entry */
   case XINE_EVENT_UI_PLAYBACK_FINISHED:
     if(event->stream == gGui->stream) {
-
+#ifdef XINE_PARAM_GAPLESS_SWITCH
+      if( xine_check_version(1,1,1) )
+        xine_set_param(gGui->stream, XINE_PARAM_GAPLESS_SWITCH, 1);
+#endif
       gui_playlist_start_next();
     }
     else if(event->stream == gGui->visual_anim.stream) {
@@ -1064,7 +1065,13 @@ static void event_listener(void *user_data, const xine_event_t *event) {
     if(event->stream == gGui->stream) {
       xine_ui_message_data_t *data = (xine_ui_message_data_t *) event->data;
       char                    buffer[8192];
-      void                    (*report)(char *message, ...) = xine_error_with_more;
+      void                    (*report)(char *message, ...)
+#if __GNUC__ >= 3
+				__attribute__ ((format (printf, 1, 2)))
+#endif
+      ;
+      
+      report = xine_error_with_more;
 
       memset(&buffer, 0, sizeof(buffer));
       
@@ -1107,9 +1114,9 @@ static void event_listener(void *user_data, const xine_event_t *event) {
 		 "%s:", _("*drum roll*, xine lib wants to take your attention "
 			  "to deliver an important message to you ;-):"));
 	if(data->explanation)
-	  sprintf(buffer, "%s %s %s", buffer, (char *) data + data->explanation, (char *) data + data->parameters);
+	  snprintf(buffer+strlen(buffer), sizeof(buffer)-strlen(buffer), " %s %s", (char *) data + data->explanation, (char *) data + data->parameters);
 	else
-	  sprintf(buffer, "%s %s", buffer, _("No Information available."));
+	  snprintf(buffer+strlen(buffer), sizeof(buffer)-strlen(buffer), " %s", _("No information available."));
 	  
 	break;
 
@@ -1118,14 +1125,14 @@ static void event_listener(void *user_data, const xine_event_t *event) {
 	snprintf(buffer, sizeof(buffer), "%s", _("The host you're trying to connect is unknown.\n"
 						 "Check the validity of the specified hostname."));
 	if(data->explanation)
-	  sprintf(buffer, "%s (%s)", buffer, (char *) data + data->parameters);
+	  snprintf(buffer+strlen(buffer), sizeof(buffer)-strlen(buffer), " (%s)", (char *) data + data->parameters);
 	break;
 	
 	/* (device name) */
       case XINE_MSG_UNKNOWN_DEVICE:
 	snprintf(buffer, sizeof(buffer), "%s", _("The device name you specified seems invalid."));
 	if(data->explanation)
-	  sprintf(buffer, "%s (%s)", buffer, (char *) data + data->parameters);
+	  snprintf(buffer+strlen(buffer), sizeof(buffer)-strlen(buffer), " (%s)", (char *) data + data->parameters);
 	break;
 
 	/* none */
@@ -1133,21 +1140,21 @@ static void event_listener(void *user_data, const xine_event_t *event) {
 	snprintf(buffer, sizeof(buffer), "%s", _("The network looks unreachable.\nCheck your network "
 						 "setup and/or the server name."));
 	if(data->explanation)
-	  sprintf(buffer, "%s (%s)", buffer, (char *) data + data->parameters);
+	  snprintf(buffer+strlen(buffer), sizeof(buffer)-strlen(buffer), " (%s)", (char *) data + data->parameters);
 	break;
 
 	/* (host name) */
       case XINE_MSG_CONNECTION_REFUSED:
 	snprintf(buffer, sizeof(buffer), "%s", _("The connection was refused.\nCheck the host name."));
 	if(data->explanation)
-	  sprintf(buffer, "%s (%s)", buffer, (char *) data + data->parameters);
+	  snprintf(buffer+strlen(buffer), sizeof(buffer)-strlen(buffer), " (%s)", (char *) data + data->parameters);
 	break;
 
 	/* (file name or mrl) */
       case XINE_MSG_FILE_NOT_FOUND:
-	snprintf(buffer, sizeof(buffer), "%s", _("The specified file or mrl is not found. Please check it twice."));
+	snprintf(buffer, sizeof(buffer), "%s", _("The specified file or MRL could not be found. Please check it twice."));
 	if(data->explanation)
-	  sprintf(buffer, "%s (%s)", buffer, (char *) data + data->parameters);
+	  snprintf(buffer+strlen(buffer), sizeof(buffer)-strlen(buffer), " (%s)", (char *) data + data->parameters);
 	break;
 
 	/* (device/file/mrl) */
@@ -1156,27 +1163,27 @@ static void event_listener(void *user_data, const xine_event_t *event) {
 						 "rights for this, or source doesn't contain data "
 						 "(e.g: not disc in drive)."));
 	if(data->explanation)
-	  sprintf(buffer, "%s (%s)", buffer, (char *) data + data->parameters);
+	  snprintf(buffer+strlen(buffer), sizeof(buffer)-strlen(buffer), " (%s)", (char *) data + data->parameters);
 	break;
 	
 	/* (library/decoder) */
       case XINE_MSG_LIBRARY_LOAD_ERROR:
-	snprintf(buffer, sizeof(buffer), "%s", _("A problem occur while loading a library or a decoder"));
+	snprintf(buffer, sizeof(buffer), "%s", _("A problem occurred while loading a library or a decoder"));
 	if(data->explanation)
-	  sprintf(buffer, "%s: %s", buffer, (char *) data + data->parameters);
+	  snprintf(buffer+strlen(buffer), sizeof(buffer)-strlen(buffer), ": %s", (char *) data + data->parameters);
 	break;
 
 	/* none */
       case XINE_MSG_ENCRYPTED_SOURCE:
 	snprintf(buffer, sizeof(buffer), "%s", _("The source seems encrypted, and can't be read."));
 	if(!strncasecmp(gGui->mmk.mrl, "dvd:/", 5)) {
-	  sprintf(buffer, "%s%s", buffer, _("\nYour DVD is probably crypted. "
-					    "According to your country laws, you can or can't "
-					    "install/use libdvdcss to be able to read this disc, "
-					    "which you bought."));
+	  strlcat(buffer, _("\nYour DVD is probably crypted. "
+			   "According to your country laws, you can or can't "
+			   "install/use libdvdcss to be able to read this disc, "
+			   "which you bought."), sizeof(buffer));
 	}
 	if(data->explanation)
-	  sprintf(buffer, "%s (%s)", buffer, (char *) data + data->parameters);
+	  snprintf(buffer+strlen(buffer), sizeof(buffer)-strlen(buffer), " (%s)", (char *) data + data->parameters);
 	break;
 
 	/* (warning message) */
@@ -1185,33 +1192,33 @@ static void event_listener(void *user_data, const xine_event_t *event) {
 	if(data->explanation)
 	  snprintf(buffer, sizeof(buffer), "%s %s", (char *) data + data->explanation, (char *) data + data->parameters);
 	else
-	  snprintf(buffer, sizeof(buffer), "%s", _("No Informations available."));
+	  snprintf(buffer, sizeof(buffer), "%s", _("No information available."));
 	break;
 
       case XINE_MSG_AUDIO_OUT_UNAVAILABLE:
 	gui_stop(NULL, NULL);
 	snprintf(buffer, sizeof(buffer), "%s", _("The audio device is unavailable. "
-						 "Please verify if another program already use it."));
+						 "Please verify if another program already uses it."));
 	break;
 
       default:
 	snprintf(buffer, sizeof(buffer), "%s", _("*sight*, unkown error."));
 	if(data->explanation)
-	  sprintf(buffer, "%s (%s %s)", buffer, (char *) data + data->explanation, (char *) data + data->parameters);
+	  snprintf(buffer+strlen(buffer), sizeof(buffer)-strlen(buffer), " (%s %s)", (char *) data + data->explanation, (char *) data + data->parameters);
 	break;
       }
       
       if(gGui->verbosity >= XINE_VERBOSITY_DEBUG) {
-	sprintf(buffer, "%s\n\n[", buffer);
+	strlcat(buffer, "\n\n[", sizeof(buffer));
 	
 	if(data->explanation)
-	  sprintf(buffer, "%s'%s' '%s'", buffer, (char *) data + data->explanation, (char *) data + data->parameters);
+	  snprintf(buffer+strlen(buffer), sizeof(buffer)-strlen(buffer), "'%s' '%s'", (char *) data + data->explanation, (char *) data + data->parameters);
 	
-	sprintf(buffer, "%s]", buffer);
+	strlcat(buffer, "]", sizeof(buffer));
       }
       
       if(strlen(buffer))
-	report(buffer);
+	report("%s", buffer);
       
     }
     break;
@@ -1220,21 +1227,36 @@ static void event_listener(void *user_data, const xine_event_t *event) {
   case XINE_EVENT_FRAME_FORMAT_CHANGE:
     break;
 
-    /* report current audio level (l/r) */
+    /* report current audio vol level (l/r/mute) */
   case XINE_EVENT_AUDIO_LEVEL:
     if(event->stream == gGui->stream) {
       xine_audio_level_data_t *aevent = (xine_audio_level_data_t *) event->data;
-      
-      gGui->mixer.volume_level = (aevent->left + aevent->right) / 2;
-      gGui->mixer.mute = aevent->mute;
 
-      if(gGui->mixer.method == SOUND_CARD_MIXER)
+      gGui->mixer.volume_level = (aevent->left + aevent->right) / 2;
+      if(gGui->mixer.method == SOUND_CARD_MIXER) {
+	gGui->mixer.mute = aevent->mute;
 	xitk_slider_set_pos(panel->mixer.slider, gGui->mixer.volume_level);
-      
-      xitk_checkbox_set_state(panel->mixer.mute, gGui->mixer.mute);
+	xitk_checkbox_set_state(panel->mixer.mute, gGui->mixer.mute);
+      }
     }
     break;
 
+#ifdef XINE_EVENT_AUDIO_AMP_LEVEL	/* Precaution for backward compatibility, will be removed some time */
+    /* report current audio amp level (l/r/mute) */
+  case XINE_EVENT_AUDIO_AMP_LEVEL:
+    if(event->stream == gGui->stream) {
+      xine_audio_level_data_t *aevent = (xine_audio_level_data_t *) event->data;
+
+      gGui->mixer.amp_level = (aevent->left + aevent->right) / 2;
+      if(gGui->mixer.method == SOFTWARE_MIXER) {
+	gGui->mixer.mute = aevent->mute;
+	xitk_slider_set_pos(panel->mixer.slider, gGui->mixer.amp_level);
+	xitk_checkbox_set_state(panel->mixer.mute, gGui->mixer.mute);
+      }
+    }
+    break;
+#endif
+
     /* last event sent when stream is disposed */
   case XINE_EVENT_QUIT:
     break;
@@ -1245,11 +1267,10 @@ static void event_listener(void *user_data, const xine_event_t *event) {
       xine_progress_data_t *pevent = (xine_progress_data_t *) event->data;
       char                  buffer[1024];
       
-      memset(&buffer, 0, sizeof(buffer));
       snprintf(buffer, sizeof(buffer), "%s [%d%%]\n", pevent->description, pevent->percent);
       gGui->mrl_overrided = 3;
       panel_set_title(buffer);
-      osd_display_info(buffer);
+      osd_display_info("%s", buffer);
     }
     break;
 
@@ -1308,6 +1329,7 @@ int main(int argc, char *argv[]) {
   int                     c = '?', aos    = 0;
   int                     option_index    = 0;
   int                     audio_channel   = -1;
+  int                     window_id       = 0;
   int                     spu_channel     = -1;
   char                   *audio_driver_id = NULL;
   char                   *video_driver_id = NULL;
@@ -1330,7 +1352,7 @@ int main(int argc, char *argv[]) {
   int                     session_argv_num = 0;
   int                     retval           = 0;
   
-#ifdef HAVE_SETLOCALE
+#ifdef ENABLE_NLS
   if((xitk_set_locale()) != NULL) {
     setlocale(LC_ALL, "");
     setlocale(LC_NUMERIC, "C");
@@ -1394,6 +1416,8 @@ int main(int argc, char *argv[]) {
   gGui->report                 = stdout;
   gGui->ssaver_enabled         = 1;
   gGui->no_gui                 = 0;
+  gGui->no_mouse               = 0;
+  gGui->wid                    = 0;
   gGui->nongui_error_msg       = NULL;
   
   window_attribute.x     = window_attribute.y      = -8192;
@@ -1420,6 +1444,7 @@ int main(int argc, char *argv[]) {
 			 long_options, &option_index)) != EOF) {
     switch(c) {
 
+
 #ifdef HAVE_LIRC
     case 'L': /* Disable LIRC support */
       gGui->lirc_enable = 0;
@@ -1572,7 +1597,7 @@ int main(int argc, char *argv[]) {
 	  char  *keymap_file = p + 5;
 	  
 	  if((gGui->keymap_file == NULL) && keymap_file && strlen(keymap_file)) {
-	    char  buffer[XITK_PATH_MAX + XITK_NAME_MAX + 1];
+	    char  buffer[XITK_PATH_MAX + XITK_NAME_MAX + 2];
 
 	    memset(&buffer, 0, sizeof(buffer));
 	    xitk_subst_special_chars(keymap_file, &buffer[0]);
@@ -1659,7 +1684,6 @@ int main(int argc, char *argv[]) {
 	gGui->playlist.loop = PLAYLIST_LOOP_LOOP;
       break;
 
-
     case OPTION_SK_SERVER:
       gGui->skin_server_url = strdup(optarg);
       break;
@@ -1689,6 +1713,10 @@ int main(int argc, char *argv[]) {
       gGui->display_logo = 0;
       break;
 
+    case OPTION_NO_MOUSE:
+      gGui->no_mouse = 1;
+      break;
+      
     case 'S':
       if(is_remote_running(((session >= 0) ? session : 0)))
 	retval = session_handle_subopt(optarg, &session);
@@ -1708,7 +1736,7 @@ int main(int argc, char *argv[]) {
     case 'c':
       {
 	char *cfg = xine_chomp(optarg);
-	char  buffer[XITK_PATH_MAX + XITK_NAME_MAX + 1];
+	char  buffer[XITK_PATH_MAX + XITK_NAME_MAX + 2];
 	
 	memset(&buffer, 0, sizeof(buffer));
 	xitk_subst_special_chars(cfg, &buffer[0]);
@@ -1778,9 +1806,11 @@ int main(int argc, char *argv[]) {
 	  printf(_("*** NOTE ***\n"));
 	  printf(_(" Bug Report mode: All output messages will be added in BUG-REPORT.TXT file.\n"));
 
-#ifdef HAVE_SETLOCALE
-	if((xitk_set_locale()) != NULL)
-	  setlocale(LC_ALL, "C");
+#ifdef ENABLE_NLS
+	if((xitk_set_locale()) != NULL) {
+	  setlocale(LC_ALL, "");
+          setlocale(LC_NUMERIC, "C");
+        }
 #endif
 
 	  if (dup2((fileno(f)), STDOUT_FILENO) < 0)
@@ -1814,10 +1844,20 @@ int main(int argc, char *argv[]) {
     case 'I':
       gGui->no_gui = 1;
       break;
-
+    case 'W': /* Select wid */
+      if(optarg != NULL) {
+        sscanf(optarg, "%i", &window_id);
+     	gGui->wid = window_id;
+      } 
+      else {
+	fprintf(stderr, _("window id required for -W option\n"));
+	exit(1);
+      }
+      break;
+      
     default:
       show_usage();
-      fprintf (stderr, _("invalid argument %d => exit\n"), c);
+      fprintf(stderr, _("invalid argument %d => exit\n"), c);
       exit(1);
     }
   }
@@ -1832,12 +1872,12 @@ int main(int argc, char *argv[]) {
 	for(i = optind; i < _argc; i++) {
 	  char enqueue_mrl[strlen(_argv[i]) + 256];
 
-	  sprintf(enqueue_mrl, "session=%d,mrl=%s", session, atoa(_argv[i]));
+	  snprintf(enqueue_mrl, sizeof(enqueue_mrl), "session=%d,mrl=%s", session, atoa(_argv[i]));
 	  (void) session_handle_subopt(enqueue_mrl, &session);
 	}
       }
       else
-	printf(_("You should specify one MRL to enqueue, at least!.\n"));
+	printf(_("You should specify at least one MRL to enqueue!\n"));
 
     }
 
@@ -1927,7 +1967,7 @@ int main(int argc, char *argv[]) {
 			      CONFIG_NO_DATA);
 
   if(old_playlist_cfg && (!(_argc - optind)) && (!no_old_playlist)) {
-    char buffer[XITK_PATH_MAX + XITK_NAME_MAX + 1];
+    char buffer[XITK_PATH_MAX + XITK_NAME_MAX + 2];
     
     snprintf(buffer, sizeof(buffer), "%s/.xine/xine-ui_old_playlist.tox", xine_get_homedir());
     mediamark_load_mediamarks(buffer);
@@ -1984,8 +2024,7 @@ int main(int argc, char *argv[]) {
     if(xine_config_lookup_entry(gGui->xine, "media.files.origin_path", &cfg_entry))
       snprintf(gGui->curdir, sizeof(gGui->curdir), "%s", cfg_entry.str_value);
     else
-      (void *) getcwd(&(gGui->curdir[0]), XITK_PATH_MAX);
-
+      getcwd(&(gGui->curdir[0]), XITK_PATH_MAX);
   }
 
   /*
@@ -2081,6 +2120,10 @@ int main(int argc, char *argv[]) {
   post_init();
 
   gGui->stream = xine_stream_new(gGui->xine, gGui->ao_port, gGui->vo_port);
+#ifdef XINE_PARAM_EARLY_FINISHED_EVENT
+  if( xine_check_version(1,1,1) )
+      xine_set_param(gGui->stream, XINE_PARAM_EARLY_FINISHED_EVENT, 1);
+#endif
 
   /* Store audio mixer level */
   if(gGui->ao_port && (gGui->mixer.method == SOUND_CARD_MIXER))
@@ -2097,7 +2140,7 @@ int main(int argc, char *argv[]) {
   gGui->logo_mode = 0;
   gGui->logo_has_changed = 0;
   gGui->logo_mrl = xine_config_register_string (gGui->xine, "gui.logo_mrl", XINE_LOGO_MRL,
-						_("Logo mrl"),
+						_("Logo MRL"),
 						CONFIG_NO_HELP, 
 						CONFIG_LEVEL_EXP,
 						main_change_logo_cb, 
@@ -2156,7 +2199,7 @@ int main(int argc, char *argv[]) {
     vpplugin_rewire_posts();
     applugin_rewire_posts();
   }
-  
+
   gui_run(session_argv);
 
   if(pplugins)
