@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2006 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: menus.c,v 1.62 2005/07/20 19:08:00 miguelfreitas Exp $
+ * $Id: menus.c,v 1.65 2006/11/06 00:23:33 fsck-p Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -439,6 +439,7 @@ void video_window_menu(xitk_widget_list_t *wl) {
   xitk_menu_widget_t   menu;
   char                 buffer[2048];
   xitk_widget_t       *w;
+  float                xmag, ymag;
 #ifdef HAVE_XINERAMA
   int                  fullscr_mode = (FULLSCR_MODE | FULLSCR_XI_MODE);
 #else
@@ -639,15 +640,18 @@ void video_window_menu(xitk_widget_list_t *wl) {
       menu_aspect, (void *) XINE_VO_ASPECT_DVB                                               },
     { _("Video/200%"),
       menu_get_shortcut("Window200"),
-      (video_window_get_mag() == 2.0) ? "<checked>" : "<check>",
+      (video_window_get_mag(&xmag, &ymag),
+        (xmag == 2.0f && ymag == 2.0f)) ? "<checked>" : "<check>",
       menu_video_ctrl, (void *) VIDEO_2X                                                     },
     { _("Video/100%"),
       menu_get_shortcut("Window100"),
-      (video_window_get_mag() == 1.0) ? "<checked>" : "<check>",
+      (video_window_get_mag(&xmag, &ymag),
+        (xmag == 1.0f && ymag == 1.0f)) ? "<checked>" : "<check>",
       menu_video_ctrl, (void *) VIDEO_1X                                                     },
     { _("Video/50%"),
       menu_get_shortcut("Window50"),
-      (video_window_get_mag() ==  .5) ? "<checked>" : "<check>",
+      (video_window_get_mag(&xmag, &ymag),
+        (xmag == 0.5f && ymag == 0.5f)) ? "<checked>" : "<check>",
       menu_video_ctrl, (void *) VIDEO__5X                                                    },
     { _("Video/SEP"),
       NULL,
@@ -793,7 +797,7 @@ void video_window_menu(xitk_widget_list_t *wl) {
   menu.parent_wlist      = wl;
   menu.skin_element_name = NULL;
   
-  w = xitk_noskin_menu_create(wl, &menu, x + 1, y + 1);
+  w = xitk_noskin_menu_create(wl, &menu, x, y);
 
   cur_entry = &menu_entries[0];
   while(cur_entry->menu) {
@@ -1040,7 +1044,6 @@ void audio_lang_menu(xitk_widget_list_t *wl, int x, int y) {
   xitk_widget_t       *w;
   xitk_menu_entry_t    menu_entries[] = {
     { NULL , NULL, "<title>",      NULL, NULL },
-    { "SEP", NULL, "<separator>",  NULL, NULL },
     { NULL,  NULL, NULL,           NULL, NULL }
   };
 
@@ -1116,7 +1119,6 @@ void spu_lang_menu(xitk_widget_list_t *wl, int x, int y) {
   xitk_widget_t       *w;
   xitk_menu_entry_t    menu_entries[] = {
     { NULL , NULL, "<title>",      NULL, NULL },
-    { "SEP", NULL, "<separator>",  NULL, NULL },
     { NULL,  NULL, NULL,           NULL, NULL }
   };
 
@@ -1191,7 +1193,6 @@ void playlist_menu(xitk_widget_list_t *wl, int x, int y, int selected) {
   char                 buffer[2048];
   xitk_menu_entry_t    menu_entries_nosel[] = {
     { NULL ,           NULL,          "<title>",     NULL,                         NULL                    },
-    { "SEP",           NULL,          "<separator>", NULL,                         NULL                    },
     { _("Scan"),       menu_get_shortcut("ScanPlaylistInfo"),
                                       NULL,          menu_scan_infos,              NULL                    },
     { _("Add"),        NULL,          NULL,          menu_open_mrlbrowser,         NULL                    },
@@ -1199,7 +1200,6 @@ void playlist_menu(xitk_widget_list_t *wl, int x, int y, int selected) {
   };
   xitk_menu_entry_t    menu_entries_sel[] = {
     { NULL ,           NULL,          "<title>",     NULL,                         NULL                    },
-    { "SEP",           NULL,          "<separator>", NULL,                         NULL                    },
     { _("Play"),       NULL,          NULL,          menu_playlist_play_current,   NULL                    },
     { "SEP",           NULL,          "<separator>", NULL,                         NULL                    },
     { _("Scan"),       NULL,          NULL,          menu_scan_infos_selected,     NULL                    },
@@ -1258,7 +1258,6 @@ void control_menu(xitk_widget_list_t *wl, int x, int y) {
   char                 buffer[2048];
   xitk_menu_entry_t    menu_entries[] = {
     { NULL ,           NULL,          "<title>",     NULL,                         NULL                    },
-    { "SEP",           NULL,          "<separator>", NULL,                         NULL                    },
     { _("Reset video settings"),   
                        NULL,          NULL,          menu_control_reset,           NULL                    },
     { NULL,            NULL,          NULL,          NULL,                         NULL                    }
