@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: menus.h,v 1.3 2003/07/14 22:44:09 f1rmb Exp $
+ * $Id: menus.h,v 1.6 2005/04/13 19:45:08 f1rmb Exp $
  *
  */
 #ifndef MENUS_H
@@ -28,5 +28,7 @@
 void video_window_menu(xitk_widget_list_t *wl);
 void audio_lang_menu(xitk_widget_list_t *wl, int x, int y);
 void spu_lang_menu(xitk_widget_list_t *wl, int x, int y);
+void playlist_menu(xitk_widget_list_t *wl, int x, int y, int selected);
+void control_menu(xitk_widget_list_t *wl, int x, int y);
 
 #endif
