@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: osd.c,v 1.32 2004/11/15 21:43:20 f1rmb Exp $
+ * $Id: osd.c,v 1.39 2006/07/15 20:37:29 dgp85 Exp $
  *
  */
 
@@ -30,6 +30,7 @@
 #include <string.h>
 #include <stdarg.h>
 #include <unistd.h>
+#include <pthread.h>
 
 #include "common.h"
 
@@ -118,6 +119,8 @@ static uint8_t trans[OVL_PALETTE_SIZE];
 #define FONT_SIZE          20
 #define UNSCALED_FONT_SIZE 24
 
+static pthread_mutex_t osd_mutex;
+
 static void  _xine_osd_show(xine_osd_t *osd, int64_t vpts) {
   if( gGui->osd.use_unscaled && gGui->osd.unscaled_available )
     xine_osd_show_unscaled(osd, vpts);
@@ -184,40 +187,76 @@ void osd_init(void) {
 
   gGui->osd.unscaled_available =
     (xine_osd_get_capabilities(gGui->osd.status.osd[0]) & XINE_OSD_CAP_UNSCALED );
+
+  pthread_mutex_init(&osd_mutex, NULL);
 }
 
-void osd_deinit(void) {
+void osd_hide_sinfo(void) {
 
+  pthread_mutex_lock(&osd_mutex);
   if(gGui->osd.sinfo.visible) {
     gGui->osd.sinfo.visible = 0;
     xine_osd_hide(gGui->osd.sinfo.osd[0], 0);
   }
+  pthread_mutex_unlock(&osd_mutex);
+}
 
+void osd_hide_bar(void) {
+
+  pthread_mutex_lock(&osd_mutex);
   if(gGui->osd.bar.visible) {
     gGui->osd.bar.visible = 0;
     xine_osd_hide(gGui->osd.bar.osd[0], 0);
     xine_osd_hide(gGui->osd.bar.osd[1], 0);
   } 
+  pthread_mutex_unlock(&osd_mutex);
+}
 
+void osd_hide_status(void) {
+
+  pthread_mutex_lock(&osd_mutex);
   if(gGui->osd.status.visible) {
     gGui->osd.status.visible = 0;
     xine_osd_hide(gGui->osd.status.osd[0], 0);
   } 
+  pthread_mutex_unlock(&osd_mutex);
+}
 
+void osd_hide_info(void) {
+
+  pthread_mutex_lock(&osd_mutex);
   if(gGui->osd.info.visible) {
     gGui->osd.info.visible = 0;
     xine_osd_hide(gGui->osd.info.osd[0], 0);
   } 
+  pthread_mutex_unlock(&osd_mutex);
+}
+
+void osd_hide(void) {
+
+  osd_hide_sinfo();
+  osd_hide_bar();
+  osd_hide_status();
+  osd_hide_info();
+}
+
+void osd_deinit(void) {
+
+  osd_hide();
 
   xine_osd_free(gGui->osd.sinfo.osd[0]);
   xine_osd_free(gGui->osd.bar.osd[0]);
   xine_osd_free(gGui->osd.bar.osd[1]);
   xine_osd_free(gGui->osd.status.osd[0]);
   xine_osd_free(gGui->osd.info.osd[0]);
+
+  pthread_mutex_destroy(&osd_mutex);
 }
 
 void osd_update(void) {
 
+  pthread_mutex_lock(&osd_mutex);
+
   if(gGui->osd.sinfo.visible) {
     gGui->osd.sinfo.visible--;
     if(!gGui->osd.sinfo.visible) {
@@ -247,6 +286,7 @@ void osd_update(void) {
     }
   }
 
+  pthread_mutex_unlock(&osd_mutex);
 }
 
 void osd_stream_infos(void) {
@@ -332,40 +372,38 @@ void osd_stream_infos(void) {
       y += h;
     }
     
-    memset(&buffer, 0, sizeof(buffer));
-
-    snprintf(buffer, sizeof(buffer), "%s", _("Audio: "));
+    strncpy(buffer, _("Audio: "), sizeof(buffer));
     len = strlen(buffer);
     switch(audiochannel) {
     case -2:
-      sprintf(buffer, "%s%s", buffer, "off");
+      strlcat(buffer, "off", sizeof(buffer));
       break;
     case -1:
       if(!xine_get_audio_lang (gGui->stream, audiochannel, &buffer[len]))
-	sprintf(buffer, "%s%s", buffer, "auto");
+	strlcat(buffer, "auto", sizeof(buffer));
       break;
     default:
       if(!xine_get_audio_lang (gGui->stream, audiochannel, &buffer[len]))
-	sprintf(buffer, "%s%3d", buffer, audiochannel);
+	snprintf(buffer+strlen(buffer), sizeof(buffer)-strlen(buffer), "%3d", audiochannel);
       break;
     }
 
-    sprintf(buffer, "%s%s", buffer, ", Spu: ");
+    strlcat(buffer, ", Spu: ", sizeof(buffer));
     len = strlen(buffer);
     switch (spuchannel) {
     case -2:
-      sprintf(buffer, "%s%s", buffer, "off");
+      strlcat(buffer, "off", sizeof(buffer));
       break;
     case -1:
       if(!xine_get_spu_lang (gGui->stream, spuchannel, &buffer[len]))
-	sprintf(buffer, "%s%s", buffer, "auto");
+	strlcat(buffer, "auto", sizeof(buffer));
       break;
     default:
       if(!xine_get_spu_lang (gGui->stream, spuchannel, &buffer[len]))
-        sprintf(buffer, "%s%3d", buffer, spuchannel);
+        snprintf(buffer+strlen(buffer), sizeof(buffer)-strlen(buffer), "%3d", spuchannel);
       break;
     }
-    sprintf(buffer, "%s.", buffer);
+    strlcat(buffer, ".", sizeof(buffer));
     xine_osd_draw_text(gGui->osd.sinfo.osd[0], x, y, buffer, XINE_OSD_TEXT1);
     xine_osd_get_text_size(gGui->osd.sinfo.osd[0], buffer, &w, &h);
     if(w > osdw)
@@ -384,12 +422,12 @@ void osd_stream_infos(void) {
       int totaldays;
       
       totaldays  = totaltime / (3600 * 24);
-      sprintf(buffer, "%s(%.0f%%) %s ", buffer, ((float)playedtime / (float)totaltime) * 100, _("of"));
+      sprintf(buffer+strlen(buffer), "(%.0f%%) %s ", ((float)playedtime / (float)totaltime) * 100, _("of"));
       
       if(totaldays > 0)
-	sprintf(buffer, "%s%d::%02d", buffer, totaldays, totaltime / 3600);
+	sprintf(buffer+strlen(buffer), "%d::%02d", totaldays, totaltime / 3600);
       else
-	sprintf(buffer, "%s%d:%02d:%02d", buffer, totaltime / 3600, (totaltime % 3600) / 60, totaltime % 60);
+	sprintf(buffer+strlen(buffer), "%d:%02d:%02d", totaltime / 3600, (totaltime % 3600) / 60, totaltime % 60);
     }
     
     xine_osd_draw_text(gGui->osd.sinfo.osd[0], x, y, buffer, XINE_OSD_TEXT1);
@@ -405,8 +443,10 @@ void osd_stream_infos(void) {
     gGui->osd.sinfo.y = 15;
     gGui->osd.sinfo.w = osdw;
 
+    pthread_mutex_lock(&osd_mutex);
     _xine_osd_show(gGui->osd.sinfo.osd[0], 0);
     gGui->osd.sinfo.visible = gGui->osd.timeout;
+    pthread_mutex_unlock(&osd_mutex);
   }
 }
 
@@ -517,11 +557,12 @@ void osd_draw_bar(char *title, int min, int max, int val, int type) {
      * it would look pretty bad.
      */
     if( wwidth > MINIMUM_WIN_WIDTH ) {
+      pthread_mutex_lock(&osd_mutex);
       _xine_osd_show(gGui->osd.bar.osd[0], 0);
       if(title)
         _xine_osd_show(gGui->osd.bar.osd[1], 0);
-   
       gGui->osd.bar.visible = gGui->osd.timeout;
+      pthread_mutex_unlock(&osd_mutex);
     }
   }
 }
@@ -532,7 +573,7 @@ void osd_stream_position(int pos) {
 
 void osd_display_info(char *info, ...) {
 
-  if(gGui->osd.enabled) {
+  if(gGui->osd.enabled && !gGui->on_quit) {
     va_list   args;
     char     *buf;
     int       n, size = 100;
@@ -562,8 +603,12 @@ void osd_display_info(char *info, ...) {
 
     xine_osd_draw_text(gGui->osd.info.osd[0], 0, 0, buf, XINE_OSD_TEXT1);
     xine_osd_set_position(gGui->osd.info.osd[0], 20, 10 + 30);
+
+    pthread_mutex_lock(&osd_mutex);
     _xine_osd_show(gGui->osd.info.osd[0], 0);
     gGui->osd.info.visible = gGui->osd.timeout;
+    pthread_mutex_unlock(&osd_mutex);
+
     SAFE_FREE(buf);
   }
 }
@@ -640,8 +685,10 @@ void osd_update_status(void) {
      * it would look pretty bad.
      */
     if( wwidth > MINIMUM_WIN_WIDTH ) {
+      pthread_mutex_lock(&osd_mutex);
       _xine_osd_show(gGui->osd.status.osd[0], 0);
       gGui->osd.status.visible = gGui->osd.timeout;
+      pthread_mutex_unlock(&osd_mutex);
     }
   }
 }
@@ -672,7 +719,7 @@ void osd_display_spu_lang(void) {
   }
   
   snprintf(buffer, sizeof(buffer), "%s%s", _("Subtitles: "), get_language_from_iso639_1(lang));
-  osd_display_info(buffer);
+  osd_display_info("%s", buffer);
 }
 
 void osd_display_audio_lang(void) {
@@ -701,16 +748,15 @@ void osd_display_audio_lang(void) {
   }
 
   snprintf(buffer, sizeof(buffer), "%s%s", _("Audio Channel: "), get_language_from_iso639_1(lang));
-  osd_display_info(buffer);
-}
-
-int osd_is_visible(void) {
-  return (gGui->osd.sinfo.visible || gGui->osd.bar.visible);
+  osd_display_info("%s", buffer);
 }
 
 void osd_update_osd(void) {
   int vwidth, vheight, wwidth, wheight;
   int x;
+
+  if(!gGui->osd.sinfo.visible && !gGui->osd.bar.visible)
+    return;
   
   vwidth  = xine_get_stream_info(gGui->stream, XINE_STREAM_INFO_VIDEO_WIDTH);
   vheight = xine_get_stream_info(gGui->stream, XINE_STREAM_INFO_VIDEO_HEIGHT);
@@ -729,6 +775,8 @@ void osd_update_osd(void) {
   
   _osd_get_output_size(&wwidth, &wheight);
   
+  pthread_mutex_lock(&osd_mutex);
+
   if(gGui->osd.sinfo.visible) {
     xine_osd_hide(gGui->osd.sinfo.osd[0], 0);
     
@@ -757,5 +805,5 @@ void osd_update_osd(void) {
 
   }
 
-
+  pthread_mutex_unlock(&osd_mutex);
 }
