@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: mediamarks.c,v 1.4 2005/02/15 20:09:28 mroi Exp $
+ * $Id: mediamarks.c,v 1.6 2006/06/19 13:56:50 dgp85 Exp $
  *
  * parsing mediamarks
  */
@@ -440,7 +440,7 @@ static int file_is_m3u(const char *mrl) {
 static void parse_m3u(const char *mrl, list_t *items) {
   FILE *file;
   char **line;
-  int *n;
+  size_t *n;
   int a;
 
   file = fopen(mrl, "r");
@@ -718,7 +718,7 @@ void mediamarks_cb (void *data) {
   snprintf(mmpath,sizeof(mmpath),"%s/.xine/oxine/mediamarks", xine_get_homedir());
   if (!read_mediamarks(items, mmpath)) {
     lprintf("trying to load system wide mediamarks\n");
-    snprintf(mmpath,1024,"%s/mediamarks", XINE_OXINEDIR);
+    snprintf(mmpath,sizeof(mmpath),"%s/mediamarks", XINE_OXINEDIR);
     if (read_mediamarks(items, mmpath)) {
       bpush(session->backpath, items);
       oxine->reentry_data = session;
