@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: panel.c,v 1.226 2005/07/24 02:40:37 fsck-p Exp $
+ * $Id: panel.c,v 1.227 2006/06/29 14:08:13 dgp85 Exp $
  *
  * xine panel related stuff
  *
@@ -845,10 +845,10 @@ void panel_toggle_audio_mute(xitk_widget_t *w, void *data, int state) {
  *  to snapshot current frame.
  */
 static void panel_snapshot_error(void *data, char *message) {
-  xine_error(message);
+  xine_error("%s", message);
 }
 static void panel_snapshot_info(void *data, char *message) {
-  xine_info(message);
+  xine_info("%s", message);
 }
 void panel_snapshot(xitk_widget_t *w, void *data) {
   create_snapshot(gGui->mmk.mrl, panel_snapshot_error, panel_snapshot_info, NULL);
