@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: playlist.h,v 1.22 2004/04/13 19:55:48 f1rmb Exp $
+ * $Id: playlist.h,v 1.26 2005/02/06 02:09:42 miguelfreitas Exp $
  *
  */
 
@@ -34,6 +34,7 @@ void playlist_mmk_editor(void);
 void playlist_scan_for_infos_selected(void);
 void playlist_scan_for_infos(void);
 void playlist_show_tips(int enabled, unsigned long timeout);
+void playlist_update_tips_timeout(unsigned long timeout);
 void playlist_mrlident_toggle(void);
 void playlist_update_playlist(void);
 void playlist_exit(xitk_widget_t *, void *);
@@ -43,8 +44,9 @@ void playlist_scan_input(xitk_widget_t *, void *);
 void playlist_raise_window(void);
 void playlist_toggle_visibility(xitk_widget_t *, void *);
 void playlist_update_focused_entry(void);
-void playlist_change_skins(void);
+void playlist_change_skins(int);
 void playlist_play_current(xitk_widget_t *w, void *data);
+void playlist_delete_entry(int j);
 void playlist_delete_current(xitk_widget_t *w, void *data);
 void playlist_delete_all(xitk_widget_t *w, void *data);
 void playlist_move_current_updown(xitk_widget_t *w, void *data);
@@ -53,4 +55,5 @@ void playlist_editor(void);
 void playlist_load_playlist(xitk_widget_t *w, void *data);
 void playlist_save_playlist(xitk_widget_t *w, void *data);
 void playlist_reparent(void);
+void playlist_get_input_focus(void);
 #endif
