@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2006 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,13 +17,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: session.c,v 1.10 2003/07/14 22:44:09 f1rmb Exp $
+ * $Id: session.c,v 1.25 2006/10/19 22:12:52 fsck-p Exp $
  *
  */
 /* Largely inspired of xmms control socket stuff */
 
 /* required for getsubopt(); the __sun test gives us strncasecmp() on solaris */
-#if ! defined (__sun) && ! defined (__OpenBSD__)
+#if ! defined (__sun) && ! defined (__OpenBSD__) && ! defined (__FreeBSD__)
 #define _XOPEN_SOURCE 500
 #endif
 /* required for S_ISSOCK */
@@ -38,6 +38,7 @@
 #include <unistd.h>
 #include <strings.h>
 #include <sys/types.h>
+#include <sys/select.h>
 #include <sys/stat.h>
 #include <sys/socket.h>
 #include <sys/un.h>
@@ -76,18 +77,21 @@ static int connect_to_session(int session) {
   int fd;
   
   if((fd = socket(AF_UNIX, SOCK_STREAM, 0)) != -1) {
-    struct sockaddr_un   saddr;
+    union {
+      struct sockaddr_un un;
+      struct sockaddr sa;
+    } saddr;
     uid_t                stored_uid, euid;
     
-    saddr.sun_family = AF_UNIX;
+    saddr.un.sun_family = AF_UNIX;
     stored_uid       = getuid();
     euid             = geteuid();
     setuid(euid);
 
-    snprintf(saddr.sun_path, 108, "%s/.xine/session.%d", (xine_get_homedir()), session);
+    snprintf(saddr.un.sun_path, 108, "%s%s%d", (xine_get_homedir()), "/.xine/session.", session);
     setreuid(stored_uid, euid);
 
-    if((connect(fd,(struct sockaddr *) &saddr, sizeof(saddr))) != -1)
+    if((connect(fd,&saddr.sa, sizeof(saddr.un))) != -1)
       return fd;
 
   }
@@ -257,12 +261,16 @@ int is_remote_running(int session) {
 
 
 #ifndef CTRL_TEST
-void *ctrlsocket_func(void *data) {
+static void *ctrlsocket_func(void *data) {
   fd_set                set;
   struct timeval        tv;
-  struct sockaddr_un    saddr;
   serv_header_packet_t *shdr;
-  int                   len, fd;
+  int                   fd;
+  socklen_t             len;
+  union {
+    struct sockaddr_un un;
+    struct sockaddr sa;
+  } saddr;
 
   while(!going)
     xine_usec_sleep(10000);
@@ -275,10 +283,10 @@ void *ctrlsocket_func(void *data) {
     tv.tv_sec  = 0;
     tv.tv_usec = 100000;
 
-    len = sizeof(saddr);
+    len = sizeof(saddr.un);
     
     if(((select(ctrl_fd + 1, &set, NULL, NULL, &tv)) <= 0) || 
-       ((fd = accept(ctrl_fd, (struct sockaddr *) &saddr, &len)) == -1))
+       ((fd = accept(ctrl_fd, &saddr.sa, &len)) == -1))
       continue;
     
     shdr = (serv_header_packet_t *) xine_xmalloc((sizeof(serv_header_packet_t)));
@@ -364,6 +372,30 @@ void *ctrlsocket_func(void *data) {
       send_ack(shdr);
       break;
 	
+    case CMD_PLAYLIST_FIRST:
+      if(gGui->playlist.num) {
+	if(xine_get_status(gGui->stream) == XINE_STATUS_PLAY)
+	  gui_stop(NULL, NULL);
+	
+	gGui->playlist.cur = 0;
+	gui_set_current_mmk(mediamark_get_current_mmk());
+	gui_play(NULL, NULL);
+      }
+      send_ack(shdr);
+      break;
+      
+    case CMD_PLAYLIST_LAST:
+      if(gGui->playlist.num) {
+	if(xine_get_status(gGui->stream) == XINE_STATUS_PLAY)
+	  gui_stop(NULL, NULL);
+	
+	gGui->playlist.cur = gGui->playlist.num - 1;
+	gui_set_current_mmk(mediamark_get_current_mmk());
+	gui_play(NULL, NULL);
+      }
+      send_ack(shdr);
+      break;
+
     case CMD_PLAYLIST_FLUSH:
       playlist_delete_all(NULL, NULL);
       send_ack(shdr);
@@ -386,37 +418,123 @@ void *ctrlsocket_func(void *data) {
 
     case CMD_PLAYLIST_LOAD:
       mediamark_load_mediamarks((const char *)shdr->data);
-      gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+      gui_set_current_mmk(mediamark_get_current_mmk());
       playlist_update_playlist();
       if((!is_playback_widgets_enabled()) && gGui->playlist.num)
 	enable_playback_controls(1);
       send_ack(shdr);
       break;
 
+    case CMD_PLAYLIST_STOP:
+      if(xine_get_status(gGui->stream) != XINE_STATUS_STOP)
+	gGui->playlist.control |= PLAYLIST_CONTROL_STOP;
+      send_ack(shdr);
+      break;
+
+    case CMD_PLAYLIST_CONTINUE:
+      if(xine_get_status(gGui->stream) != XINE_STATUS_STOP)
+	gGui->playlist.control &= ~PLAYLIST_CONTROL_STOP;
+      send_ack(shdr);
+      break;
+
     case CMD_VOLUME:
       {
-	uint32_t *vol = (uint32_t *)shdr->data;
-
-	if((*vol >= 0) && (*vol <= 100) && (gGui->mixer.caps & MIXER_CAP_VOL)) { 
-	  gGui->mixer.volume_level = *vol;
-	  xine_set_param(gGui->stream, XINE_PARAM_AUDIO_VOLUME, gGui->mixer.volume_level);
-	  osd_draw_bar(_("Audio Volume"), 0, 100, gGui->mixer.volume_level, OSD_BAR_STEPPER);
-	}
-
+	int *vol = (int *)shdr->data;
+	
+	if((gGui->mixer.method == SOUND_CARD_MIXER) && 
+	   (gGui->mixer.caps & MIXER_CAP_VOL) && ((*vol >= 0) && (*vol <= 100)))
+	  change_audio_vol(*vol);
+	else if((gGui->mixer.method == SOFTWARE_MIXER) && ((*vol >= 0) && (*vol <= 200)))
+	  change_amp_vol(*vol);
+	
 	send_ack(shdr);
       }
       break;
 
     case CMD_AMP:
       {
-	uint32_t *amp = (uint32_t *)shdr->data;
-
+	int *amp = (int *)shdr->data;
+	
 	if((*amp >= 0) && (*amp <= 200))
-	  config_update_num("gui.amp_level", (int) *amp);
+	  change_amp_vol(*amp);
 
 	send_ack(shdr);
       }
       break;
+      
+    case CMD_LOOP:
+      {
+	int *loop = (int *)shdr->data;
+
+	switch(*loop) {
+	case PLAYLIST_LOOP_NO_LOOP:
+	case PLAYLIST_LOOP_LOOP:
+	case PLAYLIST_LOOP_REPEAT:
+	case PLAYLIST_LOOP_SHUFFLE:
+	case PLAYLIST_LOOP_SHUF_PLUS:
+	case PLAYLIST_LOOP_MODES_NUM:
+	  gGui->playlist.loop = *loop;
+	  break;
+	}
+	
+	send_ack(shdr);
+      }
+      break;
+
+    case CMD_GET_SPEED_STATUS:
+      {
+	uint32_t status = 2;
+
+	if(xine_get_status(gGui->stream) == XINE_STATUS_PLAY) {
+	  int speed = xine_get_param(gGui->stream, XINE_PARAM_SPEED);
+
+	  switch(speed) {
+	  case XINE_SPEED_NORMAL:
+	    status = 3;
+	    break;
+	  case XINE_SPEED_PAUSE:
+	    status = 4;
+	    break;
+	  case XINE_SPEED_SLOW_4:
+	    status = 5;
+	    break;
+	  case XINE_SPEED_SLOW_2:
+	    status = 6;
+	    break;
+	  case XINE_SPEED_FAST_2:
+	    status = 7;
+	    break;
+	  case XINE_SPEED_FAST_4:
+	    status = 8;
+	    break;
+	  default:
+	    /* Dude! */
+	    status = 1;
+	    break;
+	  }
+	}
+	
+	send_packet(shdr->fd, CMD_GET_SPEED_STATUS, &status, sizeof(status));
+	send_ack(shdr);
+      }
+      break;
+    
+
+    case CMD_GET_TIME_STATUS_IN_SECS:
+      {
+	uint32_t status = gGui->stream_length.time / 1000;
+	send_packet(shdr->fd, CMD_GET_TIME_STATUS_IN_SECS, &status, sizeof(status));
+	send_ack(shdr);
+      }
+      break;
+
+    case CMD_GET_TIME_STATUS_IN_POS:
+      {
+	uint32_t status = gGui->stream_length.pos;
+	send_packet(shdr->fd, CMD_GET_TIME_STATUS_IN_POS, &status, sizeof(status));
+	send_ack(shdr);
+      }
+      break;
 
     case CMD_GET_VERSION:
       send_packet(shdr->fd, CMD_GET_VERSION, VERSION,  strlen(VERSION));
@@ -450,44 +568,47 @@ void deinit_session(void) {
 }
 
 int init_session(void) {
-  struct sockaddr_un   saddr;
+  union {
+    struct sockaddr_un un;
+    struct sockaddr sa;
+  } saddr;
   int                  retval = 0;
   int                  i;
   
   if((ctrl_fd = socket(AF_UNIX, SOCK_STREAM, 0)) != -1) {
     for(i = 0;; i++)	{
-      saddr.sun_family = AF_UNIX;
+      saddr.un.sun_family = AF_UNIX;
       
-      snprintf(saddr.sun_path, 108, "%s/.xine/session.%d", (xine_get_homedir()), i);
+      snprintf(saddr.un.sun_path, 108, "%s%s%d", (xine_get_homedir()), "/.xine/session.", i);
       if(!is_remote_running(i)) {
-	if((unlink(saddr.sun_path) == -1) && errno != ENOENT) {
-	  printf("setup_ctrlsocket(): Failed to unlink %s (Error: %s)", 
-		 saddr.sun_path, strerror(errno));
+	if((unlink(saddr.un.sun_path) == -1) && errno != ENOENT) {
+	  fprintf(stderr, "setup_ctrlsocket(): Failed to unlink %s (Error: %s)", 
+		  saddr.un.sun_path, strerror(errno));
 	}
       }
       else
 	continue;
       
-      if((bind(ctrl_fd, (struct sockaddr *) &saddr, sizeof (saddr))) != -1) {
+      if((bind(ctrl_fd, &saddr.sa, sizeof (saddr.un))) != -1) {
 
 	session_id = i;
 	listen(ctrl_fd, 100);
 	going = 1;
 	
 	pthread_create(&thread_server, NULL, ctrlsocket_func, NULL);
-	socket_name = strdup(saddr.sun_path);
+	socket_name = strdup(saddr.un.sun_path);
 	retval = 1;
 	break;
       }
       else {
-	printf("setup_ctrlsocket(): Failed to assign %s to a socket (Error: %s)",
-	       saddr.sun_path, strerror(errno));
+	fprintf(stderr, "setup_ctrlsocket(): Failed to assign %s to a socket (Error: %s)",
+		saddr.un.sun_path, strerror(errno));
 	break;
       }
     }
   }
   else
-    printf("setup_ctrlsocket(): Failed to open socket: %s", strerror(errno));
+    fprintf(stderr, "setup_ctrlsocket(): Failed to open socket: %s", strerror(errno));
   
   if(!retval) {
     if(ctrl_fd != -1)
@@ -498,30 +619,44 @@ int init_session(void) {
   return retval;
 }
 
-void session_handle_subopt(char *suboptarg, int *session) {
-  char        *sopts = suboptarg;
-  int          optsess = -1;
-  int          playlist_clear, playlist_next, playlist_prev = 0;
+static int sanestrcasecmp(char *s1, char *s2) {
+  if(s1 && s2)
+    return strcasecmp(s1, s2);
+  return 1;
+} 
+
+int session_handle_subopt(char *suboptarg, int *session) {
+  int          playlist_first, playlist_last, playlist_clear, playlist_next, playlist_prev, playlist_stop_cont;
   int          audio_next, audio_prev, spu_next, spu_prev;
-  int          volume, amp;
-  char        *playlist_load = NULL;
+  int          volume, amp, loop, speed_status, time_status;
   int          fullscreen, s, c;
   uint32_t     state;
   char        *optstr;
-  char       **mrls = NULL;
-  int          num_mrls = 0;
-  const char  *tokens[] = {
+  char       **mrls          = NULL;
+  int          num_mrls      = 0;
+  int          retval        = 0;
+  char        *sopts         = suboptarg;
+  int          optsess       = -1;
+  char        *playlist_load = NULL;
+  static char *const tokens[]= {
     /* Don't change order */
-    "play", "slow2", "slow4", "pause", "fast2", "fast4", "stop", "quit", "fullscreen", "eject",
-    "audio", "spu", "session", "mrl", "playlist", "pl", "volume", "amp", NULL
+    "play",  "slow2",  "slow4",   "pause",      "fast2",
+    "fast4", "stop",   "quit",    "fullscreen", "eject",
+    "audio", "spu",    "session", "mrl",        "playlist", 
+    "pl",    "volume", "amp",     "loop",       "get_speed",
+    "get_time",
+    NULL
   };
   
-  playlist_clear = playlist_next = playlist_prev = fullscreen = 0;
-  audio_next = audio_prev = spu_next = spu_prev = 0;
-  volume = amp = -1;
-  state = 0;
+  playlist_first = playlist_last = playlist_clear = playlist_next = playlist_prev = playlist_stop_cont = 0;
+  fullscreen     = 0;
+  audio_next     = audio_prev = spu_next = spu_prev = 0;
+  volume = amp   = -1;
+  state          = 0;
+  loop           = -1;
+  speed_status   = time_status = 0;
 
-  while((c = getsubopt(&sopts, (char *const *)tokens, &optstr)) != -1) {
+  while((c = getsubopt(&sopts, tokens, &optstr)) != -1) {
     switch(c) {
 
       /* play -> eject */
@@ -540,48 +675,53 @@ void session_handle_subopt(char *suboptarg, int *session) {
 
       /* audio */
     case 10:
-      if(!strcasecmp(optstr, "next"))
+      if(!sanestrcasecmp(optstr, "next"))
 	audio_next++;
-      else if(!strcasecmp(optstr, "prev"))
+      else if(!sanestrcasecmp(optstr, "prev"))
 	audio_prev++;
       break;
 
       /* spu */
     case 11:
-      if(!strcasecmp(optstr, "next"))
+      if(!sanestrcasecmp(optstr, "next"))
 	spu_next++;
-      else if(!strcasecmp(optstr, "prev"))
+      else if(!sanestrcasecmp(optstr, "prev"))
 	spu_prev++;
       break;
-
+      
       /* session */
     case 12:
       if((atoi(optstr)) >= 0)
 	optsess = atoi(optstr);
       break;
-
+      
       /* mrl */
     case 13:
-      if(num_mrls == 0)
-	mrls = (char **) xine_xmalloc(sizeof(char *) * 2);
-      else
-	mrls = (char **) realloc(mrls, sizeof(char *) * (num_mrls + 2));
-	
-	mrls[num_mrls++] = strdup(optstr);
-	mrls[num_mrls]   = NULL;
+      mrls = (char **) realloc(mrls, sizeof(char *) * (num_mrls + 2));
+      
+      mrls[num_mrls++] = strdup(optstr);
+      mrls[num_mrls]   = NULL;
       break;
 
       /* playlist */
     case 14:
     case 15:
-      if(!strcasecmp(optstr, "clear"))
+      if(!sanestrcasecmp(optstr, "first"))
+	playlist_first = 1;
+      else if(!sanestrcasecmp(optstr, "last"))
+	playlist_last = 1;
+      else if(!sanestrcasecmp(optstr, "clear"))
 	playlist_clear = 1;
-      else if(!strcasecmp(optstr, "next"))
+      else if(!sanestrcasecmp(optstr, "next"))
 	playlist_next++;
-      else if(!strcasecmp(optstr, "prev"))
+      else if(!sanestrcasecmp(optstr, "prev"))
 	playlist_prev++;
       else if(!strncasecmp(optstr, "load:", 5))
 	playlist_load = strdup(optstr + 5);
+      else if(!sanestrcasecmp(optstr, "stop"))
+	playlist_stop_cont = -1;
+      else if(!sanestrcasecmp(optstr, "cont"))
+	playlist_stop_cont = 1;
       break;
 
       /* volume */
@@ -594,6 +734,31 @@ void session_handle_subopt(char *suboptarg, int *session) {
       amp = strtol(optstr, &optstr, 10);
       break;
       
+      /* loop */
+    case 18:
+      if(!sanestrcasecmp(optstr, "none"))
+	loop = PLAYLIST_LOOP_NO_LOOP;
+      else if(!sanestrcasecmp(optstr, "loop"))
+	loop = PLAYLIST_LOOP_LOOP;
+      else if(!sanestrcasecmp(optstr, "repeat"))
+	loop = PLAYLIST_LOOP_REPEAT;
+      else if(!sanestrcasecmp(optstr, "shuffle"))
+	loop = PLAYLIST_LOOP_SHUFFLE;
+      else if(!sanestrcasecmp(optstr, "shuffle+"))
+	loop = PLAYLIST_LOOP_SHUF_PLUS;
+      break;
+
+      /* speed status */
+    case 19:
+      speed_status = 1;
+      break;
+      
+    case 20:
+      time_status = 1;
+      if(!sanestrcasecmp(optstr, "p") || !sanestrcasecmp(optstr, "pos"))
+	time_status = 2;
+      break;
+
     }
   }
   
@@ -622,6 +787,12 @@ void session_handle_subopt(char *suboptarg, int *session) {
       playlist_prev--;
     }
     
+    if(playlist_stop_cont < 0)
+      remote_cmd(*session, CMD_PLAYLIST_STOP);
+
+    if(playlist_stop_cont > 0)
+      remote_cmd(*session, CMD_PLAYLIST_CONTINUE);
+
     if(num_mrls) {
       int i;
 
@@ -671,9 +842,37 @@ void session_handle_subopt(char *suboptarg, int *session) {
     if(amp >= 0)
       send_uint32(*session, CMD_AMP, (uint32_t) amp);
 
+    if(loop > -1)
+      send_uint32(*session, CMD_LOOP, (uint32_t) loop);
+
+    if(playlist_first)
+      remote_cmd(*session, CMD_PLAYLIST_FIRST);
+
+    if(playlist_last)
+      remote_cmd(*session, CMD_PLAYLIST_LAST);
+
+    if(speed_status)
+      retval = get_uint32(*session, CMD_GET_SPEED_STATUS);
+
+    if(time_status) {
+      switch(time_status) {
+      case 1:
+	retval = get_uint32(*session, CMD_GET_TIME_STATUS_IN_SECS);
+	break;
+
+      case 2:
+	retval = get_uint32(*session, CMD_GET_TIME_STATUS_IN_POS);
+	break;
+      }
+
+      fprintf(stdout, "%d\n", retval);
+    }
+
   }
   else
-    printf(_("Session %d isn't running.\n"), *session);
+    fprintf(stderr, _("Session %d isn't running.\n"), *session);
+
+  return retval;
 }
 
 #else /* CTRL_TEST */
@@ -696,4 +895,3 @@ int main(int argc, char **argv) {
   return 1;
 }
 #endif
-
