@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: session.h,v 1.9 2004/09/08 21:13:27 f1rmb Exp $
+ * $Id: session.h,v 1.10 2005/09/14 22:01:07 f1rmb Exp $
  *
  */
 #ifndef CTRLSOCK_H
@@ -52,6 +52,8 @@ typedef enum {
   CMD_AMP,
   CMD_LOOP,
   CMD_GET_SPEED_STATUS,
+  CMD_GET_TIME_STATUS_IN_SECS,
+  CMD_GET_TIME_STATUS_IN_POS,
   CMD_GET_VERSION,
   CMD_PING
 } ctrl_commands_t;
