@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: setup.h,v 1.9 2004/04/13 19:55:49 f1rmb Exp $
+ * $Id: setup.h,v 1.11 2004/07/21 19:56:17 f1rmb Exp $
  *
  */
 
@@ -31,11 +31,13 @@
 #include "xitk.h"
 
 void setup_panel(void);
-void setup_exit(xitk_widget_t *, void *);
 int setup_is_visible(void);
 int setup_is_running(void);
 void setup_toggle_visibility(xitk_widget_t *, void *);
 void setup_raise_window(void);
 void setup_reparent(void);
+void setup_end(void);
+void setup_show_tips(int enabled, unsigned long timeout);
+void setup_update_tips_timeout(unsigned long timeout);
 
 #endif
