@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2007 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: stream_infos.c,v 1.36 2004/04/13 19:55:49 f1rmb Exp $
+ * $Id: stream_infos.c,v 1.46 2007/03/29 23:51:34 fsck-p Exp $
  *
  *
  */
@@ -38,13 +38,15 @@
 
 #include "common.h"
 	
-#define WINDOW_WIDTH        500
-#define WINDOW_HEIGHT       500
+#define WINDOW_WIDTH        570
+#define WINDOW_HEIGHT       616
+
+#define METAINFO_CHARSET    "UTF-8"
 
 extern gGui_t          *gGui;
 
 static char            *sinfosfontname     = "-*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*";
-static char            *lfontname          = "-*-helvetica-bold-r-*-*-11-*-*-*-*-*-*-*";
+static char            *lfontname          = "-*-helvetica-bold-r-*-*-10-*-*-*-*-*-*-*";
 static char            *btnfontname        = "-*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*";
 
 typedef struct {
@@ -129,12 +131,10 @@ char *get_fourcc_string(uint32_t f) {
   fcc[4] = 0;
   
   if(f <= 0xFFFF)
-    sprintf(fcc, "0x%x", f);
+    snprintf(fcc, sizeof(fcc), "0x%x", f);
   
-  if((fcc[0] == 'm') && (fcc[1] == 's')) {
-    if((fcc[2] = 0x0) && (fcc[3] == 0x55)) {
-      *(uint32_t *) fcc = 0x33706d2e; /* Force to '.mp3' */
-    }
+  if(memcmp(fcc, "ms\0\x55", 4) == 0) {
+    strncpy(fcc, "3pm.", 4); /* Force to '.mp3' */
   }
   
   return &fcc[0];
@@ -143,17 +143,25 @@ char *get_fourcc_string(uint32_t f) {
 static char *get_num_string(uint32_t num) {
   static char buffer[1024];
 
-  memset(&buffer, 0, sizeof(buffer));
-  snprintf(buffer, 1023, "%d", num);
+  snprintf(buffer, sizeof(buffer), "%d", num);
 
   return &buffer[0];
 }
 
 static void get_meta_info(xitk_widget_t *w, int meta) {
-  const char *minfo;
+  char *minfo;
+  xitk_recode_t *xr;
   
-  minfo = xine_get_meta_info(gGui->stream, meta);
+  xr = xitk_recode_init(METAINFO_CHARSET, "");
+  
+  minfo = (char *)xine_get_meta_info(gGui->stream, meta);
+  if(minfo)
+    minfo = xitk_recode(xr, minfo);
   set_label(w, (minfo) ? (char *) minfo : _("Unavailable"));
+  
+  if(minfo)
+    free(minfo);
+  xitk_recode_done(xr);
 }
 
 static void get_stream_info(xitk_widget_t *w, int info) {
@@ -180,31 +188,39 @@ static void get_stream_yesno_info(xitk_widget_t *w, int info) {
 static void get_stream_video_resolution_info(void) {
   uint32_t    video_w, video_h;
   char        buffer[1024];
-  
-  memset(&buffer, 0, sizeof(buffer));
 
   video_w = xine_get_stream_info(gGui->stream, XINE_STREAM_INFO_VIDEO_WIDTH);
   video_h = xine_get_stream_info(gGui->stream, XINE_STREAM_INFO_VIDEO_HEIGHT);
 
-  sprintf(buffer, "%d X %d", video_w, video_h);
+  snprintf(buffer, sizeof(buffer), "%d X %d", video_w, video_h);
   set_label(sinfos->infos.video_resolution, buffer);
 }
 
 char *stream_infos_get_ident_from_stream(xine_stream_t *stream) {
-  const char  *title   = NULL;
+  char        *title   = NULL;
   char        *atitle  = NULL;
-  const char  *album   = NULL;
+  char        *album   = NULL;
   char        *aartist = NULL;
-  const char  *artist  = NULL;
+  char        *artist  = NULL;
   char        *aalbum  = NULL; 
   char        *ident   = NULL;
+  xitk_recode_t *xr;
   
   if(!stream)
     return NULL;
   
-  title = xine_get_meta_info(stream, XINE_META_INFO_TITLE);
-  artist = xine_get_meta_info(stream, XINE_META_INFO_ARTIST);
-  album = xine_get_meta_info(stream, XINE_META_INFO_ALBUM);
+  title = (char *)xine_get_meta_info(stream, XINE_META_INFO_TITLE);
+  artist = (char *)xine_get_meta_info(stream, XINE_META_INFO_ARTIST);
+  album = (char *)xine_get_meta_info(stream, XINE_META_INFO_ALBUM);
+  
+  xr = xitk_recode_init(METAINFO_CHARSET, "");
+  if(title)
+    title = xitk_recode(xr, title);
+  if(artist)
+    artist = xitk_recode(xr, artist);
+  if(album)
+    album = xitk_recode(xr, album);
+  xitk_recode_done(xr);
   
   /*
    * Since meta info can be corrupted/wrong/ugly
@@ -223,6 +239,12 @@ char *stream_infos_get_ident_from_stream(xine_stream_t *stream) {
     xine_strdupa(aalbum, album);
     aalbum = atoa(aalbum);
   }
+  if(title)
+    free(title);
+  if(artist)
+    free(artist);
+  if(album)
+    free(album);
 
   if(atitle) {
     int len = strlen(atitle);
@@ -233,17 +255,17 @@ char *stream_infos_get_ident_from_stream(xine_stream_t *stream) {
       len += strlen(aalbum) + 3;
     
     ident = (char *) xine_xmalloc(len + 1);
-    sprintf(ident, "%s", atitle);
+    strcpy(ident, atitle);
     
     if((aartist && strlen(aartist)) || (aalbum && strlen(aalbum))) {
-      strcat(ident, " (");
+      strlcat(ident, " (", len);
       if(aartist && strlen(aartist))
-	sprintf(ident, "%s%s", ident, aartist);
+	strlcat(ident, aartist, len);
       if((aartist && strlen(aartist)) && (aalbum && strlen(aalbum)))
-	strcat(ident, " - ");
+	strlcat(ident, " - ", len);
       if(aalbum && strlen(aalbum))
-	sprintf(ident, "%s%s", ident, aalbum);
-      strcat(ident, ")");
+	strlcat(ident, aalbum, len);
+      strlcat(ident, ")", len);
     }
   }
 
@@ -280,6 +302,8 @@ static void stream_infos_exit(xitk_widget_t *w, void *data) {
     
     free(sinfos);
     sinfos = NULL;
+
+    try_to_set_input_focus(gGui->video_window);
   }
 }
 
@@ -289,6 +313,8 @@ static void handle_event(XEvent *event, void *data) {
   case KeyPress:
     if(xitk_get_key_pressed(event) == XK_Escape)
       stream_infos_exit(NULL, NULL);
+    else
+      gui_handle_event(event, data);
     break;
   }
 }
@@ -299,7 +325,7 @@ int stream_infos_is_visible(void) {
     if(gGui->use_root_window)
       return xitk_is_window_visible(gGui->display, xitk_window_get_window(sinfos->xwin));
     else
-      return sinfos->visible;
+      return sinfos->visible && xitk_is_window_visible(gGui->display, xitk_window_get_window(sinfos->xwin));
   }
   
   return 0;
@@ -445,14 +471,14 @@ void stream_infos_panel(void) {
   sinfos = (_stream_infos_t *) xine_xmalloc(sizeof(_stream_infos_t));
   
   x = xine_config_register_num (gGui->xine, "gui.sinfos_x", 
-				100,
+				80,
 				CONFIG_NO_DESC,
 				CONFIG_NO_HELP,
 				CONFIG_LEVEL_DEB,
 				CONFIG_NO_CB,
 				CONFIG_NO_DATA);
   y = xine_config_register_num (gGui->xine, "gui.sinfos_y",
-				100,
+				80,
 				CONFIG_NO_DESC,
 				CONFIG_NO_HELP,
 				CONFIG_LEVEL_DEB,
@@ -487,18 +513,18 @@ void stream_infos_panel(void) {
 	    bg->gc, 0, 0, width, height, 0, 0);
   XUnlockDisplay (gGui->display);
   
-  x = 5;
-  y = 35;
+  x = 15;
+  y = 34 - 6;
   
   draw_outter_frame(gGui->imlib_data, bg, _("General"), btnfontname, 
-		    x, y, WINDOW_WIDTH - 10, (4 * 20) + (4 * 15) + 15 + 5);
+		    x, y, WINDOW_WIDTH - 30, (4 * ((20 + 22) + 3) - 3 + 5 + 2) + 15);
 
   /* First Line */
-  x = 15;
+  x = 20;
   y += 15;
-  w = WINDOW_WIDTH - 30;
+  w = WINDOW_WIDTH - 40 - 1;
   draw_inner_frame(gGui->imlib_data, bg, _("Title: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -507,14 +533,13 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->meta_infos.title = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->meta_infos.title);
 
   /* New Line */
-  y += 35;
-  w = WINDOW_WIDTH - 30;
+  y += (20 + 22) + 3;
   draw_inner_frame(gGui->imlib_data, bg, _("Comment: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -523,14 +548,14 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->meta_infos.comment = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->meta_infos.comment);
 
   /* New Line */
-  y += 35;
-  w = (WINDOW_WIDTH - 60) >> 1;
+  y += (20 + 22) + 3;
+  w = (((WINDOW_WIDTH - 40 - 3 * 5) / 4) * 2) + 5 + 1;
   draw_inner_frame(gGui->imlib_data, bg, _("Artist: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -539,13 +564,13 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->meta_infos.artist = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->meta_infos.artist);
 
-  x += w + 15;
-  w = (WINDOW_WIDTH - 60) >> 2;
+  x += w + 5;
+  w = ((WINDOW_WIDTH - 40 - 3 * 5) / 4) + 1;
   draw_inner_frame(gGui->imlib_data, bg, _("Genre: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -554,12 +579,13 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->meta_infos.genre = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->meta_infos.genre);
 
-  x += w + 15;
+  x += w + 5;
+  w -= 1;
   draw_inner_frame(gGui->imlib_data, bg, _("Year: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -568,15 +594,15 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->meta_infos.year = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->meta_infos.year);
 
   /* New Line */
-  x = 15;
-  y += 35;
-  w = (WINDOW_WIDTH - 60) >> 1;
+  x = 20;
+  y += (20 + 22) + 3;
+  w = (((WINDOW_WIDTH - 40 - 3 * 5) / 4) * 2) + 5 + 1;
   draw_inner_frame(gGui->imlib_data, bg, _("Album: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -585,20 +611,20 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->meta_infos.album = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->meta_infos.album);
 
   /* frame space */
-  x = 5;
-  y += 40;
-  draw_outter_frame(gGui->imlib_data, bg, _("Misc"), btnfontname, 
-		    x, y, WINDOW_WIDTH - 10, (2 * 20) + (2 * 15) + 15 + 5);
-  /* New Line */
-  y += 15;
   x = 15;
-  w = (WINDOW_WIDTH - 75) >> 2;
+  y += ((20 + 22) + 5 + 2) + 3;
+  draw_outter_frame(gGui->imlib_data, bg, _("Misc"), btnfontname, 
+		    x, y, WINDOW_WIDTH - 30, (2 * ((20 + 22) + 3) - 3 + 5 + 2) + 15);
+  /* New Line */
+  x = 20;
+  y += 15;
+  w = (WINDOW_WIDTH - 40 - 3 * 5) / 4;
   draw_inner_frame(gGui->imlib_data, bg, _("Input Plugin: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -607,12 +633,13 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->meta_infos.input_plugin = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->meta_infos.input_plugin);
 
-  x += w + 15;
+  x += w + 5;
+  w += 1;
   draw_inner_frame(gGui->imlib_data, bg, _("System Layer: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -621,12 +648,12 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->meta_infos.systemlayer = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->meta_infos.systemlayer);
 
-  x += w + 15;
+  x += w + 5;
   draw_inner_frame(gGui->imlib_data, bg, _("Bitrate: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -635,12 +662,13 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->infos.bitrate = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->infos.bitrate);
 
-  x += w + 15;
+  x += w + 5;
+  w -= 1;
   draw_inner_frame(gGui->imlib_data, bg, _("Frame Duration: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -649,14 +677,14 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->infos.frame_duration = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->infos.frame_duration);
 
   /* New Line */
-  y += 35;
-  x = 15;
+  x = 20;
+  y += (20 + 22) + 3;
   draw_inner_frame(gGui->imlib_data, bg, _("Is Seekable: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -665,12 +693,13 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->infos.seekable = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->infos.seekable);
 
-  x += w + 15;
+  x += w + 5;
+  w += 1;
   draw_inner_frame(gGui->imlib_data, bg, _("Has Chapters: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -679,12 +708,12 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->infos.has_chapters = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->infos.has_chapters);
 
-  x += w + 15;
+  x += w + 5;
   draw_inner_frame(gGui->imlib_data, bg, _("Ignore Spu: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -693,12 +722,13 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->infos.ignore_spu = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->infos.ignore_spu);
 
-  x += w + 15;
+  x += w + 5;
+  w -= 1;
   draw_inner_frame(gGui->imlib_data, bg, _("Has Still: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -707,22 +737,21 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->infos.has_still = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->infos.has_still);
 
   /* video frame */
-  x = 5;
-  y += 40;
+  x = 15;
+  y += ((20 + 22) + 5 + 2) + 3;
   draw_outter_frame(gGui->imlib_data, bg, _("Video"), btnfontname, 
-		    x, y, WINDOW_WIDTH - 10, (2 * 20) + (2 * 15) + 15 + 5);
+		    x, y, WINDOW_WIDTH - 30, (2 * ((20 + 22) + 3) - 3 + 5 + 2) + 15);
 
   /* New Line */
+  x = 20;
   y += 15;
-  x = 15;
-
-  w = (WINDOW_WIDTH - 90) / 5;
+  w = (WINDOW_WIDTH - 40 - 4 * 5) / 5;
   draw_inner_frame(gGui->imlib_data, bg, _("Has: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -731,12 +760,12 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->infos.has_video = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->infos.has_video);
 
-  x += w + 15;
+  x += w + 5;
   draw_inner_frame(gGui->imlib_data, bg, _("Handled: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -745,12 +774,13 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->infos.video_handled = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->infos.video_handled);
 
-  x += w + 15;
+  x += w + 5;
+  w -= 1;
   draw_inner_frame(gGui->imlib_data, bg, _("Ignore: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -759,13 +789,14 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->infos.ignore_video = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->infos.ignore_video);
 
-  x += w + 15;
-  w = (w * 2) + 15 - 1;
+  x += w + 5;
+  w += 1;
+  w = (w * 2) + 5;
   draw_inner_frame(gGui->imlib_data, bg, _("Codec: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -774,15 +805,15 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->meta_infos.videocodec = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->meta_infos.videocodec);
 
   /* New Line */
-  y += 35;
-  x = 15;
-  w = (WINDOW_WIDTH - 105) / 6;
+  x = 20;
+  y += (20 + 22) + 3;
+  w = (WINDOW_WIDTH - 40 - 5 * 5) / 6;
   draw_inner_frame(gGui->imlib_data, bg, _("FourCC: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -791,12 +822,12 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->infos.video_fourcc = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->infos.video_fourcc);
   
-  x += w + 15;
+  x += w + 5;
   draw_inner_frame(gGui->imlib_data, bg, _("Channel(s): "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -805,12 +836,12 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->infos.video_channels = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->infos.video_channels);
   
-  x += w + 15;
+  x += w + 5;
   draw_inner_frame(gGui->imlib_data, bg, _("Bitrate: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -819,13 +850,12 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->infos.video_bitrate = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->infos.video_bitrate);
 
-  x += w + 15;
-  w += 4;
+  x += w + 5;
   draw_inner_frame(gGui->imlib_data, bg, _("Resolution: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -834,13 +864,12 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->infos.video_resolution = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->infos.video_resolution);
 
-  x += w + 15;
-  w -= 4;
+  x += w + 5;
   draw_inner_frame(gGui->imlib_data, bg, _("Ratio: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -849,12 +878,12 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->infos.video_ratio = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->infos.video_ratio);
   
-  x += w + 15;
+  x += w + 5;
   draw_inner_frame(gGui->imlib_data, bg, _("Stream(s): "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -863,22 +892,21 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->infos.video_streams = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->infos.video_streams);
 
   /* Audio Frame */
-  x = 5;
-  y += 40;
+  x = 15;
+  y += ((20 + 22) + 5 + 2) + 3;
   draw_outter_frame(gGui->imlib_data, bg, _("Audio"), btnfontname, 
-		    x, y, WINDOW_WIDTH - 10, (2 * 20) + (2 * 15) + 15 + 5);
+		    x, y, WINDOW_WIDTH - 30, (2 * ((20 + 22) + 3) - 3 + 5 + 2) + 15);
 
   /* New Line */
+  x = 20;
   y += 15;
-  x = 15;
-
-  w = (WINDOW_WIDTH - 90) / 5;
+  w = (WINDOW_WIDTH - 40 - 4 * 5) / 5;
   draw_inner_frame(gGui->imlib_data, bg, _("Has: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -887,12 +915,12 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->infos.has_audio = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->infos.has_audio);
 
-  x += w + 15;
+  x += w + 5;
   draw_inner_frame(gGui->imlib_data, bg, _("Handled: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -901,12 +929,13 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->infos.audio_handled = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->infos.audio_handled);
 
-  x += w + 15;
+  x += w + 5;
+  w -= 1;
   draw_inner_frame(gGui->imlib_data, bg, _("Ignore: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -915,13 +944,14 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->infos.ignore_audio = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->infos.ignore_audio);
 
-  x += w + 15;
-  w = (w * 2) + 15 - 1;
+  x += w + 5;
+  w += 1;
+  w = (w * 2) + 5;
   draw_inner_frame(gGui->imlib_data, bg, _("Codec: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -930,15 +960,15 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->meta_infos.audiocodec = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->meta_infos.audiocodec);
   
   /* New Line */
-  y += 35;
-  x = 15;
-  w = (WINDOW_WIDTH - 90) / 5;
+  x = 20;
+  y += (20 + 22) + 3;
+  w = (WINDOW_WIDTH - 40 - 4 * 5) / 5;
   draw_inner_frame(gGui->imlib_data, bg, _("FourCC: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -947,12 +977,12 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->infos.audio_fourcc = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->infos.audio_fourcc);
 
-  x += w + 15;
+  x += w + 5;
   draw_inner_frame(gGui->imlib_data, bg, _("Channel(s): "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -961,12 +991,13 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->infos.audio_channels = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->infos.audio_channels);
 
-  x += w + 15;
+  x += w + 5;
+  w -= 1;
   draw_inner_frame(gGui->imlib_data, bg, _("Bitrate: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -975,12 +1006,13 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->infos.audio_bitrate = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->infos.audio_bitrate);
 
-  x += w + 15;
+  x += w + 5;
+  w += 1;
   draw_inner_frame(gGui->imlib_data, bg, _("Bits: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -989,13 +1021,12 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->infos.audio_bits = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->infos.audio_bits);
 
-  x += w + 15;
-  w -= 1;
+  x += w + 5;
   draw_inner_frame(gGui->imlib_data, bg, _("Samplerate: "), lfontname, 
-		    x - 5, y - 2, w + 10, 20 + 15);
+		   x, y, w, (20 + 22));
   lbl.window            = xitk_window_get_window(sinfos->xwin);
   lbl.gc                = (XITK_WIDGET_LIST_GC(sinfos->widget_list));
   lbl.skin_element_name = NULL;
@@ -1004,7 +1035,7 @@ void stream_infos_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(sinfos->widget_list)), 
 			   (sinfos->infos.audio_samplerate = 
 			    xitk_noskin_label_create(sinfos->widget_list, &lbl,
-						     x, y, w, 20, sinfosfontname)));
+						     x + 5, y + 15, w - 10, 20, sinfosfontname)));
   xitk_enable_and_show_widget(sinfos->infos.audio_samplerate);
 
   /*  */
@@ -1028,7 +1059,7 @@ void stream_infos_panel(void) {
   if(gGui->stream_info_auto_update)
     xitk_hide_widget(sinfos->update);
   
-  x = WINDOW_WIDTH - 115;
+  x = WINDOW_WIDTH - (100 + 15);
   
   lb.button_type       = CLICK_BUTTON;
   lb.label             = _("Close");
