@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2007 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: videowin.c,v 1.214 2005/05/20 22:00:36 fsck-p Exp $
+ * $Id: videowin.c,v 1.226 2007/03/25 23:26:55 fsck-p Exp $
  *
  * video window handling functions
 */
@@ -27,6 +27,7 @@
 #endif
 
 #include <stdio.h>
+#include <unistd.h>
 #include <string.h>
 #include <sys/time.h>
 #include <X11/Xlib.h>
@@ -83,9 +84,10 @@ typedef struct {
   int                    video_duration_valid; /* is use_duration trustable? */
   int                    win_width;       /* size of non-fullscreen window         */
   int                    win_height;
+  int                    old_win_width;
+  int                    old_win_height;
   int                    output_width;    /* output video window width/height      */
   int                    output_height;
-  float                  mag;
 
   int                    stream_resize_window; /* Boolean, 1 if new stream resize output window */
   int                    zoom_small_stream; /* Boolean, 1 to double size small streams */
@@ -148,11 +150,16 @@ typedef struct {
   int                    second_display_running;
   
   int                    logo_synthetic;
+
+  pthread_mutex_t        mutex;
+
 } gVw_t;
 
 static gVw_t            *gVw;
 static void video_window_handle_event (XEvent *event, void *data);
 static void video_window_adapt_size (void);
+static int  video_window_check_mag (void);
+static void video_window_calc_mag_win_size (float xmag, float ymag);
 
 static void _video_window_resize_cb(void *data, xine_cfg_entry_t *cfg) {
   gVw->stream_resize_window = cfg->num_value;
@@ -192,13 +199,16 @@ static void *second_display_loop (void *dummy) {
     
     do {
         XLockDisplay(gGui->video_display);
-        got_event = XCheckMaskEvent(gGui->video_display, INPUT_MOTION, &xevent);
+        got_event = XPending(gGui->video_display);
+        if( got_event )
+          XNextEvent(gGui->video_display, &xevent);
         XUnlockDisplay(gGui->video_display);
         
         if( got_event && gGui->stream ) {
           video_window_handle_event(&xevent, NULL);
         }
     } while (got_event);
+
   }
   
   pthread_exit(NULL);
@@ -316,11 +326,31 @@ void video_window_select_visual (void) {
       XLockDisplay (gGui->display);
       gui_init_imlib (gGui->visual);
       XUnlockDisplay (gGui->display);
+      pthread_mutex_lock(&gVw->mutex);
       video_window_adapt_size ();
+      pthread_mutex_unlock(&gVw->mutex);
     }
   }
 }
 
+/*
+ * Lock the video window against WM-initiated transparency changes.
+ * At the time of writing (2006-06-29), only xfwm4 SVN understands this.
+ * Ref. http://bugzilla.xfce.org/show_bug.cgi?id=1958
+ */
+static void video_window_lock_opacity (void) {
+  Atom opacity_lock = XInternAtom(gGui->video_display, "_NET_WM_WINDOW_OPACITY_LOCKED", False);
+
+  /* This shouldn't happen, but was reported in bug #1573056 */
+  if(opacity_lock == None)
+    return;
+
+  XChangeProperty(gGui->video_display, gGui->video_window,
+		  opacity_lock,
+		  XA_CARDINAL, 32, PropModeReplace,
+		  (unsigned char *)gGui, 1);
+}
+
 /*
  * will modify/create video output window based on
  *
@@ -377,12 +407,15 @@ static void video_window_adapt_size (void) {
       attr.background_pixel  = gGui->black.pixel;
       
       border_width = 0;
-      
-      gGui->video_window = XCreateWindow(gGui->video_display, rootwindow,
-					 0, 0, gVw->fullscreen_width, gVw->fullscreen_height, 
-					 border_width, 
-					 CopyFromParent, CopyFromParent, CopyFromParent, 
-					 CWBackPixel | CWOverrideRedirect, &attr);
+
+      if(gGui->wid)
+	gGui->video_window = gGui->wid;
+      else
+	gGui->video_window = XCreateWindow(gGui->video_display, rootwindow,
+					   0, 0, gVw->fullscreen_width, gVw->fullscreen_height, 
+					   border_width, 
+					   CopyFromParent, CopyFromParent, CopyFromParent, 
+					   CWBackPixel | CWOverrideRedirect, &attr);
       
       if(gGui->video_display == gGui->display)
         xitk_widget_list_set(gVw->wl, WIDGET_LIST_WINDOW, (void *) gGui->video_window);
@@ -393,7 +426,12 @@ static void video_window_adapt_size (void) {
         XLockDisplay (gGui->video_display);
       }
 
-      XSelectInput(gGui->video_display, gGui->video_window, ExposureMask);
+      if(!(gGui->no_mouse))
+	XSelectInput(gGui->video_display, gGui->video_window, ExposureMask);
+      else
+	XSelectInput(gGui->video_display,
+                     gGui->video_window,
+                     ExposureMask & (~(ButtonPressMask | ButtonReleaseMask)));
       
       _set_window_title();
       
@@ -412,7 +450,9 @@ static void video_window_adapt_size (void) {
       hint.width  = gVw->fullscreen_width;
       hint.height = gVw->fullscreen_height;
       XSetNormalHints(gGui->video_display, gGui->video_window, &hint);
-      
+
+      video_window_lock_opacity();
+
       XClearWindow(gGui->video_display, gGui->video_window);
       
       XMapWindow(gGui->video_display, gGui->video_window);
@@ -423,7 +463,7 @@ static void video_window_adapt_size (void) {
 
       XUnlockDisplay (gGui->video_display);
 
-      if(gGui->video_display == gGui->display)
+      if((gGui->video_display == gGui->display))
         gVw->widget_key = xitk_register_event_handler("video_window", 
 						    gGui->video_window, 
 						    video_window_handle_event,
@@ -666,13 +706,15 @@ static void video_window_adapt_size (void) {
     attr.colormap	   = gVw->colormap;
 
     border_width           = 0;
-
-    gGui->video_window =
-      XCreateWindow (gGui->video_display, DefaultRootWindow(gGui->video_display),
-		     hint.x, hint.y, gVw->visible_width, gVw->visible_height,
-		     border_width, gVw->depth, InputOutput,
-		     gVw->visual,
-		     CWBackPixel | CWBorderPixel | CWColormap, &attr);
+    if(gGui->wid)
+      gGui->video_window = gGui->wid;
+    else
+      gGui->video_window =
+	XCreateWindow (gGui->video_display, DefaultRootWindow(gGui->video_display),
+		       hint.x, hint.y, gVw->visible_width, gVw->visible_height,
+		       border_width, gVw->depth, InputOutput,
+		       gVw->visual,
+		       CWBackPixel | CWBorderPixel | CWColormap, &attr);
 
     if(gGui->video_display == gGui->display)
       xitk_widget_list_set(gVw->wl, WIDGET_LIST_WINDOW, (void *) gGui->video_window);
@@ -695,6 +737,8 @@ static void video_window_adapt_size (void) {
 
     XSetWMHints(gGui->video_display, gGui->video_window, gVw->wm_hint);
 
+    video_window_lock_opacity();
+
     gVw->output_width    = hint.width;
     gVw->output_height   = hint.height;
 
@@ -764,13 +808,16 @@ static void video_window_adapt_size (void) {
 
     border_width           = 0;
 
-    gGui->video_window = 
-      XCreateWindow (gGui->video_display, DefaultRootWindow(gGui->video_display), 
-		     hint.x, hint.y, gVw->visible_width, gVw->visible_height, 
-		     border_width, gVw->depth, InputOutput,
-		     gVw->visual,
-		     CWBackPixel | CWBorderPixel | CWColormap, &attr);
-
+    if(gGui->wid)
+      gGui->video_window = gGui->wid;
+    else
+      gGui->video_window = 
+	XCreateWindow (gGui->video_display, DefaultRootWindow(gGui->video_display), 
+		       hint.x, hint.y, gVw->visible_width, gVw->visible_height, 
+		       border_width, gVw->depth, InputOutput,
+		       gVw->visual,
+		       CWBackPixel | CWBorderPixel | CWColormap, &attr);
+  
     if(gGui->video_display == gGui->display)
       xitk_widget_list_set(gVw->wl, WIDGET_LIST_WINDOW, (void *) gGui->video_window);
     
@@ -798,6 +845,8 @@ static void video_window_adapt_size (void) {
         
     XSetWMHints(gGui->video_display, gGui->video_window, gVw->wm_hint);
 
+    video_window_lock_opacity();
+
     gVw->output_width    = hint.width;
     gVw->output_height   = hint.height;
     
@@ -829,6 +878,9 @@ static void video_window_adapt_size (void) {
       hint.x           = gVw->old_xwin;
       hint.y           = gVw->old_ywin;
     }
+
+    gVw->old_win_width  = hint.width;
+    gVw->old_win_height = hint.height;
     
     gVw->output_width  = hint.width;
     gVw->output_height = hint.height;
@@ -913,12 +965,15 @@ static void video_window_adapt_size (void) {
     else
       border_width = 4;
 
-    gGui->video_window =
-      XCreateWindow(gGui->video_display, DefaultRootWindow(gGui->video_display),
-		    hint.x, hint.y, hint.width, hint.height, border_width, 
-		    gVw->depth, InputOutput, gVw->visual,
-		    CWBackPixel | CWBorderPixel | CWColormap, &attr);
-    
+    if(gGui->wid)
+      gGui->video_window = gGui->wid;
+    else
+      gGui->video_window =
+	XCreateWindow(gGui->video_display, DefaultRootWindow(gGui->video_display),
+		      hint.x, hint.y, hint.width, hint.height, border_width, 
+		      gVw->depth, InputOutput, gVw->visual,
+		      CWBackPixel | CWBorderPixel | CWColormap, &attr);
+
     if(gGui->video_display == gGui->display)
       xitk_widget_list_set(gVw->wl, WIDGET_LIST_WINDOW, (void *) gGui->video_window);
 
@@ -944,6 +999,8 @@ static void video_window_adapt_size (void) {
 
     XSetWMHints(gGui->video_display, gGui->video_window, gVw->wm_hint);
 
+    video_window_lock_opacity();
+
     if(gVw->borderless) {
       prop = XInternAtom(gGui->video_display, "_MOTIF_WM_HINTS", False);
       mwmhints.flags = MWM_HINTS_DECORATIONS;
@@ -954,8 +1011,13 @@ static void video_window_adapt_size (void) {
     }
   }
   
-  XSelectInput(gGui->video_display, gGui->video_window, INPUT_MOTION | KeymapStateMask);
-
+  if(!(gGui->no_mouse))
+    XSelectInput(gGui->video_display, gGui->video_window, INPUT_MOTION | KeymapStateMask);
+  else
+    XSelectInput(gGui->video_display,
+		 gGui->video_window,
+		 (INPUT_MOTION | KeymapStateMask) & (~(ButtonPressMask | ButtonReleaseMask)));
+  
   wm_hint = XAllocWMHints();
   if (wm_hint != NULL) {
     wm_hint->input = True;
@@ -1046,12 +1108,12 @@ static void video_window_adapt_size (void) {
 
   if(gGui->video_display == gGui->display)
     gVw->widget_key = xitk_register_event_handler("video_window", 
-						gGui->video_window, 
-						video_window_handle_event,
-						NULL,
-						gui_dndcallback,
-						NULL, NULL);
-
+						  gGui->video_window, 
+						  video_window_handle_event,
+						  NULL,
+						  gui_dndcallback,
+						  NULL, NULL);
+  
   /* take care about window decoration/pos */
   {
     Window tmp_win;
@@ -1065,11 +1127,11 @@ static void video_window_adapt_size (void) {
   oxine_adapt();
 }
 
-static float get_default_mag(int video_width, int video_height) {
+static void get_default_mag(int video_width, int video_height, float *xmag, float *ymag) {
   if(gVw->zoom_small_stream && video_width < 300 && video_height < 300 )
-    return 2.0;
+    *xmag = *ymag = 2.0f;
   else
-    return 1.0;
+    *xmag = *ymag = 1.0f;
 }
 
 /*
@@ -1080,6 +1142,8 @@ void video_window_dest_size_cb (void *data,
 				double video_pixel_aspect,
 				int *dest_width, int *dest_height,
 				double *dest_pixel_aspect)  {
+
+  pthread_mutex_lock(&gVw->mutex);
   
   gVw->frame_width = video_width;
   gVw->frame_height = video_height;
@@ -1095,16 +1159,19 @@ void video_window_dest_size_cb (void *data,
     if(gVw->video_width != video_width || gVw->video_height != video_height) {
       
       if((video_width > 0) && (video_height > 0)) {
-        double mag = get_default_mag( video_width, video_height );
+	float xmag, ymag;
+
+        get_default_mag(video_width, video_height, &xmag, &ymag);
 
         /* FIXME: this is supposed to give the same results as if a 
-         * video_window_set_mag(mag) was called. Since video_window_adapt_size()
+         * video_window_set_mag(xmag, ymag) was called. Since video_window_adapt_size()
          * check several other details (like border, xinerama, etc) this
          * may produce wrong values in some cases. (?)
-         */  
-        *dest_width  = (int) ((float) video_width * mag);
-        *dest_height = (int) ((float) video_height * mag);
+         */
+        *dest_width  = (int) ((float) video_width * xmag + 0.5f);
+        *dest_height = (int) ((float) video_height * ymag + 0.5f);
         *dest_pixel_aspect = gGui->pixel_aspect;
+        pthread_mutex_unlock(&gVw->mutex);
         return;
       }
     }
@@ -1119,6 +1186,8 @@ void video_window_dest_size_cb (void *data,
     *dest_height = gVw->output_height;
     *dest_pixel_aspect = gGui->pixel_aspect;
   }
+
+  pthread_mutex_unlock(&gVw->mutex);
 }
 
 /*
@@ -1132,6 +1201,8 @@ void video_window_frame_output_cb (void *data,
 				   double *dest_pixel_aspect,
 				   int *win_x, int *win_y) {
 
+  pthread_mutex_lock(&gVw->mutex);
+
   gVw->frame_width = video_width;
   gVw->frame_height = video_height;
 
@@ -1158,22 +1229,23 @@ void video_window_frame_output_cb (void *data,
   }
 #endif
 
-  if(gVw->video_width != video_width || gVw->video_height != video_height )
-    oxine_adapt();
+  if(gVw->video_width != video_width || gVw->video_height != video_height) {
 
-  if(!gVw->stream_resize_window) {
     gVw->video_width  = video_width;
     gVw->video_height = video_height;
-  }
-  else {
-    if(gVw->video_width != video_width || gVw->video_height != video_height ) {
-      
-      gVw->video_width  = video_width;
-      gVw->video_height = video_height;
-      
-      if(video_width > 0 && video_height > 0)
-	video_window_set_mag(get_default_mag( video_width, video_height ));
+
+    if(gVw->stream_resize_window && video_width > 0 && video_height > 0) {
+      float xmag, ymag;
+
+      /* Prepare window size */
+      get_default_mag(video_width, video_height, &xmag, &ymag);
+      video_window_calc_mag_win_size(xmag, ymag);
+      /* If actually ready to adapt window size, do it now */
+      if(video_window_check_mag())
+	video_window_adapt_size();
     }
+
+    oxine_adapt();
   }
 
   *dest_x = 0;
@@ -1193,12 +1265,15 @@ void video_window_frame_output_cb (void *data,
   *win_x = (gVw->xwin < 0) ? 0 : gVw->xwin;
   *win_y = (gVw->ywin < 0) ? 0 : gVw->ywin;
 
+  pthread_mutex_unlock(&gVw->mutex);
 }
 
 /*
  *
  */
 void video_window_set_fullscreen_mode (int req_fullscreen) {
+
+  pthread_mutex_lock(&gVw->mutex);
   
   if(!(gVw->fullscreen_mode & req_fullscreen)) {
 
@@ -1226,10 +1301,11 @@ void video_window_set_fullscreen_mode (int req_fullscreen) {
   }
 
   video_window_adapt_size();
-  try_to_set_input_focus(gGui->video_window);
-  if(osd_is_visible())
-    osd_update_osd();
 
+  try_to_set_input_focus(gGui->video_window);
+  osd_update_osd();
+
+  pthread_mutex_unlock(&gVw->mutex);
 }
 
 /*
@@ -1245,9 +1321,11 @@ int video_window_get_fullscreen_mode (void) {
  * ie: try to expend display on further screens
  */
 void video_window_set_xinerama_fullscreen_mode(int req_fullscreen) {
-  gVw->fullscreen_req = req_fullscreen;
 
+  pthread_mutex_lock(&gVw->mutex);
+  gVw->fullscreen_req = req_fullscreen;
   video_window_adapt_size ();
+  pthread_mutex_unlock(&gVw->mutex);
 }
 
 /*
@@ -1333,7 +1411,14 @@ void video_window_set_visibility(int show_window) {
 
   xine_port_send_gui_data(gGui->vo_port, XINE_GUI_SEND_VIDEOWIN_VISIBLE, (void *)show_window);
   
+  pthread_mutex_lock(&gVw->mutex);
+
   gVw->show = show_window;
+ 
+  /* Switching to visible: If new window size requested meanwhile, adapt window */
+  if((gVw->show) && (gVw->fullscreen_mode & WINDOWED_MODE) &&
+     (gVw->win_width != gVw->old_win_width || gVw->win_height != gVw->old_win_height))
+    video_window_adapt_size();
 
   XLockDisplay(gGui->video_display);
   
@@ -1360,7 +1445,6 @@ void video_window_set_visibility(int show_window) {
      && !(gVw->fullscreen_mode & FULLSCR_XI_MODE)
      && wm_not_ewmh_only())
       xitk_set_ewmh_fullscreen(gGui->video_window);
-    
   }
   else
     XUnmapWindow (gGui->video_display, gGui->video_window);
@@ -1371,6 +1455,7 @@ void video_window_set_visibility(int show_window) {
   if(gVw->hide_on_start == -1)
     gVw->hide_on_start = 0;
 
+  pthread_mutex_unlock(&gVw->mutex);
 }
 
 /*
@@ -1414,6 +1499,8 @@ void video_window_init (window_attributes_t *window_attribute, int hide_on_start
 
   gVw = (gVw_t *) xine_xmalloc(sizeof(gVw_t));
   
+  pthread_mutex_init(&gVw->mutex, NULL);
+
   if(gGui->video_display == gGui->display) {
     gVw->wl                 = xitk_widget_list_new();
     xitk_widget_list_set(gVw->wl, WIDGET_LIST_LIST, (xitk_list_new()));
@@ -1449,8 +1536,8 @@ void video_window_init (window_attributes_t *window_attribute, int hide_on_start
   gVw->desktopHeight      = DisplayHeight(gGui->video_display, gGui->video_screen);
 
 #ifdef HAVE_XTESTEXTENSION
-  gVw->fake_keys[0] = XKeysymToKeycode(gGui->video_display, XK_Scroll_Lock);
-  gVw->fake_keys[1] = XKeysymToKeycode(gGui->video_display, XK_Num_Lock);
+  gVw->fake_keys[0] = XKeysymToKeycode(gGui->video_display, XK_Shift_L);
+  gVw->fake_keys[1] = XKeysymToKeycode(gGui->video_display, XK_Control_L);
   gVw->fake_key_cur = 0;
 #endif
   
@@ -1678,6 +1765,8 @@ void video_window_init (window_attributes_t *window_attribute, int hide_on_start
     gVw->video_width  = 768;
     gVw->video_height = 480;
   }
+  gVw->old_win_width  = gVw->win_width  = gVw->video_width;
+  gVw->old_win_height = gVw->win_height = gVw->video_height;
 
 #ifdef HAVE_XF86VIDMODE
   if(xine_config_register_bool(gGui->xine, "gui.use_xvidext", 
@@ -1760,7 +1849,10 @@ void video_window_init (window_attributes_t *window_attribute, int hide_on_start
   
 #endif
 
-  video_window_set_mag(1.0);
+  /*
+   * Create initial video window with the geometry constructed above.
+   */
+  video_window_adapt_size();
 
   /*
    * for plugins that aren't really bind to the window, it's necessary that the
@@ -1813,19 +1905,22 @@ void video_window_exit (void) {
 
   gVw->second_display_running = 0;
   if(gGui->use_root_window || gGui->video_display != gGui->display) {
-    XExposeEvent event;
+    union {
+      XExposeEvent expose;
+      XEvent event;
+    } event;
     
     XLockDisplay(gGui->video_display);
     XClearWindow(gGui->video_display, gGui->video_window);
-    event.type       = Expose;
-    event.send_event = True;
-    event.display    = gGui->video_display;
-    event.window     = gGui->video_window;
-    event.x          = 0;
-    event.y          = 0;
-    event.width      = gVw->video_width;
-    event.height     = gVw->video_height;
-    XSendEvent(gGui->video_display, gGui->video_window, False, Expose, (XEvent *) &event);
+    event.expose.type       = Expose;
+    event.expose.send_event = True;
+    event.expose.display    = gGui->video_display;
+    event.expose.window     = gGui->video_window;
+    event.expose.x          = 0;
+    event.expose.y          = 0;
+    event.expose.width      = gVw->video_width;
+    event.expose.height     = gVw->video_height;
+    XSendEvent(gGui->video_display, gGui->video_window, False, Expose, &event.event);
     XUnlockDisplay(gGui->video_display);
   }
     
@@ -1834,6 +1929,7 @@ void video_window_exit (void) {
   else
     pthread_join(gVw->second_display_thread, NULL);
 
+  pthread_mutex_destroy(&gVw->mutex);
 }
 
 
@@ -1855,7 +1951,7 @@ static int video_window_translate_point(int gui_x, int gui_y,
   rect.h = 0;
 
   if (xine_port_send_gui_data(gGui->vo_port, 
-			    XINE_GUI_SEND_TRANSLATE_GUI_TO_VIDEO, (void*)&rect) != -1) {
+			      XINE_GUI_SEND_TRANSLATE_GUI_TO_VIDEO, (void*)&rect) != -1) {
     /* driver implements gui->video coordinate space translation, use it */
     *video_x = rect.x;
     *video_y = rect.y;
@@ -1864,6 +1960,8 @@ static int video_window_translate_point(int gui_x, int gui_y,
 
   /* Driver cannot convert gui->video space, fall back to old code... */
 
+  pthread_mutex_lock(&gVw->mutex);
+
   XLockDisplay(gGui->video_display);
   if(XGetGeometry(gGui->video_display, gGui->video_window, &rootwin, 
 		  &xwin, &ywin, &wwin, &hwin, &bwin, &dwin) == BadDrawable) {
@@ -1897,34 +1995,61 @@ static int video_window_translate_point(int gui_x, int gui_y,
     /* printf("hscale:a=%f s=%f x=%d, y=%d\n",aspect,scale,*video_x,*video_y);  */
   }
 
+  pthread_mutex_unlock(&gVw->mutex);
+
   return 1;
 }
 
 /*
  * Set/Get magnification.
  */
-void video_window_set_mag(float mag) {
-  
+static int video_window_check_mag(void)
+{
   if((!(gVw->fullscreen_mode & WINDOWED_MODE))
+/*
+ * Currently, no support for magnification in fullscreen mode.
+ * Commented out in order to not mess up current mag for windowed mode.
+ *
 #ifdef HAVE_XF86VIDMODE
      && !(gVw->XF86_modelines_count > 1)
 #endif
-     )
-    return;
+ */
+    )
+    return 0;
   
-  gVw->mag        = mag;
-  gVw->win_width  = (int) ((float) gVw->video_width) * gVw->mag;
-  gVw->win_height = (int) ((float) gVw->video_height) * gVw->mag;
-  
-  video_window_adapt_size ();
+  /* Allow mag only if video win is visible, so don't do something we can't see. */
+  return (xitk_is_window_visible(gGui->video_display, gGui->video_window));
 }
 
-float video_window_get_mag (void) {
+static void video_window_calc_mag_win_size(float xmag, float ymag)
+{
+  gVw->win_width  = (int) ((float) gVw->video_width * xmag + 0.5f);
+  gVw->win_height = (int) ((float) gVw->video_height * ymag + 0.5f);
+}
+
+int video_window_set_mag(float xmag, float ymag) {
+  
+  pthread_mutex_lock(&gVw->mutex);
+
+  if (!video_window_check_mag()) {
+    pthread_mutex_unlock(&gVw->mutex);
+    return 0;
+  }
+  video_window_calc_mag_win_size(xmag, ymag);
+  video_window_adapt_size ();
+
+  pthread_mutex_unlock(&gVw->mutex);
+
+  return 1;
+}
+
+void video_window_get_mag (float *xmag, float *ymag) {
   
   /* compute current mag */
-  gVw->mag = (((float) gVw->output_width / (float) gVw->video_width ) + 
-	      ((float) gVw->output_height / (float) gVw->video_height )) * .5;
-  return gVw->mag;
+  pthread_mutex_lock(&gVw->mutex);
+  *xmag = (float) gVw->output_width / (float) gVw->video_width; 
+  *ymag = (float) gVw->output_height / (float) gVw->video_height; 
+  pthread_mutex_unlock(&gVw->mutex);
 }
 
 /*
@@ -1993,7 +2118,6 @@ void video_window_change_skins(int synthetic) {
  *
  */
 static void video_window_handle_event (XEvent *event, void *data) {
-
   switch(event->type) {
 
   case DestroyNotify:
@@ -2109,6 +2233,8 @@ static void video_window_handle_event (XEvent *event, void *data) {
       Window           tmp_win;
       int              h, w;
 
+      pthread_mutex_lock(&gVw->mutex);
+
       h = gVw->output_height;
       w = gVw->output_width;
       gVw->output_width  = cev->width;
@@ -2125,15 +2251,24 @@ static void video_window_handle_event (XEvent *event, void *data) {
         gVw->ywin = cev->y;
       }
 
-      if(osd_is_visible() && ((h != gVw->output_height) || (w != gVw->output_width)))
+      /* Keep geometry memory of windowed mode in sync. */
+      if(gVw->fullscreen_mode & WINDOWED_MODE) {
+	gVw->old_win_width  = gVw->win_width  = gVw->output_width;
+	gVw->old_win_height = gVw->win_height = gVw->output_height;
+      }
+
+      if((h != gVw->output_height) || (w != gVw->output_width))
 	osd_update_osd();
 
       oxine_adapt();
+
+      pthread_mutex_unlock(&gVw->mutex);
     }
     break;
     
   }
 
+
 }
 
 void video_window_reset_ssaver(void) {
@@ -2151,20 +2286,51 @@ void video_window_reset_ssaver(void) {
       XLockDisplay(gGui->video_display);
       XTestFakeKeyEvent(gGui->video_display, gVw->fake_keys[gVw->fake_key_cur], True, CurrentTime);
       XTestFakeKeyEvent(gGui->video_display, gVw->fake_keys[gVw->fake_key_cur], False, CurrentTime);
-      XTestFakeKeyEvent(gGui->video_display, gVw->fake_keys[gVw->fake_key_cur], True, CurrentTime);
-      XTestFakeKeyEvent(gGui->video_display, gVw->fake_keys[gVw->fake_key_cur], False, CurrentTime);
       XSync(gGui->video_display, False);
       XUnlockDisplay(gGui->video_display);
     }
     else 
 #endif
+    {
+      /* Reset the gnome screensaver. Look up the command in PATH only once to save time, */
+      /* assuming its location and permission will not change during run time of xine-ui. */
       {
-	XLockDisplay(gGui->video_display);
-	XResetScreenSaver(gGui->video_display);
-	XUnlockDisplay(gGui->video_display);
-      }
-  }
+	static char *gssaver_args[] = { "gnome-screensaver-command", "--poke", NULL };
+	static char *gssaver_path   = NULL;
 
+	if(!gssaver_path) {
+	  char *path = getenv("PATH");
+
+	  if(!path)
+	    path = "/usr/local/bin:/usr/bin";
+	  do {
+	    char *p, pbuf[XITK_PATH_MAX + XITK_NAME_MAX + 2];
+	    int   plen;
+
+	    for(p = path; *path && *path != ':'; path++)
+	      ;
+	    if(p == path)
+	      plen = 1, p = ".";
+	    else
+	      plen = path - p;
+	    plen = snprintf(pbuf, sizeof(pbuf), "%.*s/%s", plen, p, gssaver_args[0]);
+	    if (plen <= -1 || plen >= sizeof(pbuf))
+	      gssaver_path = "";
+	    else
+	      gssaver_path = (access(pbuf, X_OK)) ? "" : strdup(pbuf);
+	  } while(!gssaver_path[0] && *path++);
+	}
+	if(gssaver_path[0] && (fork() == 0)) {
+	  execv(gssaver_path, gssaver_args);
+	  exit(0);
+	}
+      }
+
+      XLockDisplay(gGui->video_display);
+      XResetScreenSaver(gGui->video_display);
+      XUnlockDisplay(gGui->video_display);
+    }
+  }
 }
 
 void video_window_get_frame_size(int *w, int *h) {
