@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: viewlog.h,v 1.11 2004/04/13 19:55:49 f1rmb Exp $
+ * $Id: viewlog.h,v 1.12 2004/06/13 11:44:17 f1rmb Exp $
  *
  */
 
@@ -31,11 +31,11 @@
 #include "xitk.h"
 
 void viewlog_panel(void);
-void viewlog_exit(xitk_widget_t *, void *);
 int viewlog_is_visible(void);
 int viewlog_is_running(void);
 void viewlog_toggle_visibility(xitk_widget_t *, void *);
 void viewlog_raise_window(void);
 void viewlog_reparent(void);
+void viewlog_end(void);
 
 #endif
