@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2007 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: combo.c,v 1.39 2004/04/13 19:55:49 f1rmb Exp $
+ * $Id: combo.c,v 1.41 2007/03/25 22:51:10 fsck-p Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -221,6 +221,18 @@ static void notify_change_skin(xitk_widget_t *w, xitk_skin_config_t *skonfig) {
   }
 }
 
+static void tips_timeout(xitk_widget_t *w, unsigned long timeout) {
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
+	   (w->type & WIDGET_GROUP_WIDGET))) {
+    combo_private_data_t *private_data = (combo_private_data_t *) w->private_data;
+    
+    private_data->combo_widget->tips_timeout = timeout;
+    private_data->label_widget->tips_timeout = timeout;
+    private_data->button_widget->tips_timeout = timeout;
+    private_data->browser_widget->tips_timeout = timeout;
+  }
+}
+
 static int notify_event(xitk_widget_t *w, widget_event_t *event, widget_event_result_t *result) {
   int retval = 0;
   
@@ -237,6 +249,9 @@ static int notify_event(xitk_widget_t *w, widget_event_t *event, widget_event_re
   case WIDGET_EVENT_ENABLE:
     enability(w);
     break;
+  case WIDGET_EVENT_TIPS_TIMEOUT:
+    tips_timeout(w, event->tips_timeout);
+    break;
   }
   
   return retval;
@@ -528,6 +543,7 @@ static xitk_widget_t *_xitk_combo_create(xitk_widget_list_t *wl,
 
   itemw = xitk_get_widget_width(private_data->label_widget);
   itemw += xitk_get_widget_width(private_data->button_widget);
+  itemw -= 2; /* space for border */
 
   private_data->imlibdata                = c->imlibdata;
   private_data->skin_element_name        = (skin_element_name == NULL) ? NULL : strdup(skin_element_name);
@@ -735,7 +751,7 @@ xitk_widget_t *xitk_noskin_combo_create(xitk_widget_list_t *wl,
     
     fs = xitk_font_load_font(c->imlibdata->x.disp, DEFAULT_FONT_10);
     xitk_font_set_font(fs, c->parent_wlist->gc);
-    height = xitk_font_get_string_height(fs, FONT_HEIGHT_MODEL);
+    height = xitk_font_get_string_height(fs, " ") + 4;
     xitk_font_unload_font(fs);
 
 
@@ -748,7 +764,7 @@ xitk_widget_t *xitk_noskin_combo_create(xitk_widget_list_t *wl,
     xitk_list_append_content(c->parent_wlist->l, 
 			     (private_data->label_widget = 
 			      xitk_noskin_label_create(c->parent_wlist, &lbl,
-						       x, y, (width - height), (height + 4), DEFAULT_FONT_10)));
+						       x, y, (width - height), height, DEFAULT_FONT_10)));
     private_data->label_widget->type |= WIDGET_GROUP | WIDGET_GROUP_COMBO;
 
     cb.skin_element_name = NULL;
@@ -759,7 +775,7 @@ xitk_widget_t *xitk_noskin_combo_create(xitk_widget_list_t *wl,
 			     (private_data->button_widget = 
 			      xitk_noskin_checkbox_create(c->parent_wlist, &cb,
 							  x + (width - height), y,
-							  (height + 4), (height + 4))));
+							  height, height)));
     private_data->button_widget->type |= WIDGET_GROUP | WIDGET_GROUP_COMBO;
   
     if(lw)
@@ -769,8 +785,8 @@ xitk_widget_t *xitk_noskin_combo_create(xitk_widget_list_t *wl,
     
     mywidget->x = x;
     mywidget->y = y;
-    mywidget->width = (width - height) + (height + 4);
-    mywidget->height = (height + 4);
+    mywidget->width = width;
+    mywidget->height = height;
     
     {
       xitk_image_t *wimage = xitk_get_widget_foreground_skin(private_data->label_widget);
