@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: config.c,v 1.20 2004/04/13 19:55:49 f1rmb Exp $
+ * $Id: config.c,v 1.23 2006/05/22 17:53:26 mshopf Exp $
  *
  */
 
@@ -115,7 +115,7 @@ static void xitk_config_fonts(xitk_config_t *xtcf) {
       xtcf->fonts.fallback = strdup(c);
     else if(!strncasecmp(p, "system", 6)) {
       xtcf->fonts.system = (char *) realloc(xtcf->fonts.system, (strlen(c) + 1));
-      sprintf(xtcf->fonts.system, "%s", c);
+      strcpy(xtcf->fonts.system, c);
     }
     else if(!strncasecmp(p, "xmb", 3)) {
       xtcf->fonts.xmb = ((strtol(c, &c, 10)) >= 1) ? 1 : 0;
@@ -181,6 +181,8 @@ static void xitk_config_features(xitk_config_t *xtcf) {
       else if(!strcasecmp(c, "check"))
 	xtcf->features.checkstyle = CHECK_STYLE_CHECK;
     }
+    else if(!strncasecmp(p, "cursors", 7))
+      xtcf->features.cursors = xitk_get_bool_value(c);
     else if(!strncasecmp(p, "shm", 3))
       xtcf->features.shm = strtol(c, &c, 10);
   }
@@ -328,13 +330,14 @@ static void xitk_config_init_default_values(xitk_config_t *xtcf) {
   xtcf->colors.warn_background = -1;
   xtcf->timers.label_anim      = 50000;
   xtcf->timers.dbl_click       = 400;
-#ifdef HAVE_SHM
-  xtcf->features.shm           = 1;
-#else
+#ifdef DISABLE_SHM_DEFAULT
   xtcf->features.shm           = 0;
+#else
+  xtcf->features.shm           = 1;
 #endif
   xtcf->features.oldbarstyle   = 0;
   xtcf->features.checkstyle    = CHECK_STYLE_CHECK;
+  xtcf->features.cursors       = 0;
   xtcf->menus.shortcuts        = 1;
 }
 
@@ -362,6 +365,13 @@ int xitk_config_get_shm_feature(xitk_config_t *xtcf) {
   
   return (xtcf->features.shm > 0) ? 1 : 0;
 }
+int xitk_config_get_cursors_feature(xitk_config_t *xtcf) {
+
+  if(!xtcf)
+    return -1;
+  
+  return (xtcf->features.cursors > 0) ? 1 : 0;
+}
 char *xitk_config_get_system_font(xitk_config_t *xtcf) {
 
   if(!xtcf)
@@ -478,7 +488,7 @@ xitk_config_t *xitk_config_init(void) {
     sprintf(rcfile, "%s/.xine/%s", xitk_get_homedir(), user_rc);
     if((xtcf->fd = fopen(rcfile, "r")) == NULL) {
       rcfile = (char *) realloc(rcfile, (strlen(SYSTEM_RC) + 1));
-      sprintf(rcfile, "%s", SYSTEM_RC);
+      strcpy(rcfile, SYSTEM_RC);
       if((xtcf->fd = fopen(rcfile, "r")) == NULL) {
         XITK_FREE(rcfile);
         rcfile = NULL;
