@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2004 the xine project
+ * Copyright (C) 2004-2006 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: cursors.c,v 1.3 2004/07/26 01:00:11 f1rmb Exp $
+ * $Id: cursors.c,v 1.4 2006/01/07 20:14:38 valtri Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -29,7 +29,7 @@
 
 #include "_xitk.h"
 
-#define MAX_CURSORS  (xitk_cursor_num_glyphs + 1)
+#define MAX_CURSORS  xitk_cursor_num_glyphs
 
 #define X_CURSOR     0
 #define XITK_CURSOR  1
