@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: dnd.c,v 1.23 2004/04/13 19:55:49 f1rmb Exp $
+ * $Id: dnd.c,v 1.25 2006/06/19 13:56:50 dgp85 Exp $
  *
  *
  * Thanks to Paul Sheer for his nice xdnd implementation in cooledit.
@@ -108,7 +108,7 @@ static int _dnd_paste_prop_internal(xitk_dnd_t *xdnd, Window from,
       char  buf[nread + 2];
       
       memset(&buf, '\0', sizeof(buf));
-      sprintf(buf, "%s", s);
+      snprintf(buf, sizeof(buf), "%s", s);
       
       if(strlen(buf)) {
 	char *p, *pbuf;
@@ -228,8 +228,8 @@ static void _dnd_get_selection (xitk_dnd_t *xdnd, Window from, Atom prop, Window
  */
 static void _dnd_get_type_list (xitk_dnd_t *xdnd, Window window, Atom **typelist) {
   Atom            type, *a;
-  int             format, i;
-  unsigned long   count, remaining;
+  int             format;
+  unsigned long   count, remaining, i;
   unsigned char  *data = NULL;
   
   *typelist = 0;
