@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2007 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: font.c,v 1.41 2005/01/06 17:46:56 valtri Exp $
+ * $Id: font.c,v 1.47 2007/03/25 22:53:06 fsck-p Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -284,8 +284,7 @@ static const lang_locale_t *_get_first_lang_locale(char *lcal) {
   return NULL;
 }
 
-#ifdef WITH_XMB
-# ifndef WITH_XFT
+#if defined(WITH_XMB) && !defined(WITH_XFT)
 /*
  * Guess if error occured, release missing charsets list.
  * There is a problem with rendering when "ISO10646-1" charset is missing.
@@ -311,54 +310,69 @@ static int xitk_font_guess_error(XFontSet fs, char *name, char **missing, int co
 
   return 0;
 }
-# endif
+#endif
 
+#ifdef WITH_XFT
 /* convert a -*-* .. style font description into something Xft can digest */
 static char * xitk_font_core_string_to_xft( char * old_name) {
   static char new_name[255];
 
-  if( strncmp(old_name, "-*", 2) == 0 || strncmp(old_name, "*-", 2) == 0
-      || strncmp(old_name, "-", 1) == 0 || strncmp(old_name, "*", 1) == 0) {
-    char  font[50];
-    char  style[15];
-    char  *psize = "*", size[5], ptsize[5], pxsize[5];
+  if(old_name[0] == '-' || old_name[0] == '*') {
+    char  family[50], weight[15], slant[8], pxsize[5], ptsize[5];
     
     if( old_name[0] == '-' )
       old_name++;
     
-    sscanf( old_name, "%*[^-]-%[^-]", font );
-    sscanf( old_name, "%*[^-]-%*[^-]-%[^-]", style );
-    sscanf( old_name, "%*[^-]-%*[^-]-%*[^-]-%*[^-]-%*[^-]-%*[^-]-%[^-]", pxsize);
-    sscanf( old_name, "%*[^-]-%*[^-]-%*[^-]-%*[^-]-%*[^-]-%*[^-]-%*[^-]-%[^-]", ptsize);
+    /* Extract description elements */
+    family[0] = weight[0] = slant[0] = pxsize[0] = ptsize[0] = '\0';
+    sscanf(old_name, "%*[^-]-%49[^-]", family);
+    sscanf(old_name, "%*[^-]-%*[^-]-%14[^-]", weight);
+    sscanf(old_name, "%*[^-]-%*[^-]-%*[^-]-%7[^-]", slant);
+    sscanf(old_name, "%*[^-]-%*[^-]-%*[^-]-%*[^-]-%*[^-]-%*[^-]-%4[^-]", pxsize);
+    sscanf(old_name, "%*[^-]-%*[^-]-%*[^-]-%*[^-]-%*[^-]-%*[^-]-%*[^-]-%4[^-]", ptsize);
+
+    /* Transform the elements */
+    if(!strcmp(weight, "*"))
+      weight[0] = '\0';
+    if(!strcmp(pxsize, "*"))
+      pxsize[0] = '\0';
+    if(!strcmp(ptsize, "*"))
+      ptsize[0] = '\0';
+    strcpy(slant,
+	   (!strcmp(slant, "i")) ? "italic" :
+	   (!strcmp(slant, "o")) ? "oblique" :
+	   (!strcmp(slant, "r")) ? "roman" :
+				   "");
 
     /* Xft doesn't have lucida, which is a small font;
      * thus we make whatever is chosen 2 sizes smaller */
-    if(strlen(pxsize) && strcmp(pxsize, "*"))
-      psize = pxsize;
-    else if(strlen(ptsize) && strcmp(ptsize, "*"))
-      psize = ptsize;
-
-    if( strcmp( font, "lucida" ) == 0 ) {
-      int  sz = strtol(psize, &psize, 10);
-
-      sz -= 2;
-      snprintf(size , sizeof(size), "%i", sz);
+    if(!strcasecmp(family, "lucida")) {
+      if(pxsize[0]) {
+	int sz = strtol(pxsize, NULL, 10) - 2;
+	snprintf(pxsize, sizeof(pxsize), "%i", sz);
+      }
+      if(ptsize[0]) {
+	int sz = strtol(ptsize, NULL, 10) - 2;
+	snprintf(ptsize, sizeof(ptsize), "%i", sz);
+      }
     }
-    else
-      snprintf(size, sizeof(size), "%s", psize);
     
-    if( strcmp( style , "bold" ) != 0 )
-      snprintf( new_name, sizeof(new_name), "%s-%s", font, size );
-    else
-      snprintf( new_name, sizeof(new_name), "%s-%s:%s", font, size, style );
- 
+    /* Build Xft font description string from the elements */
+    snprintf(new_name, sizeof(new_name), "%s%s%s%s%s%s%s%s%s",
+	     family,
+	     ((weight[0]) ? ":weight="    : ""), weight,
+	     ((slant[0])  ? ":slant="     : ""), slant,
+	     ((pxsize[0]) ? ":pixelsize=" : ""), pxsize,
+	     ((ptsize[0]) ? ":size="      : ""), ptsize);
+
     return new_name;
   }
 
   return old_name;
 }
+#endif
 
-# ifndef WITH_XFT
+#if defined(WITH_XMB) && !defined(WITH_XFT)
 /*
  * XCreateFontSet requires font name starting with '-'
  */
@@ -377,7 +391,6 @@ static char *xitk_font_right_name(char *name) {
 
   return strdup(right_name);
 }
-# endif
 #endif
 
 /*
@@ -837,7 +850,7 @@ void xitk_font_draw_string(xitk_font_t *xtfs, Pixmap pix, GC gc,
     xr_color.alpha = (short)-1;
     xft_draw       = XftDrawCreate(xtfs->display, pix, visual, colormap);
     XftColorAllocValue(xtfs->display, visual, colormap, &xr_color, &xcolor);
-    XftDrawStringUtf8(xft_draw, &xcolor, xtfs->font, x, y, encoded_text, strlen(encoded_text));
+    XftDrawStringUtf8(xft_draw, &xcolor, xtfs->font, x, y, (FcChar8*)encoded_text, strlen(encoded_text));
     XftColorFree(xtfs->display, visual, colormap, &xcolor);
     XftDrawDestroy(xft_draw);
 
@@ -999,13 +1012,11 @@ int xitk_font_get_char_width(xitk_font_t *xtfs, char *c, int maxnbytes, int *nby
  *
  */
 int xitk_font_get_text_height(xitk_font_t *xtfs, const char *c, int nbytes) {
-  int lbearing, rbearing, width, ascent, descent, height;
+  int ascent, descent;
   
-  xitk_font_text_extent(xtfs, c, nbytes, &lbearing, &rbearing, &width, &ascent, &descent);
+  xitk_font_text_extent(xtfs, c, nbytes, NULL, NULL, NULL, &ascent, &descent);
  
-  height = ascent + descent;
- 
-  return height;
+  return (ascent + descent);
 }
 
 /*
@@ -1055,15 +1066,9 @@ void xitk_font_text_extent(xitk_font_t *xtfs, const char *c, int nbytes,
   char       *foo_text, *encoded_text;
 #endif
 #if defined(WITH_XMB) && !defined(WITH_XFT)
-  XRectangle  logic;
+  XRectangle  ink, logic;
 #endif
   
-#ifdef DEBUG
-  if (nbytes > strlen(c) + 1) {
-    XITK_WARNING("extent: %d > %d\n", nbytes, strlen(c));
-  }
-#endif
-
   if (nbytes <= 0) {
 #ifdef DEBUG
     if (nbytes < 0)
@@ -1077,13 +1082,19 @@ void xitk_font_text_extent(xitk_font_t *xtfs, const char *c, int nbytes,
     return;
   }
     
+#ifdef DEBUG
+  if ((size_t)nbytes > strlen(c) + 1) {
+    XITK_WARNING("extent: %d > %d\n", nbytes, strlen(c));
+  }
+#endif
+
 #ifdef WITH_XFT
   ABORT_IF_NULL(xtfs);
   ABORT_IF_NULL(xtfs->font);
   ABORT_IF_NULL(c);
 
   /* recode right part of string */
-  if (nbytes > strlen(c))
+  if ((size_t)nbytes > strlen(c))
     nbytes = strlen(c);
 
   foo_text = strdup(c);
@@ -1093,15 +1104,20 @@ void xitk_font_text_extent(xitk_font_t *xtfs, const char *c, int nbytes,
   free(foo_text);
   
   XLOCK(xtfs->display);
-  XftTextExtentsUtf8(xtfs->display, xtfs->font, encoded_text, nbytes, &xft_extents);
+  XftTextExtentsUtf8(xtfs->display, xtfs->font, (FcChar8*)encoded_text, nbytes, &xft_extents);
   XUNLOCK(xtfs->display);
   free(encoded_text);
 
-  if (width) *width       = xft_extents.width;
-  if (ascent) *ascent     = xtfs->font->ascent - 1;
-  if (descent) *descent   = xtfs->font->descent;
-  if (lbearing) *lbearing = 0;
-  if (rbearing) *rbearing = 0;
+  if (width) *width       = xft_extents.xOff;
+  /* Since Xft fonts have more top and bottom space rows than Xmb/Xcore fonts, */
+  /* we tweak ascent and descent to appear the same like Xmb/Xcore, so we get  */
+  /* unified vertical text positions. We must however be aware to eventually   */
+  /* reserve space for these rows when drawing the text.                       */
+  /* Following trick works for our font sizes 10...14.                         */
+  if (ascent) *ascent     = xtfs->font->ascent - (xtfs->font->ascent - 9) / 2;
+  if (descent) *descent   = xtfs->font->descent - (xtfs->font->descent - 0) / 2;
+  if (lbearing) *lbearing = -xft_extents.x;
+  if (rbearing) *rbearing = -xft_extents.x + xft_extents.width;
 #else
 #ifdef WITH_XMB
   if(xitk_get_xmb_enability()) {
@@ -1110,42 +1126,49 @@ void xitk_font_text_extent(xitk_font_t *xtfs, const char *c, int nbytes,
     ABORT_IF_NULL(c);
     
     XLOCK(xtfs->display);
-    XmbTextExtents(xtfs->fontset, c, nbytes, NULL, &logic);
+    XmbTextExtents(xtfs->fontset, c, nbytes, &ink, &logic);
     XUNLOCK(xtfs->display);
     if (!logic.width || !logic.height) {
       /* XmbTextExtents() has problems, try char-per-char counting */
       mbstate_t state;
       size_t    i = 0, n;
-      int       height = 0, width = 0;
+      int       height = 0, width = 0, y = 0, lb = 0, rb;
       
       memset(&state, '\0', sizeof(mbstate_t));
       
       XLOCK(xtfs->display);
       while (i < nbytes) {
-	n = mbrtowc(NULL, c, nbytes - i, &state);
+	n = mbrtowc(NULL, c + i, nbytes - i, &state);
 	if (n == (size_t)-1 || n == (size_t)-2 || i + n > nbytes) n = 1;
-	XmbTextExtents(xtfs->fontset, c + i, n, NULL, &logic);
+	XmbTextExtents(xtfs->fontset, c + i, n, &ink, &logic);
 	if (logic.height > height) height = logic.height;
+	if (logic.y < y) y = logic.y; /* y is negative! */
+	if (i == 0) lb = ink.x; /* left bearing of string */
 	width += logic.width;
 	i += n;
       }
+      rb = width - logic.width + ink.x + ink.width; /* right bearing of string */
       XUNLOCK(xtfs->display);
       
       if (!height || !width) {
 	/* char-per-char counting fails too */
 	XITK_WARNING("extents of the font \"%s\" are %dx%d!\n", xtfs->name, width, height);
-	if (!height) height = 12;
+	if (!height) height = 12, y = -10;
+	if (!width) lb = rb = 0;
       } 
       
       logic.height = height;
       logic.width = width;
+      logic.y = y;
+      ink.x = lb;
+      ink.width = rb - lb;
     }
     
     if (width) *width     = logic.width;
     if (ascent) *ascent   = -logic.y;
     if (descent) *descent = logic.height + logic.y;
-    if (lbearing) *lbearing = 0;
-    if (rbearing) *rbearing = 0;
+    if (lbearing) *lbearing = ink.x;
+    if (rbearing) *rbearing = ink.x + ink.width;
   }
   else
 #endif
@@ -1195,7 +1218,7 @@ void xitk_font_string_extent(xitk_font_t *xtfs, const char *c,
  *
  */
 int xitk_font_get_ascent(xitk_font_t *xtfs, const char *c) {
-  int lbearing, rbearing, width, ascent, descent;
+  int ascent;
   
   ABORT_IF_NULL(xtfs);
   ABORT_IF_NULL(xtfs->display);
@@ -1209,7 +1232,7 @@ int xitk_font_get_ascent(xitk_font_t *xtfs, const char *c) {
 #endif
     ABORT_IF_NULL(xtfs->font);
   
-  xitk_font_text_extent(xtfs, c, strlen(c), &lbearing, &rbearing, &width, &ascent, &descent);
+  xitk_font_text_extent(xtfs, c, strlen(c), NULL, NULL, NULL, &ascent, NULL);
 
   return ascent;
 }
@@ -1218,7 +1241,7 @@ int xitk_font_get_ascent(xitk_font_t *xtfs, const char *c) {
  *
  */
 int xitk_font_get_descent(xitk_font_t *xtfs, const char *c) {
-  int lbearing, rbearing, width, ascent, descent;
+  int descent;
  
   ABORT_IF_NULL(xtfs);
   ABORT_IF_NULL(xtfs->display);
@@ -1232,7 +1255,7 @@ int xitk_font_get_descent(xitk_font_t *xtfs, const char *c) {
 #endif
     ABORT_IF_NULL(xtfs->font);
   
-  xitk_font_text_extent(xtfs, c, strlen(c), &lbearing, &rbearing, &width, &ascent, &descent);
+  xitk_font_text_extent(xtfs, c, strlen(c), NULL, NULL, NULL, NULL, &descent);
 
   return descent;
 }
