@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2007 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: image.c,v 1.57 2004/04/13 19:55:49 f1rmb Exp $
+ * $Id: image.c,v 1.73 2007/03/29 23:07:56 fsck-p Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -27,9 +27,12 @@
 #include <stdio.h>
 #include <stdarg.h>
 #include <X11/Xlib.h>
+#include <errno.h>
 
 #include "_xitk.h"
 
+int xitk_x_error = 0;
+
 /*
  * Get a pixel color from rgb values.
  */
@@ -79,13 +82,13 @@ unsigned int xitk_get_pixel_color_gray(ImlibData *im) {
   int user_color = xitk_get_background_color();
   if(user_color >= 0)
     return (unsigned int) user_color;
-  return xitk_get_pixel_color_from_rgb(im, 192, 192, 192);
+  return xitk_get_pixel_color_from_rgb(im, 190, 190, 190);
 }
 unsigned int xitk_get_pixel_color_darkgray(ImlibData *im) {
   int user_color = xitk_get_select_color();
   if(user_color >= 0)
     return (unsigned int) user_color;
-  return xitk_get_pixel_color_from_rgb(im, 160, 160, 160);
+  return xitk_get_pixel_color_from_rgb(im, 128, 128, 128);
 }
 unsigned int xitk_get_pixel_color_warning_foreground(ImlibData *im) {
   int user_color = xitk_get_warning_foreground();
@@ -139,29 +142,35 @@ Pixmap xitk_image_create_pixmap(ImlibData *im, int width, int height) {
  *
  */
 static void xitk_image_xitk_pixmap_destroyer(xitk_pixmap_t *xpix) {
-
   ABORT_IF_NULL(xpix);
 
   XLOCK(xpix->imlibdata->x.disp);
-
-#ifdef HAVE_SHM
-  if(xpix->shm)
-    XShmDetach(xpix->imlibdata->x.disp, xpix->shminfo);
-
-  if(xpix->xim)
-    XDestroyImage(xpix->xim);
-
-  if(xpix->shm) {
-    shmdt(xpix->shminfo->shmaddr);
-    shmctl(xpix->shminfo->shmid, IPC_RMID, 0);
-    free(xpix->shminfo);
-  }
-#endif
   
   if(xpix->pixmap != None)
     XFreePixmap(xpix->imlibdata->x.disp, xpix->pixmap);
   
   XFreeGC(xpix->imlibdata->x.disp, xpix->gc);
+  XSync(xpix->imlibdata->x.disp, False);
+
+#ifdef HAVE_SHM
+  if(xpix->shm) {
+    XShmSegmentInfo *shminfo = xpix->shminfo;
+
+    XShmDetach(xpix->imlibdata->x.disp, shminfo);
+    
+    if(xpix->xim)
+      XDestroyImage(xpix->xim);
+    
+    if(shmdt(shminfo->shmaddr) < 0)
+      XITK_WARNING("shmdt() failed: '%s'\n", strerror(errno));
+
+    if(shmctl(shminfo->shmid, IPC_RMID, 0) < 0)
+      XITK_WARNING("shmctl() failed: '%s'\n", strerror(errno));
+    
+
+    free(shminfo);
+  }
+#endif
   
   XUNLOCK(xpix->imlibdata->x.disp);
   
@@ -171,18 +180,17 @@ static void xitk_image_xitk_pixmap_destroyer(xitk_pixmap_t *xpix) {
 /*
  *
  */
-xitk_pixmap_t *xitk_image_create_xitk_pixmap_with_depth(ImlibData *im, 
-							int width, int height, int depth) {
+xitk_pixmap_t *xitk_image_create_xitk_pixmap_with_depth(ImlibData *im, int width, int height, int depth) {
   xitk_pixmap_t    *xpix;
 #ifdef HAVE_SHM
   XShmSegmentInfo  *shminfo;
 #endif
-  
+
   ABORT_IF_NULL(im);
   ABORT_IF_NOT_COND(width > 0);
   ABORT_IF_NOT_COND(height > 0);
   
-  xpix = (xitk_pixmap_t *) xitk_xmalloc(sizeof(xitk_pixmap_t));
+  xpix            = (xitk_pixmap_t *) xitk_xmalloc(sizeof(xitk_pixmap_t));
   xpix->imlibdata = im;
   xpix->destroy   = xitk_image_xitk_pixmap_destroyer;
   xpix->width     = width;
@@ -242,10 +250,10 @@ xitk_pixmap_t *xitk_image_create_xitk_pixmap_with_depth(ImlibData *im,
       goto __noxshm_pixmap;
     }
 
-    xpix->xim = xim;
+    xpix->xim    = xim;
     xpix->pixmap = XShmCreatePixmap(im->x.disp, im->x.base_window, 
 				    shminfo->shmaddr, shminfo, width, height, depth);
-
+    
     if(!xpix->pixmap) {
       XITK_WARNING("XShmCreatePixmap() failed.\n");
       XShmDetach(xpix->imlibdata->x.disp, xpix->shminfo);
@@ -257,17 +265,10 @@ xitk_pixmap_t *xitk_image_create_xitk_pixmap_with_depth(ImlibData *im,
       goto __noxshm_pixmap;
     }
     else {
-      xpix->shm = 1;
-      //      XDestroyImage(xim);
-      xpix->shminfo = shminfo;
+      xpix->shm                    = 1;
+      xpix->shminfo                = shminfo;
       xpix->gcv.graphics_exposures = False;
-      xpix->gc = XCreateGC(im->x.disp, xpix->pixmap, GCGraphicsExposures, &xpix->gcv);
-      shmctl(shminfo->shmid, IPC_RMID, 0);
-
-      //      XShmPutImage(im->x.disp, xpix->pixmap, xpix->gc, xpix->xim, 
-      //		   0, 0, 0, 0, width, height, False);
-      //      XSync(im->x.disp, False);
-      
+      xpix->gc                     = XCreateGC(im->x.disp, xpix->pixmap, GCGraphicsExposures, &xpix->gcv);
       xitk_uninstall_x_error_handler();
     }
   }
@@ -303,6 +304,7 @@ void xitk_image_destroy_xitk_pixmap(xitk_pixmap_t *p) {
   p->destroy(p);
 }
 
+#if 0
 /*
  *
  */
@@ -319,6 +321,7 @@ Pixmap xitk_image_create_mask_pixmap(ImlibData *im, int width, int height) {
 
   return p;
 }
+#endif
 
 /*
  *
@@ -372,32 +375,35 @@ xitk_image_t *xitk_image_create_image(ImlibData *im, int width, int height) {
   ABORT_IF_NOT_COND(width > 0);
   ABORT_IF_NOT_COND(height > 0);
 
-  i = (xitk_image_t *) xitk_xmalloc(sizeof(xitk_image_t));
-  i->mask = NULL;
-  i->image = xitk_image_create_xitk_pixmap(im, width, height);
-  i->width = width;
+  i         = (xitk_image_t *) xitk_xmalloc(sizeof(xitk_image_t));
+  i->mask   = NULL;
+  i->image  = xitk_image_create_xitk_pixmap(im, width, height);
+  i->width  = width;
   i->height = height;
 
   return i;
 }
 
-
 /*
  *
  */
 xitk_image_t *xitk_image_create_image_with_colors_from_string(ImlibData *im, 
-							      char *fontname, 
-							      int width, int align, char *str,
-							      unsigned int foreground,
-							      unsigned int background) {
+                                                              char *fontname, 
+                                                              int width, int align, char *str,
+                                                              unsigned int foreground,
+                                                              unsigned int background) {
   xitk_image_t   *image;
   xitk_font_t    *fs;
   GC              gc;
-  int             length, height, descent;
-  char           *po, *p;
+  int             length, height, lbearing, rbearing, ascent, descent, linel, linew, wlinew, lastws;
+  int             maxw = 0;
+  const char     *p;
+  char           *bp;
   char           *lines[256];
   int             numlines = 0;
-  char            bufsubs[BUFSIZ], buf[BUFSIZ];
+  char            buf[BUFSIZ]; /* Could be allocated dynamically for bigger texts */
+
+  int             add_line_spc = 2;
 
   ABORT_IF_NULL(im);
   ABORT_IF_NULL(fontname);
@@ -410,110 +416,145 @@ xitk_image_t *xitk_image_create_image_with_colors_from_string(ImlibData *im,
   
   fs = xitk_font_load_font(im->x.disp, fontname);
   xitk_font_set_font(fs, gc);
-  length = xitk_font_get_string_length(fs, str);
-  height = xitk_font_get_string_height(fs, str);
-  descent = xitk_font_get_descent(fs, str);
+  xitk_font_string_extent(fs, str, NULL, NULL, NULL, &ascent, &descent);
+  height = ascent + descent;
 
-  memset(&bufsubs, 0, sizeof(bufsubs));
-  memset(&buf, 0, sizeof(buf));
-  p = str;
+  p      = str;
+  bp     = buf;
+  wlinew = linew = lastws = linel = 0;
 
-  while(*p != '\0') {
+  /*
+   * Create string image using exactly the ink width
+   * without left and right spacing measures
+   */
+
+  while((*p!='\0') && (bp + linel) < (buf + BUFSIZ - TABULATION_SIZE - 2)) {
+    
     switch(*p) {
       
     case '\t':
-      sprintf(bufsubs, "%s%s", bufsubs, "      ");
+      {
+	int a;
+	
+	if((linel == 0) || (bp[linel - 1] != ' ')) {
+	  lastws = linel;
+	  wlinew = linew; /* width if wrapped */
+	}
+	
+	a = TABULATION_SIZE - (linel % TABULATION_SIZE);
+	
+        while(a--)
+          bp[linel++] = ' ';
+      }
       break;
-      
-    default:
-      sprintf(bufsubs, "%s%c", bufsubs, *p);
-      break;
-    }
-    p++;
-  }
 
-  po = p = &bufsubs[0];
-
-  while(*p != '\0') {
-
-    switch(*p) {
-      
-      /* Ignore */
-    case '\a':
-    case '\b':
-    case '\f':
-    case '\r':
-    case '\v':
-      break;
-      
+      case '\a':
+      case '\b':
+      case '\f':
+      case '\r':
+      case '\v':
+        break; /* Ignore those */
+	
     case '\n':
-      lines[numlines++] = strdup(buf);
-      po = p + 1;
-      memset(&buf, 0, sizeof(buf));
+      lines[numlines++] = bp;
+      bp                = bp + linel + 1;
+      
+      wlinew = lastws = linel = 0;
+      
+      if(linew > maxw)
+	maxw = linew;
       break;
+      
+    case ' ':
+      if((linel == 0) || (bp[linel-1] != ' ')) {
+	lastws = linel;
+	wlinew = linew; /* width if wrapped */
+      }
 
+      /* fall through */
     default:
-      sprintf(buf, "%s%c", buf, *p);
+      bp[linel++] = *p;
+
+      if(!lastws)
+	wlinew = linew;
       break;
     }
 
-    if(xitk_font_get_string_length(fs, buf) >= width) {
-      char buf2[BUFSIZ];
-      char *ps;
+    bp[linel] = '\0'; /* terminate the string for reading with xitk_font_string_extent or strlen */
 
-      memset(&buf2, 0, sizeof(buf2));
-      /* step back */
-      if((ps = strrchr(buf, ' ')) != NULL) { /* Ok, there a space here */
-	ps = p;
-	while(*ps != ' ') ps--;
-	snprintf(buf2, (ps - po)+1, "%s", buf);
-	lines[numlines++] = strdup(buf2);
-	p = po = ps;
+    xitk_font_string_extent(fs, bp, &lbearing, &rbearing, NULL, NULL, NULL); 
+    if((linew = rbearing - lbearing) > width) {
+      
+      if(lastws == 0) { /* if we haven't found a whitespace */
+        bp[linel]         = bp[linel-1]; /* Move that last character down */
+        bp[linel-1]       = 0;
+        lines[numlines++] = bp;
+        bp                += linel;
+        linel             = 1;
       }
-      else { /* cut to previous char */
-	snprintf(buf2, (p - po), "%s", buf);
-	sprintf(buf2, "%s%c", buf2, '-');
-	lines[numlines++] = strdup(buf2);
-	po = (p - 1);
-	p -= 2;
+      else {
+        char *nextword = (bp + lastws);
+        int   wordlen;
+	
+        while(*nextword == ' ')
+          nextword++;
+	
+        wordlen           = (bp + linel) - nextword;
+        bp[lastws]        = '\0';
+        lines[numlines++] = bp;
+        bp                = bp + lastws + 1;
+	
+        if(bp != nextword)
+          memmove(bp, nextword, wordlen + 1);
+	
+        linel = wordlen;
       }
-      memset(&buf, 0, sizeof(buf));
+      
+      if(wlinew > maxw)
+        maxw = wlinew;
+      
+      xitk_font_string_extent(fs, bp, &lbearing, &rbearing, NULL, NULL, NULL); 
+      linew = rbearing - lbearing;
     }
-
+    
     p++;
   }
-
-  /* Last chars aren't stored */
-  if(strlen(buf))
-    lines[numlines++] = strdup(buf);
   
-  image = xitk_image_create_image(im, width, ((height * numlines) + (3 * numlines)));
+  if(linel) { /* In case last chars aren't stored */
+    lines[numlines++] = bp;
+    
+    if(linew > maxw)
+      maxw = linew;
+  }
+  
+  /* If default resp. left aligned, we may shrink the image */
+  if((align == ALIGN_DEFAULT) || (align == ALIGN_LEFT))
+    width = MIN(maxw, width);
+  
+  image = xitk_image_create_image(im, width, (height + add_line_spc) * numlines - add_line_spc);
   draw_flat_with_color(im, image->image, image->width, image->height, background);
   
   { /* Draw string in image */
-    int i, j, x = 0;
-
+    int i, y, x = 0;
+    
     XLOCK(im->x.disp);
     XSetForeground(im->x.disp, gc, foreground);
     XUNLOCK(im->x.disp);
     
-    j = height;
-    for(i = 0; i < numlines; i++, j += (height + 3)) {
-      length = xitk_font_get_string_length(fs, lines[i]);
+    for(y = ascent, i = 0; i < numlines; i++, y += (height + add_line_spc)) {
+      xitk_font_string_extent(fs, lines[i], &lbearing, &rbearing, NULL, NULL, NULL); 
+      length = rbearing - lbearing;
 
       if((align == ALIGN_DEFAULT) || (align == ALIGN_LEFT))
-	x = 0;
+        x = 0;
       else if(align == ALIGN_CENTER)
-	x = (width - length)>>1;
+        x = (width - length) >> 1;
       else if(align == ALIGN_RIGHT)
-	x = (width - length);
+        x = (width - length);
       
-      XLOCK(im->x.disp);
       xitk_font_draw_string(fs, image->image->pixmap, gc, 
-		  x, (j - descent), lines[i], strlen(lines[i]));
-      XUNLOCK(im->x.disp);
-	
-      XITK_FREE(lines[i]);
+			    (x - lbearing), y, lines[i], strlen(lines[i]));
+			    /*   ^^^^^^^^ Adjust to start of ink */
     }
   }
 
@@ -528,7 +569,7 @@ xitk_image_t *xitk_image_create_image_with_colors_from_string(ImlibData *im,
 xitk_image_t *xitk_image_create_image_from_string(ImlibData *im, 
 						  char *fontname, 
 						  int width, int align, char *str) {
-  
+
   return xitk_image_create_image_with_colors_from_string(im,fontname, width, align, str,
 							 xitk_get_pixel_color_black(im),
 							 xitk_get_pixel_color_gray(im));
@@ -555,9 +596,9 @@ void menu_draw_arrow_branch(ImlibData *im, xitk_image_t *p) {
   int            w;
   int            h;
   XPoint         points[4];
-  int            i, offset = 0;
-  short          x1, x2, x3;
-  short          y1, y2, y3;
+  int            i;
+  int            x1, x2, x3;
+  int            y1, y2, y3;
   
   ABORT_IF_NULL(im);
   ABORT_IF_NULL(p);
@@ -565,20 +606,20 @@ void menu_draw_arrow_branch(ImlibData *im, xitk_image_t *p) {
   w = p->width / 3;
   h = p->height;
   
+  x1 = (w - 5);
+  y1 = (h / 2); 
+
+  x2 = (w - 10);
+  y2 = ((h / 2) + 5);
+
+  x3 = (w - 10);
+  y3 = ((h / 2) - 5);
+  
   XLOCK(im->x.disp);
   XSetForeground(im->x.disp, p->image->gc, xitk_get_pixel_color_black(im));
   XUNLOCK(im->x.disp);
-  
-  for(i = 0; i < 3; i++) {
 
-    x1 = (w - 10) + offset;
-    y1 = (h / 4);
-    
-    x2 = (w - 10) + offset;
-    y2 = ((h / 4) * 3);
-    
-    x3 = (w - 4) + offset;
-    y3 = (h / 2); 
+  for(i = 0; i < 3; i++) {
     
     if(i == 2) {
       x1++; x2++; x3++;
@@ -593,13 +634,15 @@ void menu_draw_arrow_branch(ImlibData *im, xitk_image_t *p) {
     points[2].y = y3;
     points[3].x = x1;
     points[3].y = y1;
-    
-    offset += w;
 
     XLOCK(im->x.disp);
     XFillPolygon(im->x.disp, p->image->pixmap, p->image->gc, 
-		 &points[0], 4, Complex, CoordModeOrigin);
+		 &points[0], 4, Convex, CoordModeOrigin);
     XUNLOCK(im->x.disp);
+
+    x1 += w;
+    x2 += w;
+    x3 += w;
   }
 
 }
@@ -610,10 +653,11 @@ void menu_draw_arrow_branch(ImlibData *im, xitk_image_t *p) {
 static void _draw_arrow(ImlibData *im, xitk_image_t *p, int direction) {
   int            w;
   int            h;
-  XPoint         points[4];
-  int            i, offset = 0;
-  short          x1, x2, x3;
-  short          y1, y2, y3;
+  XSegment      *segments;
+  int            nsegments;
+  int            i, s;
+  int            x1, x2, dx;
+  int            y1, y2, dy;
 
   ABORT_IF_NULL(im);
   ABORT_IF_NULL(p);
@@ -621,79 +665,118 @@ static void _draw_arrow(ImlibData *im, xitk_image_t *p, int direction) {
   w = p->width / 3;
   h = p->height;
 
+  /*
+   * XFillPolygon doesn't yield equally shaped arbitrary sized small triangles
+   * because of its filling algorithm (see also fill-rule in XCreateGC(3X11)
+   * as for which pixels are drawn on the boundary).
+   * So we handcraft them using XDrawSegments applying Bresenham's algorithm.
+   */
+
+  /* Coords of the enclosing rectangle for the triangle:   */
+  /* Pay attention to integer precision loss and calculate */
+  /* carefully to obtain symmetrical and centered shapes.  */
+  x1 = ((w - 1) / 2) - (w / 4);
+  x2 = ((w - 1) / 2) + (w / 4);
+  y1 = ((h - 1) / 2) - (h / 4);
+  y2 = ((h - 1) / 2) + (h / 4);
+
+  dx = (x2 - x1 + 1);
+  dy = (y2 - y1 + 1);
+
+  if(direction == DIRECTION_UP || direction == DIRECTION_DOWN) {
+    int y, iy, dd;
+
+    nsegments = dy;
+    segments = (XSegment *) xitk_xmalloc(sizeof(XSegment) * nsegments);
+
+    if(direction == DIRECTION_DOWN) {
+      y = y1; iy = 1;
+    }
+    else {
+      y = y2; iy = -1;
+    }
+    dx = (dx + 1) / 2;
+    dd = 0;
+    for(s = 0; s < nsegments; s++) {
+      segments[s].y1 = y; segments[s].x1 = x1;
+      segments[s].y2 = y; segments[s].x2 = x2;
+      y += iy;
+      if(dy >= dx) {
+	if((dd += dx) >= dy) {
+	  x1++; x2--;
+	  dd -= dy;
+	}
+      }
+      else {
+	do {
+	  x1++; x2--;
+	} while((dd += dy) < dx);
+	dd -= dx;
+      }
+    }
+  }
+  else if(direction == DIRECTION_LEFT || direction == DIRECTION_RIGHT) {
+    int x, ix, dd;
+
+    nsegments = dx;
+    segments = (XSegment *) xitk_xmalloc(sizeof(XSegment) * nsegments);
+
+    if(direction == DIRECTION_RIGHT) {
+      x = x1; ix = 1;
+    }
+    else {
+      x = x2; ix = -1;
+    }
+    dy = (dy + 1) / 2;
+    dd = 0;
+    for(s = 0; s < nsegments; s++) {
+      segments[s].x1 = x; segments[s].y1 = y1;
+      segments[s].x2 = x; segments[s].y2 = y2;
+      x += ix;
+      if(dx >= dy) {
+	if((dd += dy) >= dx) {
+	  y1++; y2--;
+	  dd -= dx;
+	}
+      }
+      else {
+	do {
+	  y1++; y2--;
+	} while((dd += dx) < dy);
+	dd -= dy;
+      }
+    }
+  }
+  else {
+    XITK_WARNING("direction '%d' is unhandled.\n", direction);
+    return;
+  }
+
   XLOCK(im->x.disp);
   XSetForeground(im->x.disp, p->image->gc, xitk_get_pixel_color_black(im));
   XUNLOCK(im->x.disp);
 
   for(i = 0; i < 3; i++) {
     
-    if(direction == DIRECTION_UP) {
-      x1 = (w / 2) + offset;
-      y1 = (h / 4);
-      
-      x2 = ((w / 4) * 3) + offset;
-      y2 = ((h / 4) * 3);
-      
-      x3 = (w / 4) + offset;
-      y3 = ((h / 4) * 3); 
-    }
-    else if(direction == DIRECTION_DOWN) {
-      x1 = (w / 2) + offset;
-      y1 = ((h / 4) * 3);
-
-      x2 = (w / 4) + offset;
-      y2 = (h / 4);
-      
-      x3 = ((w / 4) * 3) + offset;
-      y3 = (h / 4); 
-    }
-    else if(direction == DIRECTION_LEFT) {
-      x1 = ((w / 4) * 3) + offset;
-      y1 = (h / 4);
-
-      x2 = ((w / 4) * 3) + offset;
-      y2 = ((h / 4) * 3);
-      
-      x3 = (w / 4) + offset;
-      y3 = (h / 2); 
-    }
-    else if(direction == DIRECTION_RIGHT) {
-      x1 = (w / 4) + offset;
-      y1 = (h / 4);
-
-      x2 = ((w / 4) * 3) + offset;
-      y2 = (h / 2);
-      
-      x3 = (w / 4) + offset;
-      y3 = ((h / 4) * 3); 
-    }
-    else {
-      XITK_WARNING("direction '%d' is unhandled.\n", direction);
-      return;
-    }
-    
     if(i == 2) {
-      x1++; x2++; x3++;
-      y1++; y2++; y3++;
+      for(s = 0; s < nsegments; s++) {
+	segments[s].x1++; segments[s].y1++;
+	segments[s].x2++; segments[s].y2++;
+      }
     }
     
-    points[0].x = x1;
-    points[0].y = y1;
-    points[1].x = x2;
-    points[1].y = y2;
-    points[2].x = x3;
-    points[2].y = y3;
-    points[3].x = x1;
-    points[3].y = y1;
-    
-    offset += w;
-
     XLOCK(im->x.disp);
-    XFillPolygon(im->x.disp, p->image->pixmap, p->image->gc, 
-		 &points[0], 4, Complex, CoordModeOrigin);
+    XDrawSegments(im->x.disp, p->image->pixmap, p->image->gc, 
+		  &segments[0], nsegments);
     XUNLOCK(im->x.disp);
+
+    for(s = 0; s < nsegments; s++) {
+      segments[s].x1 += w;
+      segments[s].x2 += w;
+    }
   }
 
+  free(segments);
 }
 
 /*
@@ -728,7 +811,7 @@ static void _draw_rectangular_box(ImlibData *im, xitk_pixmap_t *p,
     XSetForeground(im->x.disp, p->gc, xitk_get_pixel_color_black(im));
   
   XDrawLine(im->x.disp, p->pixmap, p->gc, x, y, (x + excstart), y);
-  XDrawLine(im->x.disp, p->pixmap, p->gc, (x + excstop), y, (x + width), y);
+  XDrawLine(im->x.disp, p->pixmap, p->gc, (x + excstop), y, (x + width - 1), y);
   XDrawLine(im->x.disp, p->pixmap, p->gc, x, y, x, (y + height));
   XUNLOCK(im->x.disp);
 
@@ -739,7 +822,7 @@ static void _draw_rectangular_box(ImlibData *im, xitk_pixmap_t *p,
     XSetForeground(im->x.disp, p->gc, xitk_get_pixel_color_white(im));
   
   XDrawLine(im->x.disp, p->pixmap, p->gc, (x + width), y, (x + width), (y + height));
-  XDrawLine(im->x.disp, p->pixmap, p->gc, x, (y + height), (x + width), (y + height));
+  XDrawLine(im->x.disp, p->pixmap, p->gc, (x + 1), (y + height), (x + width), (y + height));
   XUNLOCK(im->x.disp);
 }
 
@@ -759,7 +842,7 @@ static void _draw_rectangular_box_light(ImlibData *im, xitk_pixmap_t *p,
     XSetForeground(im->x.disp, p->gc, xitk_get_pixel_color_darkgray(im));
 
   XDrawLine(im->x.disp, p->pixmap, p->gc, x, y, (x + excstart), y);
-  XDrawLine(im->x.disp, p->pixmap, p->gc, (x + excstop), y, (x + width), y);
+  XDrawLine(im->x.disp, p->pixmap, p->gc, (x + excstop), y, (x + width - 1), y);
   XDrawLine(im->x.disp, p->pixmap, p->gc, x, y, x, (y + height));
   XUNLOCK(im->x.disp);
 
@@ -770,7 +853,7 @@ static void _draw_rectangular_box_light(ImlibData *im, xitk_pixmap_t *p,
     XSetForeground(im->x.disp, p->gc, xitk_get_pixel_color_white(im));
   
   XDrawLine(im->x.disp, p->pixmap, p->gc, (x + width), y, (x + width), (y + height));
-  XDrawLine(im->x.disp, p->pixmap, p->gc, x, (y + height), (x + width), (y + height));
+  XDrawLine(im->x.disp, p->pixmap, p->gc, (x + 1), (y + height), (x + width), (y + height));
   XUNLOCK(im->x.disp);
 }
 
@@ -790,7 +873,7 @@ static void _draw_rectangular_box_with_colors(ImlibData *im, xitk_pixmap_t *p,
   else if(relief == DRAW_INNER)
     XSetForeground(im->x.disp, p->gc, dcolor);
 
-  XDrawLine(im->x.disp, p->pixmap, p->gc, x, y, (x + width), y);
+  XDrawLine(im->x.disp, p->pixmap, p->gc, x, y, (x + width - 1), y);
   XDrawLine(im->x.disp, p->pixmap, p->gc, x, y, x, (y + height));
   XUNLOCK(im->x.disp);
 
@@ -801,7 +884,7 @@ static void _draw_rectangular_box_with_colors(ImlibData *im, xitk_pixmap_t *p,
     XSetForeground(im->x.disp, p->gc, lcolor);
   
   XDrawLine(im->x.disp, p->pixmap, p->gc, (x + width), y, (x + width), (y + height));
-  XDrawLine(im->x.disp, p->pixmap, p->gc, x, (y + height), (x + width), (y + height));
+  XDrawLine(im->x.disp, p->pixmap, p->gc, (x + 1), (y + height), (x + width), (y + height));
   XUNLOCK(im->x.disp);
 }
 
@@ -879,7 +962,7 @@ static void _draw_check_check(ImlibData *im, xitk_image_t *p, int x, int y, int
   
 }
 
-void draw_check_three_state_round_style(ImlibData *im, xitk_image_t *p, int x, int y, int d, int w, int checked) {
+static void draw_check_three_state_round_style(ImlibData *im, xitk_image_t *p, int x, int y, int d, int w, int checked) {
   int i;
 
   ABORT_IF_NULL(im);
@@ -896,7 +979,7 @@ void draw_check_three_state_round_style(ImlibData *im, xitk_image_t *p, int x, i
   }
 }
 
-void draw_check_three_state_check_style(ImlibData *im, xitk_image_t *p, int x, int y, int d, int w, int checked) {
+static void draw_check_three_state_check_style(ImlibData *im, xitk_image_t *p, int x, int y, int d, int w, int checked) {
   int i;
 
   ABORT_IF_NULL(im);
@@ -1064,18 +1147,12 @@ static void _draw_relief(ImlibData *im, xitk_pixmap_t *p, int w, int h, int reli
   
   if(relief != DRAW_FLATTER) {
     
-    if(relief == DRAW_OUTTER)
-      _draw_rectangular_box_with_colors(im, p, 0, 0, w-1, h-1, 
-					xitk_get_pixel_color_white(im),	
-					((light) 
-					 ? xitk_get_pixel_color_darkgray(im)
-					 : xitk_get_pixel_color_black(im)),
-					relief);
-    else if(relief == DRAW_INNER)
-      _draw_rectangular_box_with_colors(im, p, 0, 0, w-1, h-1, 
-					xitk_get_pixel_color_white(im),
-					xitk_get_pixel_color_black(im),	
-					relief);
+    _draw_rectangular_box_with_colors(im, p, 0, 0, w-1, h-1,
+				      xitk_get_pixel_color_white(im),
+				      ((light)
+				       ? xitk_get_pixel_color_darkgray(im)
+				       : xitk_get_pixel_color_black(im)),
+				      relief);
   }
 
 }
@@ -1268,43 +1345,60 @@ void draw_flat_with_color(ImlibData *im, xitk_pixmap_t *p, int w, int h, unsigne
 }
 
 /*
- *
+ * Draw a frame outline with embedded title.
  */
 static void _draw_frame(ImlibData *im, xitk_pixmap_t *p, 
 			char *title, char *fontname, int style, int x, int y, int w, int h) {
   xitk_font_t   *fs = NULL;
   int            sty[2];
-  int            yoff = 0, xstart = 0, xstop = 0, fheight = 0, fwidth = 0;
+  int            yoff = 0, xstart = 0, xstop = 0, fheight = 0;
+  int            ascent = 0, descent = 0, lbearing = 0, rbearing = 0;
+  int            titlelen = 0;
+  char          *titlebuf = NULL;
   char           buf[BUFSIZ];
 
   ABORT_IF_NULL(im);
   ABORT_IF_NULL(p);
   
   if(title) {
+    int maxinkwidth = (w - 12);
+
+    titlelen = strlen(title);
+    titlebuf = title;
 
     fs = xitk_font_load_font(im->x.disp, (fontname ? fontname : DEFAULT_FONT_12));
     xitk_font_set_font(fs, p->gc);
-    fheight = xitk_font_get_string_height(fs, title);
-    fwidth = xitk_font_get_string_length(fs, title);
+    xitk_font_text_extent(fs, title, titlelen, &lbearing, &rbearing, NULL, &ascent, &descent);
+    fheight = ascent + descent;
 
-    if(fwidth >= (w - 12)) {
-      int nchar = 1;
-      
-      memset(&buf, 0, sizeof(buf));
-      /* Limit the title to width of frame */
+    /* Limit title to frame width */
+    if((rbearing - lbearing) > maxinkwidth) {
+      char  dots[]  = "...";
+      int   dotslen = strlen(dots);
+      int   dotsrbearing;
+      int   titlewidth;
+
+      /* Cut title, append dots */
+      xitk_font_text_extent(fs, dots, dotslen, NULL, &dotsrbearing, NULL, NULL, NULL);
       do {
-	nchar++;
-	snprintf(buf, nchar, "%s", title);
-      } while(xitk_font_get_string_length(fs, buf) < (w - 12));
-      /* Cut title, add three dots a the end */
-      nchar -= 4;
-      snprintf(buf, nchar, "%s", title);
-      sprintf(buf, "%s%s", buf, "...");
+	titlelen--;
+	xitk_font_text_extent(fs, title, titlelen, NULL, NULL, &titlewidth, NULL, NULL);
+	rbearing = titlewidth + dotsrbearing;
+      } while((rbearing - lbearing) > maxinkwidth);
+      { /* Cut possible incomplete multibyte character at the end */
+	int titlewidth1;
+	while((titlelen > 0) &&
+	      (xitk_font_text_extent(fs, title, (titlelen - 1), NULL, NULL, &titlewidth1, NULL, NULL),
+	       titlewidth1 == titlewidth))
+	  titlelen--;
+      }
+      if(titlelen > (sizeof(buf) - dotslen - 1)) /* Should never happen, */
+	titlelen = (sizeof(buf) - dotslen - 1);  /* just to be sure ...  */
+      strncpy(buf, title, titlelen), buf[titlelen] = '\0';
+      strcat(buf, dots);
+      titlelen += dotslen;
+      titlebuf = buf;
     }
-    else
-      sprintf(buf, "%s", title);
-
-    fwidth = xitk_font_get_string_length(fs, buf);
   }
 
   sty[0] = (style == DRAW_INNER) ? DRAW_INNER : DRAW_OUTTER;
@@ -1312,25 +1406,26 @@ static void _draw_frame(ImlibData *im, xitk_pixmap_t *p,
 
   /* Dont draw frame box under frame title */
   if(title) {
-    yoff = (fheight>>1);
-    xstart = 3;
-    xstop = fwidth + 12;
+    yoff = (ascent >> 1) + 1; /* Roughly v-center outline to ascent part of glyphs */
+    xstart = 4 - 1;
+    xstop = (rbearing - lbearing) + 8;
   }
-  _draw_rectangular_box_light(im, p, x, (y - yoff), 
+
+  _draw_rectangular_box_light(im, p, x, (y + yoff), 
 			      xstart, xstop,
 			      w, (h - yoff), sty[0]);
   
   if(title)
-    xstart--;
+    xstart--, xstop--;
   
-  _draw_rectangular_box_light(im, p, (x + 1), ((y - yoff) + 1), 
+  _draw_rectangular_box_light(im, p, (x + 1), ((y + yoff) + 1), 
 			      xstart, xstop,
 			      (w - 2), ((h - yoff) - 2), sty[1]);
   
   if(title) {
     XLOCK(im->x.disp);
     XSetForeground(im->x.disp, p->gc, xitk_get_pixel_color_black(im));
-    xitk_font_draw_string(fs, p->pixmap, p->gc, (x + 6), y, buf, strlen(buf));
+    xitk_font_draw_string(fs, p->pixmap, p->gc, (x - lbearing + 6), (y + ascent), titlebuf, titlelen);
     XUNLOCK(im->x.disp);
     
     xitk_font_unload_font(fs);
@@ -1561,11 +1656,16 @@ xitk_image_t *xitk_image_load_image(ImlibData *im, char *image) {
   i->image->pixmap = Imlib_copy_image(im, img);
   XUNLOCK(im->x.disp);
 
-  i->mask          = xitk_image_create_xitk_mask_pixmap(im, img->rgb_width, img->rgb_height);
-  XLOCK(im->x.disp);
-  i->mask->pixmap  = Imlib_copy_mask(im, img);
-  XUNLOCK(im->x.disp);
-
+  if(img->shape_mask) {
+    i->mask          = xitk_image_create_xitk_mask_pixmap(im, img->rgb_width, img->rgb_height);
+    XLOCK(im->x.disp);
+    i->mask->pixmap  = Imlib_copy_mask(im, img);
+    XUNLOCK(im->x.disp);
+  }
+  else {
+    i->mask = NULL;
+  }
+    
   i->width         = img->rgb_width;
   i->height        = img->rgb_height;
   
@@ -1590,6 +1690,10 @@ static void notify_destroy(xitk_widget_t *w) {
   
   if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_IMAGE)) {
     private_data = (image_private_data_t *) w->private_data;
+
+    if(!private_data->skin_element_name)
+      xitk_image_free_image(private_data->imlibdata, &(private_data->skin));
+
     XITK_FREE(private_data->skin_element_name);
     XITK_FREE(private_data);
   }
