@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2007 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: inputtext.c,v 1.63 2004/04/13 19:55:49 f1rmb Exp $
+ * $Id: inputtext.c,v 1.71 2007/02/06 22:41:18 fsck-p Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -31,7 +31,6 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
-#include <X11/cursorfont.h>
 
 #include "_xitk.h"
 
@@ -48,15 +47,10 @@ static void _cursor_focus(inputtext_private_data_t *private_data, Window win, in
 
   private_data->cursor_focus = focus;
   
-  XLOCK(private_data->imlibdata->x.disp);
-
   if(focus)
-    XDefineCursor(private_data->imlibdata->x.disp, win, private_data->cursor[1]);
+    xitk_cursors_define_window_cursor(private_data->imlibdata->x.disp, win, xitk_cursor_xterm);
   else
-    XDefineCursor(private_data->imlibdata->x.disp, win, private_data->cursor[0]);
-  
-  XSync(private_data->imlibdata->x.disp, False);
-  XUNLOCK(private_data->imlibdata->x.disp);
+    xitk_cursors_restore_window_cursor(private_data->imlibdata->x.disp, win);
 }
 
 /*
@@ -68,11 +62,9 @@ static void notify_destroy(xitk_widget_t *w) {
   if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
     private_data = (inputtext_private_data_t *) w->private_data;
   
-    XLOCK(private_data->imlibdata->x.disp);
-    XFreeCursor(private_data->imlibdata->x.disp, private_data->cursor[0]);
-    XFreeCursor(private_data->imlibdata->x.disp, private_data->cursor[1]);
-    XUNLOCK(private_data->imlibdata->x.disp);
-
+    if(!private_data->skin_element_name)
+      xitk_image_free_image(private_data->imlibdata, &(private_data->skin));
+    
     XITK_FREE(private_data->skin_element_name);
     XITK_FREE(private_data->text);
     XITK_FREE(private_data->normal_color);
@@ -160,13 +152,13 @@ int xitk_get_key_modifier(XEvent *xev, int *modifier) {
   return (*modifier != MODIFIER_NOMOD);
 }
 
-int xitk_get_keysym_and_buf(XEvent *event, KeySym *ksym, char kbuf[]) {
+int xitk_get_keysym_and_buf(XEvent *event, KeySym *ksym, char kbuf[], int kblen) {
   int len = 0;
   if(event) {
     XKeyEvent  pkeyev = event->xkey;
     
     XLOCK(pkeyev.display);
-    len = XLookupString(&pkeyev, kbuf, sizeof(kbuf), ksym, NULL);
+    len = XLookupString(&pkeyev, kbuf, kblen, ksym, NULL);
     XUNLOCK(pkeyev.display);
   }
   return len;
@@ -180,7 +172,7 @@ KeySym xitk_get_key_pressed(XEvent *event) {
   
   if(event) {
     char  buf[256];
-    (void) xitk_get_keysym_and_buf(event, &pkey, buf);
+    (void) xitk_get_keysym_and_buf(event, &pkey, buf, sizeof(buf));
   }
 
   return pkey;
@@ -277,7 +269,7 @@ static void create_labelofinputtext(xitk_widget_t *w,
   }
   
   if((private_data->skin_element_name != NULL) ||
-     ((private_data->skin_element_name == NULL) && ((fg = xitk_get_black_color()) == -1))) {
+     ((private_data->skin_element_name == NULL) && ((fg = xitk_get_black_color()) == (unsigned int)-1))) {
     
     /*  Some colors configurations */
     switch(state) {
@@ -400,7 +392,12 @@ static void create_labelofinputtext(xitk_widget_t *w,
     
     width = (label && strlen(label)) ? 
       xitk_font_get_text_width(fs, plabel, (private_data->cursor_pos - private_data->disp_offset)) : 0;
-  
+
+#ifdef WITH_XFT
+    if(width)
+      width += 2;
+#endif
+    
     XLOCK(private_data->imlibdata->x.disp);
     XDrawLine(private_data->imlibdata->x.disp, pix, gc,
 	      width + 1, 2, width + 3, 2);
@@ -691,7 +688,7 @@ static void inputtext_erase_with_backspace(xitk_widget_t *w) {
   int                       offset;
   
   if(private_data->text && (strlen(private_data->text) >= 1)) {
-    if(private_data->cursor_pos >= 0) {
+    if(private_data->cursor_pos > 0) {
       
       oldtext = strdup(private_data->text);
       newtext = (char *) xitk_xmalloc(strlen(oldtext));
@@ -703,20 +700,15 @@ static void inputtext_erase_with_backspace(xitk_widget_t *w) {
       pp = newtext;
       
       while(offset < private_data->cursor_pos) {
-	*pp = *p;
-	p++;
-	pp++;
+	*pp++ = *p++;
 	offset++;
       }
       p++;
       
       while(*p != '\0') {
-	*pp = *p;
-	offset++;
-	p++;
-	pp++;
+	*pp++ = *p++;
       } 
-      *pp = 0;
+      *pp = '\0';
       
       XITK_FREE(private_data->text);
       private_data->text = strdup(newtext);
@@ -871,7 +863,7 @@ static void notify_keyevent_inputtext(xitk_widget_t *w, XEvent *xev) {
 
     private_data->have_focus = FOCUS_RECEIVED;
 
-    len = xitk_get_keysym_and_buf(xev, &key, buf);
+    len = xitk_get_keysym_and_buf(xev, &key, buf, sizeof(buf));
     buf[len] = '\0';
 
     (void) xitk_get_key_modifier(xev, &modifier);
@@ -1017,7 +1009,7 @@ static void notify_keyevent_inputtext(xitk_widget_t *w, XEvent *xev) {
 	    
 	    private_data->text = (char *) realloc(private_data->text, strlen(oldtext) + 3);
 
-	    sprintf(private_data->text, "%s", oldtext);
+	    strcpy(private_data->text, oldtext);
 	    sprintf(&private_data->text[pos], "%c%s", buf[0], &oldtext[pos]);
 	    private_data->text[strlen(private_data->text)] = '\0';
 	    free(oldtext);
@@ -1030,7 +1022,8 @@ static void notify_keyevent_inputtext(xitk_widget_t *w, XEvent *xev) {
 	else {
 	  private_data->text = (char *) xitk_xmalloc(2);
 	  
-	  sprintf(private_data->text, "%c%c", buf[0], 0);
+	  private_data->text[0] = buf[0];
+	  private_data->text[1] = 0;
 	  
 	  private_data->cursor_pos++;
 	  paint_inputtext(w);
@@ -1158,10 +1151,6 @@ static xitk_widget_t *_xitk_inputtext_create (xitk_widget_list_t *wl,
 
   private_data->disp_offset       = 0;
 
-  XLOCK(private_data->imlibdata->x.disp);
-  private_data->cursor[0] = XCreateFontCursor(private_data->imlibdata->x.disp, XC_left_ptr);
-  private_data->cursor[1] = XCreateFontCursor(private_data->imlibdata->x.disp, XC_xterm);
-  XUNLOCK(private_data->imlibdata->x.disp);
   private_data->cursor_focus      = 0;
 
   private_data->callback          = it->callback;
@@ -1226,6 +1215,5 @@ xitk_widget_t *xitk_noskin_inputtext_create (xitk_widget_list_t *wl,
   i = xitk_image_create_image(it->imlibdata, width * 2, height);
   draw_bevel_two_state(it->imlibdata, i);
   
-
   return _xitk_inputtext_create(wl, NULL, it, x, y, NULL, i, fontname, ncolor, fcolor, 0, 0);
 }
