@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: label.h,v 1.23 2004/04/13 19:55:50 f1rmb Exp $
+ * $Id: label.h,v 1.25 2004/05/27 22:25:50 f1rmb Exp $
  *
  */
 
@@ -35,7 +35,6 @@ typedef struct {
   int                     char_length; /* length of 1 char */
   int                     char_height; /* height of 1 char */
 
-  //  Pixmap                  labelpix;
   xitk_pixmap_t          *labelpix;
 
   int                     length;      /* length in char */
@@ -52,10 +51,11 @@ typedef struct {
   int                     anim_running;
   int                     anim_offset;
 
-  int                     on_change;
+  int                     label_visible;
 
   pthread_t               thread;
   pthread_mutex_t         paint_mutex;
+  pthread_mutex_t         change_mutex;
 
 } label_private_data_t;
 
