@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2007 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: labelbutton.c,v 1.60 2004/06/13 02:36:35 f1rmb Exp $
+ * $Id: labelbutton.c,v 1.62 2007/01/12 00:17:52 fsck-p Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -134,7 +134,7 @@ static void create_labelofbutton(xitk_widget_t *lb,
   }
 
   if((private_data->skin_element_name != NULL) ||
-     ((private_data->skin_element_name == NULL) && ((fg = xitk_get_black_color()) == -1))) {
+     ((private_data->skin_element_name == NULL) && ((fg = xitk_get_black_color()) == (unsigned int)-1))) {
     
     /*  Some colors configurations */
     switch(state) {
@@ -461,7 +461,7 @@ char *xitk_labelbutton_get_shortcut_label(xitk_widget_t *w) {
 }
 
 /*
- * Handle motion on button
+ * Handle focus on button
  */
 static int notify_focus_labelbutton (xitk_widget_t *w, int focus) {
   lbutton_private_data_t *private_data;
@@ -536,10 +536,6 @@ static int notify_event(xitk_widget_t *w, widget_event_t *event, widget_event_re
     break;
   case WIDGET_EVENT_FOCUS:
     notify_focus_labelbutton(w, event->focus);
-    if((w->type & WIDGET_GROUP_MENU) && 
-       (event->type == WIDGET_EVENT_FOCUS) && (event->focus == FOCUS_MOUSE_IN)) {
-      menu_auto_pop(w);
-    }
     break;
   case WIDGET_EVENT_INSIDE:
     result->value = notify_inside(w, event->x, event->y);
