@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: list.c,v 1.20 2004/04/16 12:29:40 f1rmb Exp $
+ * $Id: list.c,v 1.21 2004/10/17 22:05:08 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -226,3 +226,10 @@ void xitk_list_delete_current (xitk_list_t *l) {
   //  XITK_FREE(node_cur->content);
   free(node_cur);
 }
+
+xitk_node_t *xitk_list_get_current(xitk_list_t *l) {
+  if(l && l->cur)
+    return l->cur;
+  
+  return NULL;
+}
