@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2006 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: mrlbrowser.c,v 1.85 2005/04/28 00:57:58 fsck-p Exp $
+ * $Id: mrlbrowser.c,v 1.89 2006/10/18 23:04:17 fsck-p Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -63,6 +63,8 @@ static xitk_mrlbrowser_filter_t __mrl_filters[] = {
   { NULL                  , NULL           }
 };
 
+extern void gui_handle_event(XEvent *event, void *data);
+
 /*
  *
  */
@@ -336,7 +338,7 @@ static void mrlbrowser_grab_mrls(xitk_widget_t *w, void *data) {
     private_data->last_mrl_source = (char *) realloc(private_data->last_mrl_source, strlen(lbl) + 1);
     
     old_old_src = strdup(private_data->last_mrl_source);
-    sprintf(private_data->last_mrl_source, "%s", lbl);
+    strcpy(private_data->last_mrl_source, lbl);
 
     {
       int num_mrls;
@@ -345,7 +347,7 @@ static void mrlbrowser_grab_mrls(xitk_widget_t *w, void *data) {
 					       NULL, &num_mrls);
       if(!mtmp) {
 	private_data->last_mrl_source = (char *) realloc(private_data->last_mrl_source, strlen(old_old_src) + 1);
-	sprintf(private_data->last_mrl_source, "%s", old_old_src);
+	strcpy(private_data->last_mrl_source, old_old_src);
 	return;
       }
       
@@ -1089,12 +1091,13 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
    */
   XLOCK(mb->imlibdata->x.disp);
   if(mb->layer_above) {
-    XA_WIN_LAYER = XInternAtom(mb->imlibdata->x.disp, "_WIN_LAYER", False);
+    if((XA_WIN_LAYER = XInternAtom(mb->imlibdata->x.disp, "_WIN_LAYER", False)) != None) {
     
-    data[0] = 10;
-    XChangeProperty(mb->imlibdata->x.disp, private_data->window, XA_WIN_LAYER,
-		    XA_CARDINAL, 32, PropModeReplace, (unsigned char *)data,
-		    1);
+      data[0] = 10;
+      XChangeProperty(mb->imlibdata->x.disp, private_data->window, XA_WIN_LAYER,
+		      XA_CARDINAL, 32, PropModeReplace, (unsigned char *)data,
+		      1);
+    }
   }
   
   /*
