@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: mrlbrowser.h,v 1.18 2003/07/14 22:44:10 f1rmb Exp $
+ * $Id: mrlbrowser.h,v 1.21 2006/05/22 17:53:26 mshopf Exp $
  *
  */
 
@@ -53,13 +53,13 @@
  * Duplicate two mrls entries (s = source, d = destination).
  */
 #define MRL_DUPLICATE(s, d) {                                                 \
-  assert((s) != NULL);                                                        \
-  assert((d) != NULL);                                                        \
+  ABORT_IF_NULL(s);                                                           \
+  ABORT_IF_NULL(d);                                                           \
                                                                               \
   if((s)->origin) {                                                           \
     if((d)->origin) {                                                         \
       (d)->origin = (char *) realloc((d)->origin, strlen((s)->origin) + 1);   \
-      sprintf((d)->origin, "%s", (s)->origin);                                \
+      strcpy((d)->origin, (s)->origin);                                	      \
     }                                                                         \
     else                                                                      \
       (d)->origin = strdup((s)->origin);                                      \
@@ -70,7 +70,7 @@
   if((s)->mrl) {                                                              \
     if((d)->mrl) {                                                            \
       (d)->mrl = (char *) realloc((d)->mrl, strlen((s)->mrl) + 1);            \
-      sprintf((d)->mrl, "%s", (s)->mrl);                                      \
+      strcpy((d)->mrl, (s)->mrl);                                      	      \
     }                                                                         \
     else                                                                      \
       (d)->mrl = strdup((s)->mrl);                                            \
@@ -81,7 +81,7 @@
   if((s)->link) {                                                             \
     if((d)->link) {                                                           \
       (d)->link = (char *) realloc((d)->link, strlen((s)->link) + 1);         \
-      sprintf((d)->link, "%s", (s)->link);                                    \
+      strcpy((d)->link, (s)->link);                                    	      \
     }                                                                         \
     else                                                                      \
       (d)->link = strdup((s)->link);                                          \
@@ -99,8 +99,8 @@
 #define MRLS_DUPLICATE(s, d) {                                                \
   int i = 0;                                                                  \
                                                                               \
-  assert((s) != NULL);                                                        \
-  assert((d) != NULL);                                                        \
+  ABORT_IF_NULL(s);                                                           \
+  ABORT_IF_NULL(d);                                                           \
                                                                               \
   while((s) != NULL) {                                                        \
     d[i] = (xine_mrl_t *) malloc(sizeof(xine_mrl_t));                         \
