@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: skin.c,v 1.29 2004/04/13 19:55:50 f1rmb Exp $
+ * $Id: skin.c,v 1.40 2006/07/15 20:26:20 dgp85 Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -31,6 +31,7 @@
 #include <pthread.h>
 #include <errno.h>
 
+#include "utils.h"
 #include "_xitk.h"
 
 #undef DEBUG_SKIN
@@ -44,21 +45,33 @@ static pthread_mutex_t  skin_mutex = PTHREAD_MUTEX_INITIALIZER;
 /*
  *
  */
-static void skin_load_to_cache(xitk_skin_config_t *skonfig, char *pixmap) {
+static xitk_image_t *skin_load_to_cache(xitk_skin_config_t *skonfig, char *pixmap) {
 
   if(skonfig && pixmap) {
     xitk_image_t  *image  = xitk_image_load_image(skonfig->im, pixmap);
     cache_entry_t *cache  = skonfig->cache;
     cache_entry_t *ncache = NULL;
-    
+
     if(image) {
+
+      if(cache) {
+	while(cache) {
+	  if(!strcmp(cache->filename, pixmap)) {
+	    xitk_image_free_image(skonfig->im, &image);
+	    return cache->image;
+	  }
+	  cache = cache->next;
+	}
+	cache = skonfig->cache;
+      }
+
       ncache           = (cache_entry_t *) xitk_xmalloc(sizeof(cache_entry_t));
       ncache->image    = image;
       ncache->filename = strdup(pixmap);
       ncache->next     = NULL;
     }
     else
-      return;
+      return image;
     
     if(!cache) {
       skonfig->cache = ncache;
@@ -69,7 +82,11 @@ static void skin_load_to_cache(xitk_skin_config_t *skonfig, char *pixmap) {
       
       cache->next  = ncache;
     }
+
+    return image;
   }
+
+  return NULL;
 }
 
 static void skin_free_cache(xitk_skin_config_t *skonfig) {
@@ -91,7 +108,8 @@ static void skin_free_cache(xitk_skin_config_t *skonfig) {
 /*
  *
  */
-static char *_get_expanded_command(xitk_skin_config_t *skonfig, char *cmd) {
+#warning FIXME
+static char *_expanded(xitk_skin_config_t *skonfig, char *cmd) {
   char *p;
   char *ret = NULL;
   char  buf[BUFSIZ], buf2[BUFSIZ], var[BUFSIZ];
@@ -102,10 +120,9 @@ static char *_get_expanded_command(xitk_skin_config_t *skonfig, char *cmd) {
 
     if(strchr(cmd, '$')) {
 
-      memset(&buf, 0, sizeof(buf));
-      snprintf(buf, (BUFSIZ - 1), "%s", cmd);
+      snprintf(buf, sizeof(buf), "%s", cmd);
 
-      memset(&buf2, 0, sizeof(buf2));
+      buf2[0] = 0;
       
       p = buf;
 
@@ -129,47 +146,48 @@ static char *_get_expanded_command(xitk_skin_config_t *skonfig, char *cmd) {
 		char ppath[XITK_PATH_MAX + XITK_NAME_MAX + 1];
 		char *z;
 		
-		sprintf(ppath, "%s", skonfig->path);
+		snprintf(ppath, sizeof(ppath), "%s", skonfig->path);
 		if((z = strrchr(ppath, '/')) != NULL) {
 		  *z = '\0';
-		  sprintf(buf2, "%s%s", buf2, ppath);
+		  strlcat(buf2, ppath, sizeof(buf2));
 		}
 	      }
 	    }
 	    else if(!strncmp("SKIN_VERSION", var, strlen(var))) {
 	      if(skonfig->version >= 0)
-		sprintf(buf2, "%s%d", buf2, skonfig->version);
+		snprintf(buf2+strlen(buf2), sizeof(buf2)-strlen(buf2), "%d", skonfig->version);
 	    }
 	    else if(!strncmp("SKIN_AUTHOR", var, strlen(var))) {
 	      if(skonfig->author)
-		sprintf(buf2, "%s%s", buf2, skonfig->author);
+		strlcat(buf2, skonfig->author, sizeof(buf2));
 	    }
 	    else if(!strncmp("SKIN_PATH", var, strlen(var))) {
 	      if(skonfig->path)
-		sprintf(buf2, "%s%s", buf2, skonfig->path);
+		strlcat(buf2, skonfig->path, sizeof(buf2));
 	    }
 	    else if(!strncmp("SKIN_NAME", var, strlen(var))) {
 	      if(skonfig->name)
-		sprintf(buf2, "%s%s", buf2, skonfig->name);
+		strlcat(buf2, skonfig->name, sizeof(buf2));
 	    }
 	    else if(!strncmp("SKIN_DATE", var, strlen(var))) {
 	      if(skonfig->date)
-		sprintf(buf2, "%s%s", buf2, skonfig->date);
+		strlcat(buf2, skonfig->date, sizeof(buf2));
 	    }
 	    else if(!strncmp("SKIN_URL", var, strlen(var))) {
 	      if(skonfig->url)
-		sprintf(buf2, "%s%s", buf2, skonfig->url);
+		strlcat(buf2, skonfig->url, sizeof(buf2));
 	    }
 	    else if(!strncmp("HOME", var, strlen(var))) {
 	      if(skonfig->url)
-		sprintf(buf2, "%s%s", buf2, xitk_get_homedir());
+		strlcat(buf2, xitk_get_homedir(), sizeof(buf2));
 	    }
 	    /* else ignore */
 	  }
 	  break;
 	  
 	default:
-	  sprintf(buf2, "%s%c", buf2, *p);
+	  buf2[strlen(buf2) + 1] = 0;
+	  buf2[strlen(buf2)] = *p;
 	  break;
 	}
 	p++;
@@ -291,20 +309,16 @@ static void skin_get_next_line(xitk_skin_config_t *skonfig) {
   ABORT_IF_NULL(skonfig);
   ABORT_IF_NULL(skonfig->fd);
   
- __get_next_line:
+  do {
+    skonfig->ln = fgets(skonfig->buf, 255, skonfig->fd);
+    
+    while(skonfig->ln && (*skonfig->ln == ' ' || *skonfig->ln == '\t')) ++skonfig->ln;
 
-  skonfig->ln = fgets(skonfig->buf, 255, skonfig->fd);
-
-  while(skonfig->ln && (*skonfig->ln == ' ' || *skonfig->ln == '\t')) ++skonfig->ln;
-
-  if(skonfig->ln) {
-    if((strncmp(skonfig->ln, "//", 2) == 0) ||
-       (strncmp(skonfig->ln, "/*", 2) == 0) || /**/
-       (strncmp(skonfig->ln, ";", 1) == 0) ||
-       (strncmp(skonfig->ln, "#", 1) == 0)) {
-      goto __get_next_line;
-    }
-  }
+  } while(skonfig->ln && 
+	  (!strncmp(skonfig->ln, "//", 2) ||
+	   !strncmp(skonfig->ln, "/*", 2) || /* */
+	   !strncmp(skonfig->ln, ";", 1) ||
+	   !strncmp(skonfig->ln, "#", 1)));
 
   skin_clean_eol(skonfig);
 }
@@ -607,11 +621,11 @@ static void skin_parse_section(xitk_skin_config_t *skonfig) {
 	  skin_set_pos_to_value(&p);
 
 	  if(!strncasecmp(section, "unload_command", 14)) {
-	    skonfig->unload_command = _get_expanded_command(skonfig, p);
+	    skonfig->unload_command = _expanded(skonfig, p);
 	    return;
 	  }
 	  else if(!strncasecmp(section, "load_command", 12)) {
-	    skonfig->load_command = _get_expanded_command(skonfig, p);
+	    skonfig->load_command = _expanded(skonfig, p);
 	    return;
 	  }
 	  else if(!strncasecmp(section, "animation", 9)) {
@@ -635,7 +649,7 @@ static void skin_parse_section(xitk_skin_config_t *skonfig) {
 	    return;
 	  }
 	  else if(!strncasecmp(section, "logo", 4)) {
-	    skonfig->logo = strdup(p);
+	    skonfig->logo = _expanded(skonfig, p);
 	    return;
 	  }
 	  else if(!strncasecmp(section, "url", 3)) {
@@ -691,8 +705,8 @@ static void check_skonfig(xitk_skin_config_t *skonfig) {
       }
 
       if(s->browser_entries > -1)
-	printf("  browser entries = %d\n", s->browset_entries);
-
+	printf("  browser entries = %d\n", s->browser_entries);
+      
       printf("  animation   = %d\n", s->animation);
       printf("  step        = %d\n", s->animation_step);
       printf("  print       = %d\n", s->print);
@@ -848,7 +862,7 @@ int xitk_skin_load_config(xitk_skin_config_t *skonfig, char *path, char *filenam
   skonfig->path     = strdup(path);
   skonfig->skinfile = strdup(filename);
 
-  snprintf(buf, 2048, "%s/%s", skonfig->path, skonfig->skinfile);
+  snprintf(buf, sizeof(buf), "%s/%s", skonfig->path, skonfig->skinfile);
 
   if((skonfig->fd = fopen(buf, "r")) != NULL) {
     
@@ -957,6 +971,21 @@ int xitk_skin_get_visibility(xitk_skin_config_t *skonfig, const char *str) {
   return 1;
 }
 
+/*
+ *
+ */
+int xitk_skin_get_printability(xitk_skin_config_t *skonfig, const char *str) {
+  xitk_skin_element_t *s;
+  
+  ABORT_IF_NULL(skonfig);
+  
+  if((s = skin_lookup_section(skonfig, str)) != NULL)
+    return s->print;
+
+  return 1;
+}
+
+
 /*
  *
  */
@@ -1270,7 +1299,8 @@ xitk_image_t *xitk_skin_get_image(xitk_skin_config_t *skonfig, const char *str)
     }
   }
 
-  return (xitk_image_load_image(skonfig->im, (char *) str));
+  return (skin_load_to_cache(skonfig, (char *) str));
+  //  return (xitk_image_load_image(skonfig->im, (char *) str));
 }
 
 int xitk_skin_get_max_buttons(xitk_skin_config_t *skonfig, const char *str) {
