@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: skin.h,v 1.20 2003/07/14 22:44:10 f1rmb Exp $
+ * $Id: skin.h,v 1.23 2004/04/13 19:55:50 f1rmb Exp $
  *
  */
 #ifndef HAVE_XITK_SKIN_H
@@ -60,14 +60,26 @@ typedef struct xitk_skin_element_s {
   char                         *color_focus;
   char                         *color_click;
   char                         *font;
+
+  int                          max_buttons;
 } xitk_skin_element_t;
 
+typedef struct cache_entry_s cache_entry_t;
+struct cache_entry_s {
+  xitk_image_t   *image;
+  char           *filename;
+  cache_entry_t  *next;
+};
+
 struct xitk_skin_config_s {
+  ImlibData                    *im;
   FILE                         *fd;
   char                         *path;
   char                         *skinfile;
   char                         *ln;
   char                          buf[256];
+
+  cache_entry_t                *cache;
   
   char                         *name;
   int                           version;
@@ -77,7 +89,6 @@ struct xitk_skin_config_s {
   char                         *logo;
   char                         *animation;
 
-  pthread_mutex_t               mutex;
   char                         *load_command;
   char                         *unload_command;
   
