@@ -137,6 +137,7 @@ static int init_test(void) {
   test->imlibdata = Imlib_init_with_params(test->display, &imlib_init);
   if (test->imlibdata == NULL) {
     fprintf(stderr, _("Imlib_init_with_params() failed.\n"));
+    XUnlockDisplay (test->display);
     return 0;
   }
 
@@ -962,13 +963,12 @@ int main(int argc, char **argv) {
     exit(1);
   }
 
-#ifdef HAVE_SETLOCALE
+#ifdef ENABLE_NLS
   xitk_set_locale();
-  setlocale (LC_ALL, "");
-  printf("locale\n");
+  printf("locale: %s\n", setlocale (LC_ALL, ""));
 #endif
   
-  bindtextdomain("xitk", XITK_LOCALE);
+  printf("bindtextdomain: %s\n", bindtextdomain("xitk", XITK_LOCALE));
   textdomain("xitk");
 
   /* Create window */
@@ -978,6 +978,22 @@ int main(int argc, char **argv) {
   
   XLockDisplay (test->display);
 
+#undef DUMP_ATOMS
+#ifdef DUMP_ATOMS
+  {
+    int   i = 1;
+    char *atom_name;
+
+    for(;;i++) {
+      if((atom_name = XGetAtomName(test->display, (Atom)i)))
+	printf("Atom %d: '%s'\n", i, atom_name);
+      else
+	break;
+    }
+  }
+#endif
+
+
   gc = XCreateGC(test->display, 
 		 (xitk_window_get_window(test->xwin)), None, None);
 
