@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2006 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: tips.c,v 1.42 2005/05/20 23:56:44 fsck-p Exp $
+ * $Id: tips.c,v 1.46 2006/12/31 21:58:10 fsck-p Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -55,15 +55,6 @@ typedef struct {
 
 static _tips_t tips;
 
-static void _tips_handle_event(XEvent *event, void *data) {
-  switch(event->type) {
-  case ButtonRelease:
-  case ButtonPress:
-    xitk_tips_hide_tips();
-    break;
-  }
-}
-
 static void *_tips_loop_thread(void *data) {
 
   tips.running = 1;
@@ -99,21 +90,20 @@ static void *_tips_loop_thread(void *data) {
     tips.new_widget = NULL;
     
     if(tips.widget && (tips.widget->tips_timeout > 0) && tips.widget->tips_string && strlen(tips.widget->tips_string)) {
-      int                  x, y, w, h, string_length;
+      int                  x, y, w, h;
       xitk_window_t       *xwin;
       xitk_register_key_t  key;
       xitk_image_t        *image;
       xitk_font_t         *fs;
-      XWindowAttributes    wattr;
-      Status               status;
-      unsigned int         cwarnfore, cwarnback;
+      unsigned int         cfore, cback;
       struct timeval       tv;
       struct timespec      ts;
 
       int                  disp_w = xitk_get_display_width();
       int                  disp_h = xitk_get_display_height();
 
-      int                  margin = 10;
+      int                  x_margin = 12, y_margin = 6;
+      int                  bottom_gap = 16; /* To avoid mouse cursor overlaying tips on bottom of widget */
 
       tips.visible = 1;
 
@@ -126,31 +116,31 @@ static void *_tips_loop_thread(void *data) {
       fs = xitk_font_load_font(tips.display, DEFAULT_FONT_10);
       xitk_font_set_font(fs, tips.widget->wl->gc);
 
-      string_length = MIN((xitk_font_get_string_length(fs, tips.widget->tips_string)), (disp_w/3));
-
       xitk_font_unload_font(fs);
       
-      cwarnfore = xitk_get_pixel_color_warning_foreground(tips.widget->imlibdata);
-      cwarnback = xitk_get_pixel_color_warning_background(tips.widget->imlibdata);
+      cfore = xitk_get_pixel_color_black(tips.widget->imlibdata);
+      cback = xitk_get_pixel_color_lightgray(tips.widget->imlibdata);
       
+      /* Note: disp_w/3 is max. width, returned image with ALIGN_LEFT will be as small as possible */
       image = xitk_image_create_image_with_colors_from_string(tips.widget->imlibdata, DEFAULT_FONT_10,
-							      string_length + 1, ALIGN_LEFT, 
-							      tips.widget->tips_string, cwarnfore, cwarnback);
+							      (disp_w/3), ALIGN_LEFT, 
+							      tips.widget->tips_string, cfore, cback);
       
       /* Create the tips window, horizontally centered from parent widget */
       /* If necessary, adjust position to display it fully on screen      */
-      /* 1 px dist to widget prevents odd behavior of mouse pointer when  */
-      /* pointer is moved slowly from widget to tips, at least under FVWM */
-      w = image->width + margin;
-      h = image->height + margin;
-      x -= ((w >> 1) - (tips.widget->width >> 1));
-      y += (tips.widget->height + 1);
+      w = image->width + x_margin;
+      h = image->height + y_margin;
+      x -= ((w - tips.widget->width) >> 1);
+      y += (tips.widget->height + bottom_gap);
       if(x > disp_w - w)
 	x = disp_w - w;
       else if(x < 0)
 	x = 0;
       if(y > disp_h - h)
-	y -= (tips.widget->height + h + 2);
+	/* 1 px dist to widget prevents odd behavior of mouse pointer when  */
+	/* pointer is moved slowly from widget to tips, at least under FVWM */
+	/*                                           v                      */
+	y -= (tips.widget->height + h + bottom_gap + 1);
       /* No further alternative to y-position the tips (just either below or above widget) */
       xwin = xitk_window_create_simple_window(tips.widget->imlibdata, x, y, w, h);
       
@@ -180,15 +170,15 @@ static void *_tips_loop_thread(void *data) {
 	XUNLOCK(tips.display);
 	
 	XLOCK(tips.display);
-	XSetForeground(tips.display, gc, cwarnfore);
+	XSetForeground(tips.display, gc, cfore);
 	XDrawRectangle(tips.display, bg->pixmap, gc, 0, 0, width - 1, height - 1);
 	XUNLOCK(tips.display);
 	
 	XLOCK(tips.display);
-	XSetForeground(tips.display, gc, cwarnback);
+	XSetForeground(tips.display, gc, cback);
 	XFillRectangle(tips.display, bg->pixmap, gc, 1, 1, width - 2, height - 2);
 	XCopyArea(tips.display, image->image->pixmap, bg->pixmap,
-		  gc, 0, 0, image->width, image->height, (width - image->width)>>1, ((height - image->height)>>1) + 1);
+		  gc, 0, 0, image->width, image->height, (width - image->width)>>1, (height - image->height)>>1);
 	XUNLOCK(tips.display);
 	
 	xitk_window_change_background(tips.widget->imlibdata, xwin, bg->pixmap, width, height);
@@ -203,23 +193,8 @@ static void *_tips_loop_thread(void *data) {
       }
       
       XLOCK(tips.display);
-      status = XGetWindowAttributes(tips.display, tips.widget->wl->win, &wattr);
       XMapRaised(tips.display, (xitk_window_get_window(xwin)));
       XUNLOCK(tips.display);
-      
-      if((status != BadDrawable) && (status != BadWindow) && (wattr.map_state == IsViewable)) {
-	XLOCK(tips.display);
-	XSetInputFocus(tips.display, tips.widget->wl->win, RevertToParent, CurrentTime);
-	XUNLOCK(tips.display);
-      }
-      
-      key = xitk_register_event_handler("xitk tips", 
-					(xitk_window_get_window(xwin)),
-					_tips_handle_event,
-					NULL,
-					NULL,
-					tips.widget->wl,
-					NULL);
 
       gettimeofday(&tv, NULL);
       /* We round to ms instead of us (tips_timeout is ms anyway) to get out most of an int */
@@ -297,9 +272,13 @@ void xitk_tips_deinit(void) {
     pthread_cond_signal(&tips.timer_cond);
   else
     pthread_cond_signal(&tips.new_cond);
-
   pthread_mutex_unlock(&tips.mutex);
 
+  /* Wait until tips are hidden to avoid a race condition causing a segfault when */
+  /* the parent widget is already destroyed before it's referenced during hiding. */
+  while(tips.visible)
+    xitk_usec_sleep(5000);
+
   pthread_join(tips.thread, NULL);
 
   pthread_mutex_destroy(&tips.mutex);
@@ -325,6 +304,11 @@ void xitk_tips_hide_tips(void) {
       pthread_cond_signal(&tips.timer_cond);
   }
   pthread_mutex_unlock(&tips.mutex);
+
+  /* Wait until tips are hidden to avoid a race condition causing a segfault when */
+  /* the parent widget is already destroyed before it's referenced during hiding. */
+  while(tips.visible)
+    xitk_usec_sleep(5000);
 }
 
 /*
