@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2007 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xitk.c,v 1.119 2004/04/13 19:55:50 f1rmb Exp $
+ * $Id: xitk.c,v 1.161 2007/04/15 16:05:22 valtri Exp $
  *
  */
 #define _XITK_C_
@@ -42,13 +42,21 @@
 #include <time.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <sys/time.h>
 
+#ifdef ENABLE_NLS
 #include <locale.h>
+#endif
+
+#ifdef HAVE_EXECINFO_H
+#include <execinfo.h>
+#endif
 
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 #include <X11/keysym.h>
 
+#include "utils.h"
 #include "_xitk.h"
 
 extern char **environ;
@@ -79,6 +87,10 @@ static int ml = 0;
 #define MUTUNLOCK() { pthread_mutex_unlock(&gXitk->mutex); }
 #endif
 
+#define FXLOCK(_fx) { pthread_mutex_lock(&_fx->mutex); }
+#define FXUNLOCK(_fx) { pthread_mutex_unlock(&_fx->mutex); }
+
+
 typedef struct {
   Window                      window;
   Atom                        XA_XITK;
@@ -107,10 +119,16 @@ typedef struct {
   xitk_register_key_t         key;
   xitk_dnd_t                 *xdnd;
   void                       *user_data;
+
+  pthread_mutex_t             mutex;
+  int                         destroy;
 } __gfx_t;
 
 typedef struct {
   Display                    *display;
+  XColor                      black;
+  int                         display_width;
+  int                         display_height;
   int                         verbosity;
   xitk_list_t                *list;
   xitk_list_t                *gfx;
@@ -129,6 +147,19 @@ typedef struct {
 
   Window                      modalw;
   xitk_widget_t              *menu;
+  
+  struct timeval              keypress;
+
+  KeyCode                     ignore_keys[2];
+
+  pthread_t                  *tips_thread;
+  unsigned long               tips_timeout;
+
+  struct {
+    Window                    window;
+    int                       focus;
+  } parent;
+  
 } __xitk_t;
 
 static __xitk_t    *gXitk;
@@ -208,17 +239,18 @@ int xitk_system(int dont_run_as_root, char *command) {
 void xitk_usec_sleep(unsigned long usec) {
 #ifdef HAVE_NANOSLEEP
   /* nanosleep is prefered on solaris, because it's mt-safe */
-  struct timespec ts;
+  struct timespec ts, remaining;
   
   ts.tv_sec =   usec / 1000000;
   ts.tv_nsec = (usec % 1000000) * 1000;
-  nanosleep(&ts, NULL);
+  while (nanosleep (&ts, &remaining) == -1 && errno == EINTR)
+    ts = remaining;
 #else
   usleep(usec);
 #endif
 }
 
-int _x_error_handler(Display *display, XErrorEvent *xevent) {
+static int _x_error_handler(Display *display, XErrorEvent *xevent) {
   char buffer[2048];
   
   XGetErrorText(display, xevent->error_code, &buffer[0], 1023);
@@ -289,12 +321,32 @@ static void xitk_signal_handler(int sig) {
     }
     break;
 
-#ifndef DEBUG
   case SIGSEGV:
-    fprintf(stderr, "xiTK received SIGSEGV signal, RIP.\n");
-    abort();
-    break;
+    {
+      fprintf(stderr, "xiTK received SIGSEGV signal, RIP.\n");
+#ifndef DEBUG
+      abort();
+#elif defined(HAVE_EXECINFO_H)
+      void    *backtrace_array[255];
+      char   **backtrace_strings;
+      int      entries, i;
+      
+      if((entries = backtrace(backtrace_array, 255))) {
+	if((backtrace_strings = backtrace_symbols(backtrace_array, entries))) {
+	  printf("Backtrace:\n");
+	  
+	  for(i = 0; i < entries; i++) {
+	    printf("  [%d] %s\n", i, backtrace_strings[i]);
+	  }
+	  
+	  free(backtrace_strings);
+	  printf("--\n");
+	}
+      }
 #endif
+    }
+    break;
+
 
   }
 }
@@ -416,7 +468,6 @@ static uint32_t xitk_check_wm(Display *display) {
     type |= WM_TYPE_LARSWM;
   }
   else if(((atom = XInternAtom(display, "KWIN_RUNNING", True)) != None) &&
-	  ((atom = XInternAtom(display, "_KDE_NET_USER_TIME", True)) != None) &&
 	  ((atom = XInternAtom(display, "_DT_SM_WINDOW_INFO", True)) != None)) {
     type |= WM_TYPE_KWIN;
   }
@@ -443,7 +494,7 @@ static uint32_t xitk_check_wm(Display *display) {
 	unsigned char   *prop_return2 = NULL;
 	Window           win_id;
 	
-	win_id = *(long *)prop_return;
+	win_id = *(unsigned long *)prop_return;
 	
 	xitk_install_x_error_handler();
 	
@@ -457,7 +508,7 @@ static uint32_t xitk_check_wm(Display *display) {
 	if((status == Success) && (type_return != None) && (type_return == XA_CARDINAL)) {
 	  
 	  if((format_return == 32) && (nitems_return == 1) 
-	     && (bytes_after_return == 0) && (win_id == *(long *)prop_return2)) {
+	     && (bytes_after_return == 0) && (win_id == *(unsigned long *)prop_return2)) {
 	    type |= WM_TYPE_GNOME_COMP;
 	  }
 	}
@@ -498,7 +549,7 @@ static uint32_t xitk_check_wm(Display *display) {
 	unsigned char   *prop_return2 = NULL;
 	Window           win_id;
 	
-	win_id = *(long *)prop_return;
+	win_id = *(unsigned long *)prop_return;
 	
 	xitk_install_x_error_handler();
 	
@@ -513,7 +564,7 @@ static uint32_t xitk_check_wm(Display *display) {
 	if((status == Success) && (type_return != None) && (type_return == XA_WINDOW) &&
 	   (format_return == 32) && (nitems_return == 1) && (bytes_after_return == 0)) {
 	  
-	  if(win_id == *(long *)prop_return) {
+	  if(win_id == *(unsigned long *)prop_return) {
 	    if (wm_name) free(wm_name);
 	    wm_name = get_wm_name(display, win_id, "_NET_WM_NAME");
 	    type |= WM_TYPE_EWMH_COMP;
@@ -566,6 +617,7 @@ static uint32_t xitk_check_wm(Display *display) {
   case WM_TYPE_METACITY: /* Untested */
   case WM_TYPE_AFTERSTEP:
   case WM_TYPE_BLACKBOX:
+  case WM_TYPE_DTWM:
     XA_WIN_LAYER = XInternAtom(display, "_WIN_LAYER", False);
     break;
   }
@@ -685,10 +737,11 @@ void xitk_set_layer_above(Window window) {
   
 
   if(gXitk->wm_type & WM_TYPE_EWMH_COMP) {
+    XEvent xev;
+
+    memset(&xev, 0, sizeof xev);
+    XLockDisplay(gXitk->display);
     if(gXitk->wm_type & WM_TYPE_KWIN) {
-      XEvent xev;
-      
-      XLockDisplay(gXitk->display);
       xev.xclient.type         = ClientMessage;
       xev.xclient.display      = gXitk->display;
       xev.xclient.window       = window;
@@ -701,13 +754,8 @@ void xitk_set_layer_above(Window window) {
       xev.xclient.data.l[4]    = 0l;
       
       XSendEvent(gXitk->display, DefaultRootWindow(gXitk->display), True, SubstructureRedirectMask, &xev);
-      XUnlockDisplay(gXitk->display);
     }
     else {
-      XEvent xev;
-      
-      XLockDisplay(gXitk->display);
-      
       xev.xclient.type         = ClientMessage;
       xev.xclient.serial       = 0;
       xev.xclient.send_event   = True;
@@ -722,8 +770,8 @@ void xitk_set_layer_above(Window window) {
       XSendEvent(gXitk->display, DefaultRootWindow(gXitk->display), 
 		 False, SubstructureRedirectMask | SubstructureNotifyMask, (XEvent*) &xev);
       
-      XUnlockDisplay(gXitk->display);
     }
+    XUnlockDisplay(gXitk->display);
     
     return;
   }
@@ -731,7 +779,6 @@ void xitk_set_layer_above(Window window) {
   switch(gXitk->wm_type & WM_TYPE_COMP_MASK) {
   case WM_TYPE_MOTIF:
   case WM_TYPE_LARSWM:
-    return;
     break;
     
   case WM_TYPE_KWIN:
@@ -750,7 +797,7 @@ void xitk_set_layer_above(Window window) {
   case WM_TYPE_METACITY: /* Untested */
   case WM_TYPE_AFTERSTEP:
   case WM_TYPE_BLACKBOX:
-  default:
+  case WM_TYPE_DTWM:
     {
       long propvalue[1];
       
@@ -765,6 +812,7 @@ void xitk_set_layer_above(Window window) {
     break;
   }
 }
+
 void xitk_set_window_layer(Window window, int layer) {
   XEvent xev;
 
@@ -773,6 +821,7 @@ void xitk_set_window_layer(Window window, int layer) {
     return;
   }
 
+  memset(&xev, 0, sizeof xev);
   xev.type                 = ClientMessage;
   xev.xclient.type         = ClientMessage;
   xev.xclient.window       = window;
@@ -795,6 +844,7 @@ static void _set_ewmh_state(Window window, Atom atom, int enable) {
   if((window == None) || (atom == None))
     return;
 
+  memset(&xev, 0, sizeof(xev));
   xev.xclient.type         = ClientMessage;
   xev.xclient.message_type = XA_NET_WM_STATE;
   xev.xclient.display      = gXitk->display;
@@ -861,22 +911,8 @@ static void _set_wm_window_type(Window window, xitk_wm_window_type_t type, int v
     }
     
     if(atom) {
-      XEvent xev;
-
-      xev.xclient.type         = ClientMessage;
-      xev.xclient.display      = gXitk->display;
-      xev.xclient.window       = window;
-      xev.xclient.message_type = XA_WIN_LAYER;
-      xev.xclient.format       = 32;
-      xev.xclient.data.l[0]    = *atom;
-      xev.xclient.data.l[1]    = 0l;
-      xev.xclient.data.l[2]    = 0l;
-      xev.xclient.data.l[3]    = 0l;
-      xev.xclient.data.l[4]    = 0l;
-  
       XLOCK(gXitk->display);
-      XSendEvent(gXitk->display, DefaultRootWindow(gXitk->display), False, SubstructureNotifyMask, &xev);
-
+      XChangeProperty(gXitk->display, window, XA_WM_WINDOW_TYPE, XA_ATOM, 32, PropModeReplace, (unsigned char *)atom, 1);
       XRaiseWindow(gXitk->display, window);
       XUNLOCK(gXitk->display);
     }
@@ -898,14 +934,14 @@ void xitk_set_wm_window_type(Window window, xitk_wm_window_type_t type) {
 xitk_widget_list_t *xitk_widget_list_new (void) {
   xitk_widget_list_t *l;
 
-  MUTLOCK();
-
   l = (xitk_widget_list_t *) xitk_xmalloc(sizeof(xitk_widget_list_t));
 
   l->widget_focused     = NULL;
   l->widget_under_mouse = NULL;
   l->widget_pressed     = NULL;
 
+  MUTLOCK();
+
   xitk_list_append_content(gXitk->list, l);
 
   MUTUNLOCK();
@@ -927,6 +963,7 @@ void xitk_change_window_for_event_handler (xitk_register_key_t key, Window windo
   
   while(fx) {
 
+    FXLOCK(fx);
     if(fx->key == key) {
 
       fx->window = window;
@@ -936,9 +973,12 @@ void xitk_change_window_for_event_handler (xitk_register_key_t key, Window windo
 	  xitk_unset_dnd_callback(fx->xdnd);
       }
       
+      FXUNLOCK(fx);
       MUTUNLOCK();
       return;
     }
+    
+    FXUNLOCK(fx);
     fx = (__gfx_t *) xitk_list_next_content(gXitk->gfx);
   }
 
@@ -978,6 +1018,7 @@ xitk_register_key_t xitk_register_event_handler(char *name, Window window,
   fx->width     = 0;
   fx->height    = 0;
   fx->user_data = user_data;
+  pthread_mutex_init(&fx->mutex, NULL);
   
   if(window != None) {
     XWindowAttributes wattr;
@@ -1055,6 +1096,65 @@ xitk_register_key_t xitk_register_event_handler(char *name, Window window,
   return fx->key;
 }
 
+static void __fx_destroy(__gfx_t *fx, int locked) {
+  __gfx_t *cur;
+  
+  if(!fx)
+    return;
+  
+  if(!locked)
+    MUTLOCK();
+  
+  cur = (__gfx_t *) xitk_list_get_current(gXitk->gfx);
+  
+  if(fx->xdnd) {
+    xitk_unset_dnd_callback(fx->xdnd);
+    free(fx->xdnd);
+  }
+  
+  fx->xevent_callback = NULL;
+  fx->newpos_callback = NULL;
+  fx->user_data       = NULL;
+
+  free(fx->name);
+  
+  if(cur) {
+    if(cur == fx)
+      xitk_list_delete_current(gXitk->gfx); 
+    else {
+      __gfx_t *cfx;
+      
+      cfx = (__gfx_t *) xitk_list_first_content(gXitk->gfx);
+      while(cfx) {
+	if(cfx == fx) {
+	  xitk_list_delete_current(gXitk->gfx); 
+	  break;
+	}
+	
+	cfx = (__gfx_t *) xitk_list_next_content(gXitk->gfx);
+      }
+      
+      /* set current back */
+      cfx = (__gfx_t *) xitk_list_first_content(gXitk->gfx);
+      while(cfx) {
+	
+	if(cfx == cur)
+	  break;
+	
+	cfx = (__gfx_t *) xitk_list_next_content(gXitk->gfx);
+      }
+    }
+  }
+  
+  FXUNLOCK(fx);
+  pthread_mutex_destroy(&fx->mutex);
+
+  free(fx);
+  
+  if(!locked)
+    MUTUNLOCK();
+}
+
 /*
  * Remove from the list the window/event_handler
  * specified by the key.
@@ -1069,27 +1169,24 @@ void xitk_unregister_event_handler(xitk_register_key_t *key) {
   fx = (__gfx_t *) xitk_list_first_content(gXitk->gfx);
   
   while(fx) {
-    
-    if(fx->key == *key) {
-      *key = 0; 
-      
-      if(fx->xdnd) {
-	xitk_unset_dnd_callback(fx->xdnd);
-	free(fx->xdnd);
+
+    if(pthread_mutex_trylock(&fx->mutex)) {
+      if(fx->key == *key) {
+	*key        = 0; 
+	fx->destroy = 1;
+	MUTUNLOCK();
+	return;
       }
-
-      fx->xevent_callback = NULL;
-      fx->newpos_callback = NULL;
-      fx->user_data       = NULL;
-
-      xitk_list_delete_current(gXitk->gfx); 
-
-      free(fx->name);
-      free(fx);
-      fx = NULL;
-      
-      MUTUNLOCK();
-      return;
+    }
+    else {
+      if(fx->key == *key) {
+	*key = 0; 
+	
+	__fx_destroy(fx, 1);
+	MUTUNLOCK();
+	return;
+      }
+      FXUNLOCK(fx);
     }
 
     fx = (__gfx_t *) xitk_list_next_content(gXitk->gfx);
@@ -1103,13 +1200,17 @@ void xitk_unregister_event_handler(xitk_register_key_t *key) {
  */
 int xitk_get_window_info(xitk_register_key_t key, window_info_t *winf) {
   __gfx_t  *fx;
-  
+
   MUTLOCK();
 
   fx = (__gfx_t *) xitk_list_first_content(gXitk->gfx);
     
   while(fx) {
+    int  already_locked = 0;
     
+    if(pthread_mutex_trylock(&fx->mutex))
+      already_locked = 1;
+
     if((fx->key == key) && (fx->window != None)) {
       Window c;
       
@@ -1129,27 +1230,50 @@ int xitk_get_window_info(xitk_register_key_t key, window_info_t *winf) {
       winf->height = fx->height;
       winf->width  = fx->width;
       
+      if(!already_locked)
+	FXUNLOCK(fx);
+
       MUTUNLOCK();
       return 1;
 
     }
+    
+    if(!already_locked)
+      FXUNLOCK(fx);
+    
     fx = (__gfx_t *) xitk_list_next_content(gXitk->gfx);
-
   }
 
   MUTUNLOCK();
   return 0;
 }
+
 /*
  * Here events are handled. All widget are locally
  * handled, then if a event handler callback was passed
  * at register time, it will be called.
  */
 void xitk_xevent_notify(XEvent *event) {
-  __gfx_t  *fx;
-    
-  fx = (__gfx_t *) xitk_list_first_content(gXitk->gfx);
+  __gfx_t  *fx, *fxd;
+     
+  if(!(fx = (__gfx_t *) xitk_list_first_content(gXitk->gfx)))
+    return;
 
+  if(event->type == KeyPress || event->type == KeyRelease) {
+
+    /* Filter keys that dont't need to be handled by xine  */
+    /* and could be used by our screen saver reset "ping". */
+    /* So they will not kill tips and menus.               */
+
+    size_t i;
+
+    for(i = 0; i < sizeof(gXitk->ignore_keys)/sizeof(gXitk->ignore_keys[0]); ++i)
+      if(event->xkey.keycode == gXitk->ignore_keys[i])
+	return;
+  }
+  
+  FXLOCK(fx);
+  
   if(gXitk->modalw != None) {
     while(fx && (fx->window != gXitk->modalw)) {
       
@@ -1161,6 +1285,9 @@ void xitk_xevent_notify(XEvent *event) {
   }
   
   while(fx) {
+
+    if(event->type == KeyRelease)
+      gettimeofday(&gXitk->keypress, 0);
     
     if(fx->window != None) {
       
@@ -1193,13 +1320,8 @@ void xitk_xevent_notify(XEvent *event) {
 	  len = XLookupString(&mykeyevent, kbuf, sizeof(kbuf), &mykey, NULL);
 	  XUNLOCK(gXitk->display);
 
-	  if(fx->widget_list && fx->widget_list->widget_under_mouse)
-	    xitk_tips_tips_kill(fx->widget_list->widget_under_mouse);
+	  xitk_tips_hide_tips();
 	  
-	  if(fx->widget_list && fx->widget_list->widget_focused)
-	    xitk_tips_tips_kill(fx->widget_list->widget_focused);
-
-
 	  if(fx->widget_list && fx->widget_list->widget_focused) {
 	    w = fx->widget_list->widget_focused;
 	  }
@@ -1209,8 +1331,7 @@ void xitk_xevent_notify(XEvent *event) {
 	    
 	    xitk_send_key_event(w, event);
 	    
-	    if((mykey == XK_Return) || 
-	       (mykey == XK_KP_Enter) || (mykey == XK_ISO_Enter) || (mykey == XK_ISO_Enter)) {
+	    if((mykey == XK_Return) || (mykey == XK_KP_Enter) || (mykey == XK_ISO_Enter)) {
 	      widget_event_t  event;
 	      
 	      event.type = WIDGET_EVENT_PAINT;
@@ -1219,18 +1340,21 @@ void xitk_xevent_notify(XEvent *event) {
 	      xitk_set_focus_to_next_widget(fx->widget_list, 0);
 	    }
 
+	    FXUNLOCK(fx);
 	    return;
 	  }
 	  
-	  /* set focus to next widget */
+	  /* close menu */
 	  if(mykey == XK_Escape) {
 	    if(w && ((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MENU)) {
 	      xitk_widget_t *m = xitk_menu_get_menu(w);
 	      
 	      xitk_menu_destroy(m);
+	      FXUNLOCK(fx);
 	      return;
 	    }
 	  }
+	  /* set focus to next widget */
 	  else if((mykey == XK_Tab) || (mykey == XK_KP_Tab) || (mykey == XK_ISO_Left_Tab)) {
 	    if(fx->widget_list) {
 	      handled = 1;
@@ -1239,10 +1363,12 @@ void xitk_xevent_notify(XEvent *event) {
 	  }
 	  /* simulate click event on space/return/enter key event */
 	  else if((mykey == XK_space) || (mykey == XK_Return) || 
-		  (mykey == XK_KP_Enter) || (mykey == XK_ISO_Enter) || (mykey == XK_ISO_Enter)) {
+		  (mykey == XK_KP_Enter) || (mykey == XK_ISO_Enter)) {
 	    if(w && (((w->type & WIDGET_CLICKABLE) && (w->type & WIDGET_KEYABLE))
 		     && w->visible && w->enable)) {
 
+	    __menu_sim_click:
+
 	      if(w && (((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_BUTTON) ||
 		       ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON) ||
 		       ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX))) {
@@ -1268,7 +1394,7 @@ void xitk_xevent_notify(XEvent *event) {
 	      }
 	    }
 	  }
-	  /* move sliders */
+	  /* move sliders, handle menu items */
 	  else if(((mykey == XK_Left) || (mykey == XK_Right) 
 		   || (mykey == XK_Up) || (mykey == XK_Down)
 		   || (mykey == XK_Prior) || (mykey == XK_Next)) 
@@ -1278,6 +1404,7 @@ void xitk_xevent_notify(XEvent *event) {
 	      xitk_widget_t *b = xitk_browser_get_browser(w);
 	      
 	      if(b) {
+		handled = 1;
 		if(mykey == XK_Up)
 		  xitk_browser_step_down(b, NULL);
 		else if(mykey == XK_Down)
@@ -1290,27 +1417,41 @@ void xitk_xevent_notify(XEvent *event) {
 		  xitk_browser_page_down(b, NULL);
 		else if(mykey == XK_Next)
 		  xitk_browser_page_up(b, NULL);
-
 	      }
 	    }
 	    else if(w && (((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER) 
 			  && (w->type & WIDGET_KEYABLE))) {
-	      if((mykey == XK_Left) || (mykey == XK_Down)) {
-		handled = 1;
+	      handled = 1;
+	      if((mykey == XK_Left) || (mykey == XK_Down) || (mykey == XK_Next)) {
 		xitk_slider_make_backstep(w);
-		xitk_slider_callback_exec(w);
 	      }
 	      else {
-		handled = 1;
 		xitk_slider_make_step(w);
-		xitk_slider_callback_exec(w);
+	      }
+	      xitk_slider_callback_exec(w);
+	    }
+	    else if(w && ((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MENU)) {
+	      handled = 1;
+	      if(mykey == XK_Left) {
+		/* close menu branch */
+		xitk_menu_destroy_branch(w);
+	      }
+	      else if(mykey == XK_Right) {
+		/* simulate click event: trigger action of menu item (e.g. open submenu) */
+		goto __menu_sim_click; // (goto is bad but simple ...)
+	      }
+	      else {
+		/* next/previous menu item */
+		if(fx->widget_list)
+		  xitk_set_focus_to_next_widget(fx->widget_list,
+						((mykey == XK_Up) || (mykey == XK_Prior)));
 	      }
 	    }
-
 	  }
 	  else if((mykey == XK_0) || (mykey == XK_1) || (mykey == XK_2) || (mykey == XK_3) || 
 		  (mykey == XK_4) || (mykey == XK_5) || (mykey == XK_6) || (mykey == XK_7) || 
 		  (mykey == XK_8) || (mykey == XK_9) || 
+		  (mykey == XK_underscore) ||
 		  (mykey == XK_a) || (mykey == XK_A) || (mykey == XK_b) || (mykey == XK_B) ||
 		  (mykey == XK_c) || (mykey == XK_C) || (mykey == XK_d) || (mykey == XK_D) ||
 		  (mykey == XK_e) || (mykey == XK_E) || (mykey == XK_f) || (mykey == XK_F) ||
@@ -1329,8 +1470,10 @@ void xitk_xevent_notify(XEvent *event) {
 	    if(w && ((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER)) {
 	      xitk_widget_t *b = xitk_browser_get_browser(w);
 	      
-	      if(b)
+	      if(b) {
+		handled = 1;
 		xitk_browser_warp_jump(b, kbuf, modifier);
+	      }
 
 	    }
 	  }
@@ -1353,8 +1496,7 @@ void xitk_xevent_notify(XEvent *event) {
 	    }
 	    
 	    if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT) && 
-	       ((mykey == XK_Return) || (mykey == XK_KP_Enter) 
-		|| (mykey == XK_ISO_Enter) || (mykey == XK_ISO_Enter))) {
+	       ((mykey == XK_Return) || (mykey == XK_KP_Enter) || (mykey == XK_ISO_Enter))) {
 	      widget_event_t  event;
 	      
 	      event.type = WIDGET_EVENT_PAINT;
@@ -1364,20 +1506,26 @@ void xitk_xevent_notify(XEvent *event) {
 	      
 	    }
 	  }
+	  
+	  if(fx->destroy)
+	    __fx_destroy(fx, 0);
+	  else
+	    FXUNLOCK(fx);
+
 	  return;
 	}
 	break;
 
 	case Expose:
 	  if (fx->widget_list) {
-	    XEvent xev;
-
-	    xitk_paint_widget_list (fx->widget_list);
 
 	    XLOCK(gXitk->display);
 	    while(XCheckTypedWindowEvent(gXitk->display, fx->window, 
-					 Expose, &xev) == True);
+					 Expose, event) == True);
 	    XUNLOCK(gXitk->display);
+
+	    if(event->xexpose.count == 0)
+	      xitk_paint_widget_list(fx->widget_list);
 	  }
 	  break;
 	  
@@ -1428,24 +1576,36 @@ void xitk_xevent_notify(XEvent *event) {
 	    }
 	  }
 	}
-	break;
-	
+	  break;
+	  
+	case LeaveNotify:
+	  if(!(fx->widget_list && fx->widget_list->widget_pressed &&
+	       (fx->widget_list->widget_pressed->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER))
+	    event->xcrossing.x = event->xcrossing.y = -1; /* Simulate moving out of any widget */
+	    /* but leave the actual coords for an active slider, otherwise the slider may jump */
+	  /* fall through */
+	case EnterNotify:
+	  if(fx->widget_list)
+	    if(event->xcrossing.mode == NotifyNormal) /* Ptr. moved rel. to win., not (un)grab */
+	      xitk_motion_notify_widget_list (fx->widget_list,
+					      event->xcrossing.x,
+					      event->xcrossing.y, event->xcrossing.state);
+	  break;
+	  
 	case ButtonPress: {
 	  XWindowAttributes   wattr;
 	  Status              status;
-	  Window              focused_window;
-	  int                 revert;
+
+	  xitk_tips_hide_tips();
 	  
 	  XLOCK(gXitk->display);
 	  status = XGetWindowAttributes(gXitk->display, fx->window, &wattr);
-	  XGetInputFocus(gXitk->display, &focused_window, &revert);
-	  
 	  /* 
-	   * Give focus(and raise) to a window after a click, and 
-	   * only if window isn't the current focused one.
+	   * Give focus (and raise) to window after click
+	   * if it's viewable (e.g. not iconified).
 	   */
 	  if((status != BadDrawable) && (status != BadWindow) 
-	     && (wattr.map_state == IsViewable)/* && (focused_window != fx->window)*/) {
+	     && (wattr.map_state == IsViewable)) {
 	    XRaiseWindow(gXitk->display, fx->window);
 	    XSetInputFocus(gXitk->display, fx->window, RevertToParent, CurrentTime);
 	  }
@@ -1458,6 +1618,9 @@ void xitk_xevent_notify(XEvent *event) {
 	      (!fx->widget_list)))  {
 
 	    xitk_set_current_menu(NULL);
+
+	    FXUNLOCK(fx);
+	    return;
 	  }
 	  
 	  if(fx->widget_list) {
@@ -1511,6 +1674,8 @@ void xitk_xevent_notify(XEvent *event) {
 	break;
 	  
 	case ButtonRelease:
+
+	  xitk_tips_hide_tips();
 	  
 	  if(fx->move.enabled) {
 
@@ -1545,19 +1710,21 @@ void xitk_xevent_notify(XEvent *event) {
 	    }
 	  }
 
-	  XLOCK(gXitk->display);
-	  err = XGetWindowAttributes(gXitk->display, fx->window, &wattr);
-	  XUNLOCK(gXitk->display);
-
-	  if(err != BadDrawable && err != BadWindow) {
-	    fx->width = wattr.width;
-	    fx->height = wattr.height;
-	  }
 	  /* Inform application about window movement. */
-	  if(fx->newpos_callback)
+	  if(fx->newpos_callback) {
+
+	    XLOCK(gXitk->display);
+	    err = XGetWindowAttributes(gXitk->display, fx->window, &wattr);
+	    XUNLOCK(gXitk->display);
+
+	    if(err != BadDrawable && err != BadWindow) {
+	      fx->width = wattr.width;
+	      fx->height = wattr.height;
+	    }
 	    fx->newpos_callback(event->xconfigure.x,
 				event->xconfigure.y,
 				fx->width, fx->height);
+	  }
 	}
 	break;
 
@@ -1573,35 +1740,45 @@ void xitk_xevent_notify(XEvent *event) {
 	}
       }
     }
-    else {
-      
-      /* Don't forward event to all of windows */
-      if(fx->xevent_callback && (fx->window != None && event->type != KeyRelease))
-	fx->xevent_callback(event, fx->user_data);
-
-    }
+    
+    fxd = fx;
+    fx = (__gfx_t *) xitk_list_next_content(gXitk->gfx);
 
+    if(fxd->destroy)
+      __fx_destroy(fxd, 0);
+    else
+      FXUNLOCK(fxd);
+    
+#warning FIXME
     if(gXitk->modalw != None) {
-      fx = (__gfx_t *) xitk_list_next_content(gXitk->gfx);
 
       /* Flush remain fxs */
       while(fx && (fx->window != gXitk->modalw)) {
+	FXLOCK(fx);
+	
 	if(fx->xevent_callback && (fx->window != None && event->type != KeyRelease))
 	  fx->xevent_callback(event, fx->user_data);
-
+	
+	fxd = fx;
 	fx = (__gfx_t *) xitk_list_next_content(gXitk->gfx);
+
+	if(fxd->destroy)
+	  __fx_destroy(fxd, 0);
+	else
+	  FXUNLOCK(fxd);
       }
       return;
     }
-    
-    fx = (__gfx_t *) xitk_list_next_content(gXitk->gfx);
+
+    if(fx)
+      FXLOCK(fx);
   }
 }
 
 /*
  * Initiatization of widget internals.
  */
-void xitk_init(Display *display, int verbosity) {
+void xitk_init(Display *display, XColor black, int verbosity) {
   char buffer[256];
   
   xitk_pid = getppid();
@@ -1612,6 +1789,9 @@ void xitk_init(Display *display, int verbosity) {
 
   gXitk = (__xitk_t *) xitk_xmalloc(sizeof(__xitk_t));
 
+  gXitk->black           = black;
+  gXitk->display_width   = DisplayWidth(display, DefaultScreen(display));
+  gXitk->display_height  = DisplayHeight(display, DefaultScreen(display));
   gXitk->verbosity       = verbosity;
   gXitk->list            = xitk_list_new();
   gXitk->gfx             = xitk_list_new();
@@ -1624,11 +1804,16 @@ void xitk_init(Display *display, int verbosity) {
   xitk_x_error           = 0;
   gXitk->x_error_handler = NULL;
   gXitk->modalw          = None;
+  gXitk->ignore_keys[0]  = XKeysymToKeycode(display, XK_Shift_L);
+  gXitk->ignore_keys[1]  = XKeysymToKeycode(display, XK_Control_L);
+  gXitk->tips_timeout    = TIPS_TIMEOUT;
+  XGetInputFocus(display, &(gXitk->parent.window), &(gXitk->parent.focus));
+
+  memset(&gXitk->keypress, 0, sizeof(gXitk->keypress));
 
   pthread_mutex_init (&gXitk->mutex, NULL);
   
-  sprintf(buffer, "-[ xiTK version %d.%d.%d ", 
-	  XITK_MAJOR_VERSION, XITK_MINOR_VERSION, XITK_SUB_VERSION);
+  snprintf(buffer, sizeof(buffer), "-[ xiTK version %d.%d.%d ", XITK_MAJOR_VERSION, XITK_MINOR_VERSION, XITK_SUB_VERSION);
   
   /* Check if SHM is working */
 #ifdef HAVE_SHM
@@ -1652,7 +1837,6 @@ void xitk_init(Display *display, int verbosity) {
 	shminfo.shmaddr = xim->data =  shmat(shminfo.shmid, 0, 0);
 	if(shminfo.shmaddr == (char *) -1) {
 	  XDestroyImage(xim);
-	  shmctl(shminfo.shmid, IPC_RMID, 0);
 	  gXitk->use_xshm = 0;
 	}
 	else {
@@ -1667,41 +1851,39 @@ void xitk_init(Display *display, int verbosity) {
 	    gXitk->use_xshm = 0;
 	  else {
 	    XShmDetach(display, &shminfo);
-	    sprintf(buffer, "%s%s", buffer, "[XShm]");
+	    strlcat(buffer, "[XShm]", sizeof(buffer));
 	  }
 	  
 	  XDestroyImage(xim);
 	  shmdt(shminfo.shmaddr);
-	  shmctl(shminfo.shmid, IPC_RMID, 0);
 
 	  xitk_uninstall_x_error_handler();
 	  xitk_x_error = 0;
 	}
+	shmctl(shminfo.shmid, IPC_RMID, 0);
       }
     }
   }
 #endif
 
 #ifdef WITH_XFT
-  sprintf(buffer, "%s%s", buffer, "[XFT]");
+  strlcat(buffer, "[XFT]", sizeof(buffer));
 #elif defined(WITH_XMB)
-  sprintf(buffer, "%s%s", buffer, "[XMB]");
+  strlcat(buffer, "[XMB]", sizeof(buffer));
 #endif
   
-  sprintf(buffer, "%s%s", buffer, " ]-");
+  strlcat(buffer, " ]-", sizeof(buffer));
 
   if(verbosity)
-    printf(buffer);
+    printf("%s", buffer);
 
   gXitk->wm_type = xitk_check_wm(display);
   
   /* init font caching */
   xitk_font_cache_init();
-}
-
-/* Return True is event type isn't completion */
-static Bool is_event(Display *display, XEvent *event, XPointer arg) {
-  return True;
+  
+  xitk_cursors_init(display);
+  xitk_tips_init(display);
 }
 
 /*
@@ -1775,10 +1957,12 @@ void xitk_run(xitk_startup_callback_t cb, void *data) {
   fx = (__gfx_t *) xitk_list_first_content(gXitk->gfx);
   
   while(fx) {
-    
+    FXLOCK(fx);
+   
     if(fx->window != None && fx->widget_list) {
       XEvent xexp;
-      
+
+      memset(&xexp, 0, sizeof xexp);
       xexp.xany.type          = Expose;
       xexp.xexpose.type       = Expose;
       xexp.xexpose.send_event = True;
@@ -1792,6 +1976,8 @@ void xitk_run(xitk_startup_callback_t cb, void *data) {
       }
       XUNLOCK(gXitk->display);
     }
+
+    FXUNLOCK(fx);
     fx = (__gfx_t *) xitk_list_next_content(gXitk->gfx);
   }
 
@@ -1819,7 +2005,9 @@ void xitk_run(xitk_startup_callback_t cb, void *data) {
     select(xconnection + 1, &r, 0, 0, &tv);
     
     XLOCK(gXitk->display);
-    got_event = XCheckIfEvent(gXitk->display, &myevent, is_event, (XPointer) NULL);
+    got_event = (XPending(gXitk->display) != 0);
+    if( got_event )
+      XNextEvent(gXitk->display, &myevent);
     XUNLOCK(gXitk->display);
       
     while(got_event == True) {
@@ -1827,7 +2015,9 @@ void xitk_run(xitk_startup_callback_t cb, void *data) {
       xitk_xevent_notify(&myevent);
 
       XLOCK(gXitk->display);
-      got_event = XCheckIfEvent(gXitk->display, &myevent, is_event, (XPointer) NULL);
+      got_event = (XPending(gXitk->display) != 0);
+      if( got_event )
+        XNextEvent(gXitk->display, &myevent);
       XUNLOCK(gXitk->display);
     }
 
@@ -1848,7 +2038,12 @@ void xitk_run(xitk_startup_callback_t cb, void *data) {
  * Stop the wait xevent loop
  */
 void xitk_stop(void) {
+  xitk_tips_deinit();
+  xitk_cursors_deinit(gXitk->display);
   gXitk->running = 0;
+
+  if(gXitk->parent.window != None)
+    XSetInputFocus(gXitk->display, gXitk->parent.window, gXitk->parent.focus, CurrentTime);
 }
  
 char *xitk_get_system_font(void) {
@@ -1896,9 +2091,31 @@ int xitk_get_barstyle_feature(void) {
 int xitk_get_checkstyle_feature(void) {
   return xitk_config_get_checkstyle_feature(gXitk->config);
 }
+int xitk_get_cursors_feature(void) {
+  return xitk_config_get_cursors_feature(gXitk->config);
+}
+
 int xitk_get_menu_shortcuts_enability(void) {
   return xitk_config_get_menu_shortcuts_enability(gXitk->config);
 }
+
+int xitk_get_display_width(void) {
+  return gXitk->display_width;
+}
+int xitk_get_display_height(void) {
+  return gXitk->display_height;
+}
+XColor xitk_get_black_pixel_color(void) {
+  return gXitk->black;
+}
+
+unsigned long xitk_get_tips_timeout(void) {
+  return gXitk->tips_timeout;
+}
+void xitk_set_tips_timeout(unsigned long timeout) {
+  gXitk->tips_timeout = timeout;
+}
+
 /*
  * copy src to dest and substitute special chars. dest should have 
  * enought space to store chars.
@@ -1959,16 +2176,30 @@ void xitk_subst_special_chars(char *src, char *dest) {
 char *xitk_set_locale(void) {
   char *cur_locale = NULL;
   
+#ifdef ENABLE_NLS
   if(setlocale (LC_ALL,"") == NULL) {
     XITK_WARNING("locale not supported by C library\n");
     return NULL;
   }
   
   cur_locale = setlocale(LC_ALL, NULL);
+#endif
   
   return cur_locale;
 }
 
+
+/*
+ *
+ */
+long int xitk_get_last_keypressed_time(void) {
+  struct timeval tm, tm_diff;
+  
+  gettimeofday(&tm, NULL);
+  timersub(&tm, &gXitk->keypress, &tm_diff);
+  return tm_diff.tv_sec;
+}
+
 /*
  * Return home directory.
  */
