@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xitk.h,v 1.122 2004/04/13 19:55:50 f1rmb Exp $
+ * $Id: xitk.h,v 1.139 2006/06/29 13:33:55 dgp85 Exp $
  *
  * Unique public header file for xitk TK.
  *
@@ -232,13 +232,12 @@ typedef struct {
 /* Grouped widgets */
 #define WIDGET_GROUP_MASK           0x03FFE000
 #define WIDGET_GROUP_BROWSER        0x00002000
-#define WIDGET_GROUP_FILEBROWSER    0x00004000
-#define WIDGET_GROUP_MRLBROWSER     0x00008000
-#define WIDGET_GROUP_COMBO          0x00010000
-#define WIDGET_GROUP_TABS           0x00020000
-#define WIDGET_GROUP_INTBOX         0x00040000
-#define WIDGET_GROUP_DOUBLEBOX      0x00080000
-#define WIDGET_GROUP_MENU           0x00100000
+#define WIDGET_GROUP_MRLBROWSER     0x00004000
+#define WIDGET_GROUP_COMBO          0x00008000
+#define WIDGET_GROUP_TABS           0x00010000
+#define WIDGET_GROUP_INTBOX         0x00020000
+#define WIDGET_GROUP_DOUBLEBOX      0x00040000
+#define WIDGET_GROUP_MENU           0x00080000
 
 /* Real widgets. */
 #define WIDGET_TYPE_MASK            0x00001FFF
@@ -333,6 +332,88 @@ typedef enum {
 void xitk_set_wm_window_type(Window window, xitk_wm_window_type_t type);
 void xitk_unset_wm_window_type(Window window, xitk_wm_window_type_t type);
 
+typedef enum {
+  xitk_cursor_invisible,
+  xitk_cursor_X_cursor,
+  xitk_cursor_arrow,
+  xitk_cursor_based_arrow_down,
+  xitk_cursor_based_arrow_up,
+  xitk_cursor_boat,
+  xitk_cursor_bogosity,
+  xitk_cursor_bottom_left_corner,
+  xitk_cursor_bottom_right_corner,
+  xitk_cursor_bottom_side,
+  xitk_cursor_bottom_tee,
+  xitk_cursor_box_spiral,
+  xitk_cursor_center_ptr,
+  xitk_cursor_circle,
+  xitk_cursor_clock,
+  xitk_cursor_coffee_mug,
+  xitk_cursor_cross,
+  xitk_cursor_cross_reverse,
+  xitk_cursor_crosshair,
+  xitk_cursor_diamond_cross,
+  xitk_cursor_dot,
+  xitk_cursor_dotbox,
+  xitk_cursor_double_arrow,
+  xitk_cursor_draft_large,
+  xitk_cursor_draft_small,
+  xitk_cursor_draped_box,
+  xitk_cursor_exchange,
+  xitk_cursor_fleur,
+  xitk_cursor_gobbler,
+  xitk_cursor_gumby,
+  xitk_cursor_hand1,
+  xitk_cursor_hand2,
+  xitk_cursor_heart,
+  xitk_cursor_icon,
+  xitk_cursor_iron_cross,
+  xitk_cursor_left_ptr,
+  xitk_cursor_left_side,
+  xitk_cursor_left_tee,
+  xitk_cursor_leftbutton,
+  xitk_cursor_ll_angle,
+  xitk_cursor_lr_angle,
+  xitk_cursor_man,
+  xitk_cursor_middlebutton,
+  xitk_cursor_mouse,
+  xitk_cursor_pencil,
+  xitk_cursor_pirate,
+  xitk_cursor_plus,
+  xitk_cursor_question_arrow,
+  xitk_cursor_right_ptr,
+  xitk_cursor_right_side,
+  xitk_cursor_right_tee,
+  xitk_cursor_rightbutton,
+  xitk_cursor_rtl_logo,
+  xitk_cursor_sailboat,
+  xitk_cursor_sb_down_arrow,
+  xitk_cursor_sb_h_double_arrow,
+  xitk_cursor_sb_left_arrow,
+  xitk_cursor_sb_right_arrow,
+  xitk_cursor_sb_up_arrow,
+  xitk_cursor_sb_v_double_arrow,
+  xitk_cursor_shuttle,
+  xitk_cursor_sizing,
+  xitk_cursor_spider,
+  xitk_cursor_spraycan,
+  xitk_cursor_star,
+  xitk_cursor_target,
+  xitk_cursor_tcross,
+  xitk_cursor_top_left_arrow,
+  xitk_cursor_top_left_corner,
+  xitk_cursor_top_right_corner,
+  xitk_cursor_top_side,
+  xitk_cursor_top_tee,
+  xitk_cursor_trek,
+  xitk_cursor_ul_angle,
+  xitk_cursor_umbrella,
+  xitk_cursor_ur_angle,
+  xitk_cursor_watch,
+  xitk_cursor_xterm,
+  xitk_cursor_num_glyphs
+} xitk_cursors_t;
+
 /* 
  * See xitk_widget_list_[set/get]()
  */
@@ -453,58 +534,6 @@ typedef struct {
   xitk_widget_list_t               *parent_wlist;
 } xitk_browser_widget_t;
 
-typedef struct {
-  int                               magic;
-  ImlibData                        *imlibdata;
-
-  char                             *skin_element_name;
-  Window                            window_trans;
-  int                               layer_above;
-
-  int                               x;
-  int                               y;
-  char                             *window_title;
-  char                             *resource_name;
-  char                             *resource_class;
-
-  struct {
-    char                           *skin_element_name;
-  } sort_default;
-
-  struct {
-    char                           *skin_element_name;
-  } sort_reverse;
-
-  struct {
-    char                           *cur_directory;
-    char                           *skin_element_name;
-  } current_dir;
-  
-  xitk_dnd_callback_t               dndcallback;
-
-  struct {
-    char                           *caption;
-    char                           *skin_element_name;
-  } homedir;
-
-  struct {
-    char                           *caption;
-    char                           *skin_element_name;
-    xitk_string_callback_t          callback;
-  } select;
-
-  struct {
-    char                           *caption;
-    char                           *skin_element_name;
-  } dismiss;
-
-  struct {
-    xitk_simple_callback_t          callback;
-  } kill;
- 
-  xitk_browser_widget_t             browser;
-} xitk_filebrowser_widget_t;
-
 #ifdef NEED_MRLBROWSER
 
 typedef struct {
@@ -715,13 +744,18 @@ int xitk_get_window_info(xitk_register_key_t key, window_info_t *winf);
 /*
  * Initialization function, should be the first call to widget lib.
  */
-void xitk_init(Display *display, int verbosity);
+void xitk_init(Display *display, XColor black, int verbosity);
 
 /*
  *
  */
 char *xitk_set_locale(void);
 
+/*
+ *
+ */
+long int xitk_get_last_keypressed_time(void);
+
 /*
  *
  */
@@ -989,7 +1023,12 @@ void xitk_set_widget_tips_default(xitk_widget_t *w, char *str);
 /**
  *
  */
-void xitk_set_widget_tips_and_timeout(xitk_widget_t *w, char *str, unsigned int timeout);
+void xitk_set_widget_tips_and_timeout(xitk_widget_t *w, char *str, unsigned long timeout);
+
+/**
+ *
+ */
+unsigned long xitk_get_widget_tips_timeout(xitk_widget_t *w);
 
 /**
  *
@@ -1157,15 +1196,19 @@ void *xitk_list_prev_content (xitk_list_t *l);
 void xitk_list_append_content (xitk_list_t *l, void *content);
 
 /**
- * Insert content in list. NOT IMPLEMENTED
+ * Insert content in list.
  */
 void xitk_list_insert_content (xitk_list_t *l, void *content);
 
 /**
- * Remove current content in list. NOT IMPLEMENTED
+ * Remove current content in list.
  */
 void xitk_list_delete_current (xitk_list_t *l);
 
+/**
+ * Return current content
+ */
+xitk_node_t *xitk_list_get_current(xitk_list_t *l);
 
 /*
  * *** Label Buttons
@@ -1433,65 +1476,6 @@ void xitk_browser_warp_jump(xitk_widget_t *w, char *key, int modifier);
 
 xitk_widget_t *xitk_browser_get_browser(xitk_widget_t *w);
 
-/*
- * Filebrowser
- */
-/**
- *
- */
-xitk_widget_t *xitk_filebrowser_create(xitk_widget_list_t *wl,
-				       xitk_skin_config_t *skonfig, xitk_filebrowser_widget_t *fb);
-
-/**
- *
- */
-void xitk_filebrowser_change_skins(xitk_widget_t *w, xitk_skin_config_t *skonfig);
-
-/**
- *
- */
-int xitk_filebrowser_is_running(xitk_widget_t *w);
-
-/**
- *
- */
-int xitk_filebrowser_is_visible(xitk_widget_t *w);
-
-/**
- *
- */
-void xitk_filebrowser_hide(xitk_widget_t *w);
-
-/**
- *
- */
-void xitk_filebrowser_show(xitk_widget_t *w);
-
-/**
- *
- */
-void xitk_filebrowser_set_transient(xitk_widget_t *w, Window window);
-
-/**
- *
- */
-void xitk_filebrowser_destroy(xitk_widget_t *w);
-
-/**
- *
- */
-char *xitk_filebrowser_get_current_dir(xitk_widget_t *w);
-
-/**
- *
- */
-int xitk_filebrowser_get_window_info(xitk_widget_t *w, window_info_t *inf);
-
-/**
- *
- */
-Window xitk_filebrowser_get_window_id(xitk_widget_t *w);
-
 #ifdef NEED_MRLBROWSER
 /**
  *
@@ -1556,7 +1540,7 @@ void xitk_mrlbrowser_set_tips_timeout(xitk_widget_t *w, int enabled, unsigned lo
  * define ksym
  * kbuf should be large enought (256 usually)
  */
-int xitk_get_keysym_and_buf(XEvent *event, KeySym *ksym, char kbuf[]);
+int xitk_get_keysym_and_buf(XEvent *event, KeySym *ksym, char kbuf[], int kblen);
 
 /**
  * return keypressed
@@ -1639,6 +1623,11 @@ int xitk_skin_get_direction(xitk_skin_config_t *skonfig, const char *str);
  */
 int xitk_skin_get_visibility(xitk_skin_config_t *, const char *);
 
+/*
+ *
+ */
+int xitk_skin_get_printability(xitk_skin_config_t *, const char *);
+
 /*
  *
  */
@@ -1986,8 +1975,7 @@ xitk_image_t *xitk_image_create_image_from_string(ImlibData *im,
 						  int width, int align, char *str);
 xitk_image_t *xitk_image_create_image(ImlibData *im, int width, int height);
 
-xitk_pixmap_t *xitk_image_create_xitk_pixmap_with_depth(ImlibData *im, 
-							int width, int height, int depth);
+xitk_pixmap_t *xitk_image_create_xitk_pixmap_with_depth(ImlibData *im, int width, int height, int depth);
 
 xitk_pixmap_t *xitk_image_create_xitk_pixmap(ImlibData *im, int width, int height);
 
@@ -1999,7 +1987,6 @@ void xitk_image_destroy_xitk_pixmap(xitk_pixmap_t *p);
  * Free an image object.
  */
 void xitk_image_free_image(ImlibData *im, xitk_image_t **src);
-
 /**
  *
  */
@@ -2163,6 +2150,11 @@ Window xitk_window_get_window(xitk_window_t *w);
  */
 Pixmap xitk_window_get_background(xitk_window_t *w);
 
+/**
+ *
+ */
+Pixmap xitk_window_get_background_mask(xitk_window_t *w);
+
 /**
  *
  */
@@ -2173,6 +2165,11 @@ void xitk_window_apply_background(ImlibData *im, xitk_window_t *w);
  */
 int xitk_window_change_background(ImlibData *im, xitk_window_t *w, Pixmap bg, int width, int height);
 
+/**
+ *
+ */
+int xitk_window_change_background_with_image(ImlibData *im, xitk_window_t *w, xitk_image_t *img, int width, int height);
+
 /**
  *
  */
@@ -2200,6 +2197,11 @@ void xitk_set_window_title(Display *display, Window window, char *title);
  */
 void xitk_window_set_window_title(ImlibData *im, xitk_window_t *w, char *title);
 
+/*
+ *
+ */
+int xitk_is_window_iconified(Display *display, Window window);
+
 /*
  *
  */
@@ -2216,27 +2218,31 @@ Window xitk_get_desktop_root_window(Display *display, int screen, Window *client
 int xitk_is_window_size(Display *display, Window window, int width, int height);
 
 xitk_window_t *xitk_window_dialog_button_free_with_width(ImlibData *im, char *title,
-							 int window_width, int align, char *message, ...);
+							 int window_width, int align, char *message, ...)
+							 __attribute__ ((format (printf, 5, 6)));
 
 /*
  *
  */
 xitk_window_t *xitk_window_dialog_one_button_with_width(ImlibData *im, char *title, char *button_label,
 							xitk_state_callback_t cb, void *userdata, 
-							int window_width, int align, char *message, ...);
+							int window_width, int align, char *message, ...)
+							__attribute__ ((format (printf, 8, 9)));
 
 /**
  *
  */
 xitk_window_t *xitk_window_dialog_ok_with_width(ImlibData *im, char *title,
 						xitk_state_callback_t cb, void *userdata, 
-						int window_width, int align, char *message, ...);
+						int window_width, int align, char *message, ...)
+						__attribute__ ((format (printf, 7, 8)));
 
 /**
  *
  */
 xitk_window_t *xitk_window_dialog_ok(ImlibData *im, char *title,
-				     xitk_state_callback_t cb, void *userdata, int align, char *message, ...);
+				     xitk_state_callback_t cb, void *userdata, int align, char *message, ...)
+				     __attribute__ ((format (printf, 6, 7)));
 
 
 #ifdef	__GNUC__
@@ -2269,7 +2275,8 @@ xitk_window_t *xitk_window_dialog_three_buttons_with_width(ImlibData *im, char *
 							   xitk_state_callback_t cb2, 
 							   xitk_state_callback_t cb3, 
 							   void *userdata, 
-							   int window_width, int align, char *message, ...);
+							   int window_width, int align, char *message, ...)
+							   __attribute__ ((format (printf, 12, 13)));
 /**
  *
  */
@@ -2278,7 +2285,8 @@ xitk_window_t *xitk_window_dialog_yesnocancel_with_width(ImlibData *im, char *ti
 							 xitk_state_callback_t ncb, 
 							 xitk_state_callback_t ccb, 
 							 void *userdata, 
-							 int window_width, int align, char *message, ...);
+							 int window_width, int align, char *message, ...)
+							 __attribute__ ((format (printf, 9, 10)));
 
 /**
  *
@@ -2287,7 +2295,8 @@ xitk_window_t *xitk_window_dialog_yesnocancel(ImlibData *im, char *title,
 					      xitk_state_callback_t ycb, 
 					      xitk_state_callback_t ncb, 
 					      xitk_state_callback_t ccb, 
-					      void *userdata, int align, char *message, ...);
+					      void *userdata, int align, char *message, ...)
+					      __attribute__ ((format (printf, 8, 9)));
 
 #ifdef __GNUC__
 #define xitk_window_dialog_yesnocancel(im, title, ycb, ncb, ccb, userdata, align, message, args...) \
@@ -2300,12 +2309,23 @@ xitk_window_t *xitk_window_dialog_yesnocancel(ImlibData *im, char *title,
 /*
  *
  */
-xitk_window_t *xitk_window_dialog_two_buttons_with_width(ImlibData *im, char *title,
+xitk_window_t *xitk_window_dialog_checkbox_two_buttons_with_width(ImlibData *im, char *title,
 							 char *button1_label, char *button2_label,
 							 xitk_state_callback_t cb1, 
 							 xitk_state_callback_t cb2, 
+                                                         char *checkbox_label, int checkbox_state,
+                                                         xitk_state_callback_t cb3,
 							 void *userdata, 
-							 int window_width, int align, char *message, ...);
+							 int window_width, int align, char *message, ...)
+  __attribute__((__format__(__printf__, 13, 14)));
+
+#ifdef __GNUC__
+#define xitk_window_dialog_two_buttons_with_width(im, title, bl1, bl2, cb1, cb2, userdata, window_width, align, message, args...) \
+  xitk_window_dialog_checkbox_two_buttons_with_width(im, title, bl1, bl2, cb1, cb2, NULL, 0, NULL, userdata, window_width, align, message, ##args)
+#else
+#define xitk_window_dialog_two_buttons_with_width(im, title, bl1, bl2, cb1, cb2, userdata, window_width, align, ...) \
+  xitk_window_dialog_checkbox_two_buttons_with_width(im, title, bl1, bl2, cb1, cb2, NULL, 0, NULL, userdata, window_width, align, __VA_ARGS__)
+#endif
 
 /**
  *
@@ -2314,7 +2334,8 @@ xitk_window_t *xitk_window_dialog_yesno_with_width(ImlibData *im, char *title,
 						   xitk_state_callback_t ycb, 
 						   xitk_state_callback_t ncb, 
 						   void *userdata, 
-						   int window_width, int align, char *message, ...);
+						   int window_width, int align, char *message, ...)
+						   __attribute__ ((format (printf, 8, 9)));
 
 /**
  *
@@ -2322,7 +2343,8 @@ xitk_window_t *xitk_window_dialog_yesno_with_width(ImlibData *im, char *title,
 xitk_window_t *xitk_window_dialog_yesno(ImlibData *im, char *title,
 					xitk_state_callback_t ycb, 
 					xitk_state_callback_t ncb, 
-					void *userdata, int align, char *message, ...);
+					void *userdata, int align, char *message, ...)
+					__attribute__ ((format (printf, 7, 8)));
 #ifdef __GNUC__
 #define xitk_window_dialog_yesno(im, title, ycb, ncb, userdata, align, message, args...) \
   xitk_window_dialog_yesno_with_width(im, title, ycb, ncb, userdata, 400, align, message, ##args)
@@ -2331,6 +2353,8 @@ xitk_window_t *xitk_window_dialog_yesno(ImlibData *im, char *title,
   xitk_window_dialog_yesno_with_width(im, title, ycb, ncb, userdata, 400, align, __VA_ARGS__)
 #endif
 
+void xitk_window_set_parent_window(xitk_window_t *xwin, Window parent);
+
 void xitk_window_set_modal(xitk_window_t *w);
 void xitk_window_dialog_set_modal(xitk_window_t *w);
 
@@ -2367,7 +2391,12 @@ void xitk_menu_show_menu(xitk_widget_t *w);
 void xitk_menu_add_entry(xitk_widget_t *w, xitk_menu_entry_t *me);
 xitk_widget_t *xitk_menu_get_menu(xitk_widget_t *w);
 void xitk_menu_destroy_sub_branchs(xitk_widget_t *w);
+void xitk_menu_destroy_branch(xitk_widget_t *w);
 void xitk_menu_destroy(xitk_widget_t *w);
 int xitk_menu_show_sub_branchs(xitk_widget_t *w);
 
+
+void xitk_cursors_define_window_cursor(Display *display, Window window, xitk_cursors_t cursor);
+void xitk_cursors_restore_window_cursor(Display *display, Window window);
+
 #endif
